@@ -24,7 +24,6 @@
 #include <QResizeEvent>
 #include <KDebug>
 #include <KMessageBox>
-#include <KUrl>
 #include <config-kaffeine.h>
 #include <errno.h>
 #include <unistd.h>
@@ -258,7 +257,6 @@ XineMediaWidget::XineMediaWidget(QWidget *parent) : QWidget(parent), sequenceNum
 	angleCount(0), currentAngle(0), videoSize(0)
 {
 	setAttribute(Qt::WA_NativeWindow);
-	setAttribute(Qt::WA_DontCreateNativeAncestors);
 	setAttribute(Qt::WA_PaintOnScreen);
 
 	QPalette palette = QWidget::palette();
