@@ -23,6 +23,7 @@
 #include <QDBusMetaType>
 #include <KAboutData>
 #include <KApplication>
+#include "dvb/dvbmanager.h"
 #include "dvb/dvbtab.h"
 #include "playlist/playlisttab.h"
 
@@ -60,6 +61,25 @@ const QDBusArgument &operator>>(const QDBusArgument &argument, MprisVersionStruc
 	return argument;
 }
 
+QDBusArgument &operator<<(QDBusArgument &argument, const TelevisionScheduleEntryStruct &entry)
+{
+	argument.beginStructure();
+	argument << entry.key << entry.name << entry.channel << entry.begin << entry.duration <<
+		entry.repeat << entry.isRunning;
+	argument.endStructure();
+	return argument;
+}
+
+const QDBusArgument &operator>>(const QDBusArgument &argument,
+	TelevisionScheduleEntryStruct &entry)
+{
+	argument.beginStructure();
+	argument >> entry.key >> entry.name >> entry.channel >> entry.begin >> entry.duration >>
+		entry.repeat >> entry.isRunning;
+	argument.endStructure();
+	return argument;
+}
+
 MprisRootObject::MprisRootObject(QObject *parent) : QObject(parent)
 {
 	 qDBusRegisterMetaType<MprisVersionStruct>();
@@ -285,9 +305,13 @@ void MprisTrackListObject::SetRandom(bool random)
 	playlistTab->setRandom(random);
 }
 
+#if HAVE_DVB == 1
+
 DBusTelevisionObject::DBusTelevisionObject(DvbTab *dvbTab_, QObject *parent) : QObject(parent),
 	dvbTab(dvbTab_)
 {
+	qDBusRegisterMetaType<TelevisionScheduleEntryStruct>();
+	qDBusRegisterMetaType<QList<TelevisionScheduleEntryStruct> >();
 }
 
 DBusTelevisionObject::~DBusTelevisionObject()
@@ -320,3 +344,56 @@ void DBusTelevisionObject::ToggleOsd()
 {
 	dvbTab->toggleOsd();
 }
+
+QList<TelevisionScheduleEntryStruct> DBusTelevisionObject::ListProgramSchedule()
+{
+	QList<TelevisionScheduleEntryStruct> entries;
+	DvbRecordingModel *recordingModel = dvbTab->getManager()->getRecordingModel();
+
+	foreach (const DvbSharedRecording &recording, recordingModel->getRecordings()) {
+		TelevisionScheduleEntryStruct entry;
+		entry.key = recording->sqlKey;
+		entry.name = recording->name;
+		entry.channel = recording->channel->name;
+		entry.begin = (recording->begin.toString(Qt::ISODate) + 'Z');
+		entry.duration = recording->duration.toString(Qt::ISODate);
+		entry.repeat = recording->repeat;
+		entry.isRunning = (recording->status != DvbRecording::Inactive);
+		entries.append(entry);
+	}
+
+	return entries;
+}
+
+quint32 DBusTelevisionObject::ScheduleProgram(const QString &name, const QString &channel,
+	const QString &begin, const QString &duration, int repeat)
+{
+	DvbRecording recording;
+	recording.name = name;
+	recording.channel = dvbTab->getManager()->getChannelModel()->findChannelByName(channel);
+	recording.begin = QDateTime::fromString(begin, Qt::ISODate).toUTC();
+	recording.duration = QTime::fromString(duration, Qt::ISODate);
+	recording.repeat = (repeat & ((1 << 7) - 1));
+	DvbSharedRecording newRecording =
+		dvbTab->getManager()->getRecordingModel()->addRecording(recording);
+
+	if (newRecording.isValid()) {
+		return newRecording->sqlKey;
+	}
+
+	return 0;
+}
+
+void DBusTelevisionObject::RemoveProgram(quint32 key)
+{
+	DvbRecordingModel *recordingModel = dvbTab->getManager()->getRecordingModel();
+	SqlKey sqlKey;
+	sqlKey.sqlKey = key;
+	DvbSharedRecording recording = recordingModel->getRecordings().value(sqlKey);
+
+	if (recording.isValid()) {
+		recordingModel->removeRecording(recording);
+	}
+}
+
+#endif /* HAVE_DVB == 1 */
