@@ -22,6 +22,7 @@
 #define DBUSOBJECTS_H
 
 #include <QVariantMap>
+#include <config-kaffeine.h>
 
 class DvbTab;
 class MainWindow;
@@ -30,6 +31,7 @@ class PlaylistTab;
 
 struct MprisStatusStruct;
 struct MprisVersionStruct;
+struct TelevisionScheduleEntryStruct;
 
 class MprisRootObject : public QObject
 {
@@ -128,6 +130,8 @@ private:
 	PlaylistTab *playlistTab;
 };
 
+#if HAVE_DVB == 1
+
 class DBusTelevisionObject : public QObject
 {
 	Q_OBJECT
@@ -142,11 +146,17 @@ public slots:
 	void PlayLastChannel();
 	void ToggleInstantRecord();
 	void ToggleOsd();
+	QList<TelevisionScheduleEntryStruct> ListProgramSchedule();
+	quint32 ScheduleProgram(const QString &name, const QString &channel, const QString &begin,
+		const QString &duration, int repeat);
+	void RemoveProgram(quint32 key);
 
 private:
 	DvbTab *dvbTab;
 };
 
+#endif /* HAVE_DVB == 1 */
+
 struct MprisStatusStruct
 {
 	int state;
@@ -165,4 +175,18 @@ struct MprisVersionStruct
 
 Q_DECLARE_METATYPE(MprisVersionStruct)
 
+struct TelevisionScheduleEntryStruct
+{
+	quint32 key;
+	QString name;
+	QString channel;
+	QString begin;
+	QString duration;
+	int repeat;
+	bool isRunning;
+};
+
+Q_DECLARE_METATYPE(TelevisionScheduleEntryStruct)
+Q_DECLARE_METATYPE(QList<TelevisionScheduleEntryStruct>)
+
 #endif /* DBUSOBJECTS_H */
