@@ -43,7 +43,7 @@
 DvbConfigDialog::DvbConfigDialog(DvbManager *manager_, QWidget *parent) : KDialog(parent),
 	manager(manager_)
 {
-	setCaption(i18n("Configure Television"));
+	setCaption(i18nc("@title:window", "Configure Television"));
 
 	tabWidget = new KTabWidget(this);
 	setMainWidget(tabWidget);
@@ -140,7 +140,8 @@ DvbConfigDialog::DvbConfigDialog(DvbManager *manager_, QWidget *parent) : KDialo
 
 	longitudeEdit = new KLineEdit(widget);
 	longitudeEdit->setText(QString::number(manager->getLongitude(), 'g', 10));
-	connect(longitudeEdit, SIGNAL(textChanged(QString)), this, SLOT(longitudeChanged(QString)));
+	connect(longitudeEdit, SIGNAL(textChanged(QString)),
+		this, SLOT(longitudeChanged(QString)));
 	gridLayout->addWidget(longitudeEdit, 1, 2);
 
 	longitudeValidLabel = new QLabel(widget);
@@ -378,7 +379,7 @@ DvbScanFileDownloadDialog::DvbScanFileDownloadDialog(DvbManager *manager_, QWidg
 	KDialog(parent), manager(manager_)
 {
 	setButtons(KDialog::Cancel);
-	setCaption("Update Scan Data");
+	setCaption(i18n("Update Scan Data"));
 
 	QWidget *widget = new QWidget(this);
 	setMainWidget(widget);
@@ -461,7 +462,8 @@ DvbConfigPage::DvbConfigPage(QWidget *parent, DvbManager *manager,
 		connect(pushButton, SIGNAL(clicked()), this, SIGNAL(resetConfig()));
 		horizontalLayout->addWidget(pushButton);
 	} else {
-		QPushButton *pushButton = new QPushButton(KIcon("edit-delete"), i18n("Remove"), this);
+		QPushButton *pushButton =
+			new QPushButton(KIcon("edit-delete"), i18nc("@action", "Remove"), this);
 		connect(pushButton, SIGNAL(clicked()), this, SLOT(removeConfig()));
 		horizontalLayout->addWidget(pushButton);
 	}
@@ -478,7 +480,7 @@ DvbConfigPage::DvbConfigPage(QWidget *parent, DvbManager *manager,
 		return;
 	}
 
-	DvbBackendDevice::TransmissionTypes transmissionTypes =
+	DvbDevice::TransmissionTypes transmissionTypes =
 		deviceConfig->device->getTransmissionTypes();
 
 	DvbConfig dvbCConfig;
@@ -503,7 +505,7 @@ DvbConfigPage::DvbConfigPage(QWidget *parent, DvbManager *manager,
 		}
 	}
 
-	if ((transmissionTypes & DvbBackendDevice::DvbC) != 0) {
+	if ((transmissionTypes & DvbDevice::DvbC) != 0) {
 		addHSeparator(i18n("DVB-C"));
 
 		if (dvbCConfig.constData() == NULL) {
@@ -516,8 +518,8 @@ DvbConfigPage::DvbConfigPage(QWidget *parent, DvbManager *manager,
 		configs.append(dvbCConfig);
 	}
 
-	if ((transmissionTypes & (DvbBackendDevice::DvbS | DvbBackendDevice::DvbS2)) != 0) {
-		bool dvbS2 = ((transmissionTypes & DvbBackendDevice::DvbS2) != 0);
+	if ((transmissionTypes & (DvbDevice::DvbS | DvbDevice::DvbS2)) != 0) {
+		bool dvbS2 = ((transmissionTypes & DvbDevice::DvbS2) != 0);
 
 		if (dvbS2) {
 			addHSeparator(i18n("DVB-S2"));
@@ -528,7 +530,7 @@ DvbConfigPage::DvbConfigPage(QWidget *parent, DvbManager *manager,
 		dvbSObject = new DvbSConfigObject(this, boxLayout, manager, dvbSConfigs, dvbS2);
 	}
 
-	if ((transmissionTypes & DvbBackendDevice::DvbT) != 0) {
+	if ((transmissionTypes & DvbDevice::DvbT) != 0) {
 		addHSeparator(i18n("DVB-T"));
 
 		if (dvbTConfig.constData() == NULL) {
@@ -541,7 +543,7 @@ DvbConfigPage::DvbConfigPage(QWidget *parent, DvbManager *manager,
 		configs.append(dvbTConfig);
 	}
 
-	if ((transmissionTypes & DvbBackendDevice::Atsc) != 0) {
+	if ((transmissionTypes & DvbDevice::Atsc) != 0) {
 		addHSeparator(i18n("ATSC"));
 
 		if (atscConfig.constData() == NULL) {
@@ -809,7 +811,8 @@ DvbSConfigObject::DvbSConfigObject(QWidget *parent_, QBoxLayout *boxLayout, DvbM
 		layout->addWidget(comboBox, lnbNumber + 2, 1);
 
 		diseqcConfigs.append(DvbConfig(config));
-		lnbConfigs.append(new DvbSLnbConfigObject(timeoutBox, comboBox, pushButton, config));
+		lnbConfigs.append(new DvbSLnbConfigObject(timeoutBox, comboBox, pushButton,
+			config));
 	}
 
 	// USALS rotor / Positions rotor
