@@ -1,7 +1,7 @@
 /*
  * dvbdevice.h
  *
- * Copyright (C) 2007-2009 Christoph Pfister <christophpfister@gmail.com>
+ * Copyright (C) 2007-2010 Christoph Pfister <christophpfister@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,25 +21,39 @@
 #ifndef DVBDEVICE_H
 #define DVBDEVICE_H
 
+#include <QExplicitlySharedDataPointer>
 #include <QMap>
+#include <QMutex>
 #include <QTimer>
 #include "dvbbackenddevice.h"
-#include "dvbchannel.h"
+#include "dvbtransponder.h"
 
 class DvbConfigBase;
-class DvbFilterData;
+class DvbDataDumper;
+class DvbDeviceDataBuffer;
 class DvbFilterInternal;
+class DvbSectionFilterInternal;
 
-class DvbPidFilter
+class DvbDummyPidFilter : public DvbPidFilter
 {
 public:
-	DvbPidFilter() { }
-	virtual ~DvbPidFilter() { }
+	DvbDummyPidFilter() { }
+	~DvbDummyPidFilter() { }
 
-	virtual void processData(const char data[188]) = 0;
+	void processData(const char [188]) { }
 };
 
-class DvbDevice : public QObject, private DvbAbstractDeviceBuffer
+class DvbDummySectionFilter : public DvbSectionFilter
+{
+public:
+	DvbDummySectionFilter() { }
+	~DvbDummySectionFilter() { }
+
+	void processSection(const char *, int) { }
+};
+
+// FIXME make DvbDevice shared ...
+class DvbDevice : public QObject, public DvbFrontendDevice
 {
 	Q_OBJECT
 public:
@@ -53,12 +67,12 @@ public:
 		// FIXME introduce a TuningFailed state
 	};
 
-	DvbDevice(DvbBackendDevice *backendDevice, QObject *parent);
+	DvbDevice(DvbBackendDevice *backend_, QObject *parent);
 	~DvbDevice();
 
 	const DvbBackendDevice *getBackendDevice() const
 	{
-		return backend.device;
+		return backend;
 	}
 
 	DeviceState getDeviceState() const
@@ -66,19 +80,21 @@ public:
 		return deviceState;
 	}
 
-	DvbBackendDevice::TransmissionTypes getTransmissionTypes() const;
+	TransmissionTypes getTransmissionTypes() const;
 	QString getDeviceId() const;
 	QString getFrontendName() const;
 
 	void tune(const DvbTransponder &transponder);
 	void autoTune(const DvbTransponder &transponder);
 	bool addPidFilter(int pid, DvbPidFilter *filter);
+	bool addSectionFilter(int pid, DvbSectionFilter *filter);
 	void removePidFilter(int pid, DvbPidFilter *filter);
-	void startDescrambling(const DvbPmtSection &pmtSection, QObject *user);
-	void stopDescrambling(int serviceId, QObject *user);
+	void removeSectionFilter(int pid, DvbSectionFilter *filter);
+	void startDescrambling(const QByteArray &pmtSectionData, QObject *user);
+	void stopDescrambling(const QByteArray &pmtSectionData, QObject *user);
 	bool isTuned() const;
-	int getSignal() const; // 0 - 100 [%]
-	int getSnr() const; // 0 - 100 [%]
+	int getSignal() const; // 0 - 100 [%] or -1 = not supported
+	int getSnr() const; // 0 - 100 [%] or -1 = not supported
 	DvbTransponder getAutoTransponder() const;
 
 	/*
@@ -101,32 +117,33 @@ private:
 	void discardBuffers();
 	void stop();
 
-	int size();
-	char *getCurrent();
-	void submitCurrent(int packets);
+	void processData(const char data[188]);
+	DvbDataBuffer getBuffer();
+	void writeBuffer(const DvbDataBuffer &dataBuffer);
 	void customEvent(QEvent *);
 
-	DvbBackendDeviceAdapter backend;
+	DvbBackendDevice *backend;
 	DeviceState deviceState;
 	QExplicitlySharedDataPointer<const DvbConfigBase> config;
 
 	int frontendTimeout;
 	QTimer frontendTimer;
 	QMap<int, DvbFilterInternal> filters;
-	DvbPidFilter *dummyFilter;
-	DvbPidFilter *dataDumper;
+	QMap<int, DvbSectionFilterInternal> sectionFilters;
+	DvbDummyPidFilter dummyPidFilter;
+	DvbDummySectionFilter dummySectionFilter;
+	DvbDataDumper *dataDumper;
 	bool cleanUpFilters;
 	QMultiMap<int, QObject *> descramblingServices;
 
 	bool isAuto;
-	DvbTTransponder *autoTTransponder;
 	DvbTransponder autoTransponder;
-	DvbBackendDevice::Capabilities capabilities;
+	Capabilities capabilities;
 
-	DvbFilterData *currentUnused;
-	DvbFilterData *currentUsed;
-	int totalBuffers;
-	QAtomicInt usedBuffers;
+	DvbDeviceDataBuffer *unusedBuffersHead;
+	DvbDeviceDataBuffer *usedBuffersHead;
+	DvbDeviceDataBuffer *usedBuffersTail;
+	QMutex dataChannelMutex;
 };
 
 #endif /* DVBDEVICE_H */
