@@ -21,7 +21,7 @@
 #include "dvbdevice_linux.h"
 
 #include <QFile>
-#include <QThread>
+#include <Solid/Device>
 #include <Solid/DeviceNotifier>
 #include <Solid/DvbInterface>
 #include <KDebug>
@@ -32,230 +32,121 @@
 #include <poll.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
-#include "dvbsi.h"
+#include "dvbtransponder.h"
 
 // krazy:excludeall=syscalls
 
-class DvbDeviceThread : private QThread
+DvbLinuxDevice::DvbLinuxDevice(QObject *parent) : QThread(parent), ready(false), frontend(NULL),
+	enabled(false), frontendFd(-1), dvrFd(-1), dvrBuffer(NULL, 0)
 {
-public:
-	DvbDeviceThread() : dvrFd(-1)
-	{
-		if (pipe(pipes) != 0) {
-			kError() << "pipe() failed";
-			pipes[0] = -1;
-			pipes[1] = -1;
-		}
-	}
-
-	~DvbDeviceThread()
-	{
-		if (isRunning()) {
-			stop();
-		}
-
-		close(pipes[0]);
-		close(pipes[1]);
-	}
-
-	void start(int dvrFd_, DvbAbstractDeviceBuffer *buffer_);
-	void discardBuffers();
-	void stop();
-
-	bool isRunning()
-	{
-		return (dvrFd != -1);
-	}
-
-private:
-	void run();
-
-	int pipes[2];
-	int dvrFd;
-	DvbAbstractDeviceBuffer *buffer;
-};
-
-void DvbDeviceThread::start(int dvrFd_, DvbAbstractDeviceBuffer *buffer_)
-{
-	Q_ASSERT(dvrFd == -1);
-	dvrFd = dvrFd_;
-	buffer = buffer_;
-	QThread::start();
-}
-
-void DvbDeviceThread::discardBuffers()
-{
-	Q_ASSERT(dvrFd != -1);
-
-	if (write(pipes[1], " ", 1) != 1) {
-		kError() << "write() failed";
-	}
-
-	wait();
-
-	char temp;
-	read(pipes[0], &temp, 1);
-
-	while (read(dvrFd, buffer->getCurrent(), buffer->size()) > 0) {
-	}
-
-	QThread::start();
-}
-
-void DvbDeviceThread::stop()
-{
-	Q_ASSERT(dvrFd != -1);
-
-	if (write(pipes[1], " ", 1) != 1) {
-		kError() << "write() failed";
-	}
-
-	wait();
-
-	char temp;
-	read(pipes[0], &temp, 1);
-	dvrFd = -1;
-}
-
-void DvbDeviceThread::run()
-{
-	pollfd pfds[2];
-	memset(&pfds, 0, sizeof(pfds));
-
-	pfds[0].fd = pipes[0];
-	pfds[0].events = POLLIN;
-
-	pfds[1].fd = dvrFd;
-	pfds[1].events = POLLIN;
-
-	int bufferSize = buffer->size();
-
-	while (true) {
-		if (poll(pfds, 2, -1) < 0) {
-			if (errno == EINTR) {
-				continue;
-			}
-
-			kWarning() << "poll() failed";
-			break;
-		}
-
-		if ((pfds[0].revents & POLLIN) != 0) {
-			break;
-		}
-
-		while (true) {
-			int size = read(dvrFd, buffer->getCurrent(), bufferSize);
-
-			if (size < 0) {
-				if (errno == EAGAIN) {
-					break;
-				}
-
-				size = read(dvrFd, buffer->getCurrent(), bufferSize);
-
-				if (size < 0) {
-					if (errno == EAGAIN) {
-						break;
-					}
-
-					kWarning() << "read() failed";
-					return;
-				}
-			}
-
-			buffer->submitCurrent(size / 188);
-
-			if (size != bufferSize) {
-				break;
-			}
-		}
-
-		msleep(1);
-	}
-}
-
-DvbLinuxDevice::DvbLinuxDevice(int adapter_, int index_) : adapter(adapter_), index(index_),
-	buffer(NULL), ready(false), frontendFd(-1), dvrFd(-1)
-{
-	thread = new DvbDeviceThread();
+	dvrPipe[0] = -1;
+	dvrPipe[1] = -1;
 }
 
 DvbLinuxDevice::~DvbLinuxDevice()
 {
-	release();
-	delete thread;
+	stopDevice();
 }
 
-bool DvbLinuxDevice::componentAdded(const Solid::Device &component)
+bool DvbLinuxDevice::isReady() const
 {
-	const Solid::DvbInterface *dvbInterface = component.as<Solid::DvbInterface>();
+	return ready;
+}
 
-	bool wasPresent = ((internalState & DevicePresent) == DevicePresent);
+void DvbLinuxDevice::startDevice(const QString &deviceId_)
+{
+	Q_ASSERT(!ready);
+	int fd = open(QFile::encodeName(frontendPath), O_RDONLY | O_NONBLOCK);
 
-	switch (dvbInterface->deviceType()) {
-	case Solid::DvbInterface::DvbCa:
-		caComponent = component;
-		caPath = dvbInterface->device();
-		internalState |= CaPresent;
-		cam.startCa(caPath);
+	if (fd < 0) {
+		kWarning() << "cannot open frontend" << frontendPath;
+		return;
+	}
+
+	dvb_frontend_info frontend_info;
+	memset(&frontend_info, 0, sizeof(frontend_info));
+
+	if (ioctl(fd, FE_GET_INFO, &frontend_info) != 0) {
+		kWarning() << "ioctl FE_GET_INFO failed for frontend" << frontendPath;
+		close(fd);
+		return;
+	}
+
+	close(fd);
+	deviceId = deviceId_;
+	frontendName = QString::fromUtf8(frontend_info.name);
+
+	switch (frontend_info.type) {
+	case FE_QAM:
+		transmissionTypes = DvbC;
 		break;
-	case Solid::DvbInterface::DvbDemux:
-		demuxComponent = component;
-		demuxPath = dvbInterface->device();
-		internalState |= DemuxPresent;
+	case FE_QPSK:
+		transmissionTypes = DvbS;
+
+		if (((frontend_info.caps & FE_CAN_2G_MODULATION) != 0) ||
+		    (strcmp(frontend_info.name, "Conexant CX24116/CX24118") == 0) ||
+		    (strcmp(frontend_info.name, "Genpix 8psk-to-USB2 DVB-S") == 0) ||
+		    (strcmp(frontend_info.name, "STB0899 Multistandard") == 0)) {
+			transmissionTypes |= DvbS2;
+		}
+
 		break;
-	case Solid::DvbInterface::DvbDvr:
-		dvrComponent = component;
-		dvrPath = dvbInterface->device();
-		internalState |= DvrPresent;
+	case FE_OFDM:
+		transmissionTypes = DvbT;
 		break;
-	case Solid::DvbInterface::DvbFrontend:
-		frontendComponent = component;
-		frontendPath = dvbInterface->device();
-		internalState |= FrontendPresent;
+	case FE_ATSC:
+		transmissionTypes = Atsc;
 		break;
 	default:
-		return false;
+		kWarning() << "unknown type" << frontend_info.type << "for frontend" <<
+			frontendPath;
+		return;
 	}
 
-	if (!wasPresent && ((internalState & DevicePresent) == DevicePresent)) {
-		if (identifyDevice()) {
-			ready = true;
-			return true;
-		}
+	capabilities = 0;
+
+	if ((frontend_info.caps & FE_CAN_QAM_AUTO) != 0) {
+		capabilities |= DvbTModulationAuto;
 	}
 
-	return false;
+	if ((frontend_info.caps & FE_CAN_FEC_AUTO) != 0) {
+		capabilities |= DvbTFecAuto;
+	}
+
+	if ((frontend_info.caps & FE_CAN_TRANSMISSION_MODE_AUTO) != 0) {
+		capabilities |= DvbTTransmissionModeAuto;
+	}
+
+	if ((frontend_info.caps & FE_CAN_GUARD_INTERVAL_AUTO) != 0) {
+		capabilities |= DvbTGuardIntervalAuto;
+	}
+
+	kDebug() << "found dvb device" << deviceId << '/' << frontendName;
+	ready = true;
 }
 
-bool DvbLinuxDevice::componentRemoved(const QString &udi)
+void DvbLinuxDevice::startCa()
 {
-	if (caComponent.udi() == udi) {
+	Q_ASSERT(ready && !caPath.isEmpty());
+
+	if (enabled) {
+		cam.startCa(caPath);
+	}
+}
+
+void DvbLinuxDevice::stopCa()
+{
+	Q_ASSERT(ready && caPath.isEmpty());
+
+	if (enabled) {
 		cam.stopCa();
-		internalState &= ~CaPresent;
-	} else if (demuxComponent.udi() == udi) {
-		internalState &= ~DemuxPresent;
-	} else if (dvrComponent.udi() == udi) {
-		internalState &= ~DvrPresent;
-	} else if (frontendComponent.udi() == udi) {
-		internalState &= ~FrontendPresent;
-	} else {
-		return false;
 	}
-
-	if (ready && ((internalState & DevicePresent) != DevicePresent)) {
-		release();
-		ready = false;
-		return true;
-	}
-
-	return false;
 }
 
-void DvbLinuxDevice::setBuffer(DvbAbstractDeviceBuffer *buffer_)
+void DvbLinuxDevice::stopDevice()
 {
-	buffer = buffer_;
+	setDeviceEnabled(false);
+	ready = false;
 }
 
 QString DvbLinuxDevice::getDeviceId()
@@ -270,71 +161,69 @@ QString DvbLinuxDevice::getFrontendName()
 	return frontendName;
 }
 
-DvbBackendDevice::TransmissionTypes DvbLinuxDevice::getTransmissionTypes()
+DvbLinuxDevice::TransmissionTypes DvbLinuxDevice::getTransmissionTypes()
 {
 	Q_ASSERT(ready);
 	return transmissionTypes;
 }
 
-DvbBackendDevice::Capabilities DvbLinuxDevice::getCapabilities()
+DvbLinuxDevice::Capabilities DvbLinuxDevice::getCapabilities()
 {
 	Q_ASSERT(ready);
 	return capabilities;
 }
 
+void DvbLinuxDevice::setFrontendDevice(DvbFrontendDevice *frontend_)
+{
+	frontend = frontend_;
+}
+
+void DvbLinuxDevice::setDeviceEnabled(bool enabled_)
+{
+	Q_ASSERT(ready);
+
+	if (enabled != enabled_) {
+		enabled = enabled_;
+
+		if (enabled) {
+			if (!caPath.isEmpty()) {
+				cam.startCa(caPath);
+			}
+		} else {
+			release();
+			cam.stopCa();
+		}
+	}
+}
+
 bool DvbLinuxDevice::acquire()
 {
-	Q_ASSERT(ready && (frontendFd < 0) && (dvrFd < 0));
-
+	Q_ASSERT(enabled && (frontendFd < 0) && (dvrFd < 0));
 	frontendFd = open(QFile::encodeName(frontendPath), O_RDWR | O_NONBLOCK);
 
 	if (frontendFd < 0) {
-		kWarning() << "couldn't open" << frontendPath;
+		kWarning() << "cannot open frontend" << frontendPath;
 		return false;
 	}
 
 	dvrFd = open(QFile::encodeName(dvrPath), O_RDONLY | O_NONBLOCK);
 
 	if (dvrFd < 0) {
-		kWarning() << "couldn't open" << dvrPath;
+		kWarning() << "cannot open dvr" << dvrPath;
 		close(frontendFd);
 		frontendFd = -1;
 		return false;
 	}
 
-	thread->start(dvrFd, buffer);
 	return true;
 }
 
-void DvbLinuxDevice::release()
-{
-	foreach (int dmxFd, dmxFds) {
-		close(dmxFd);
-	}
-
-	dmxFds.clear();
-
-	if (thread->isRunning()) {
-		thread->stop();
-	}
-
-	if (dvrFd >= 0) {
-		close(dvrFd);
-		dvrFd = -1;
-	}
-
-	if (frontendFd >= 0) {
-		close(frontendFd);
-		frontendFd = -1;
-	}
-}
-
 bool DvbLinuxDevice::setTone(SecTone tone)
 {
 	Q_ASSERT(frontendFd >= 0);
 
-	if (ioctl(frontendFd, FE_SET_TONE, (tone == ToneOff) ? SEC_TONE_OFF : SEC_TONE_ON) != 0) {
-		kWarning() << "ioctl FE_SET_TONE failed for" << frontendPath;
+	if (ioctl(frontendFd, FE_SET_TONE, (tone == ToneOn) ? SEC_TONE_ON : SEC_TONE_OFF) != 0) {
+		kWarning() << "ioctl FE_SET_TONE failed for frontend" << frontendPath;
 		return false;
 	}
 
@@ -346,8 +235,8 @@ bool DvbLinuxDevice::setVoltage(SecVoltage voltage)
 	Q_ASSERT(frontendFd >= 0);
 
 	if (ioctl(frontendFd, FE_SET_VOLTAGE,
-		  (voltage == Voltage13V) ? SEC_VOLTAGE_13 : SEC_VOLTAGE_18) != 0) {
-		kWarning() << "ioctl FE_SET_VOLTAGE failed for" << frontendPath;
+		  (voltage == Voltage18V) ? SEC_VOLTAGE_18 : SEC_VOLTAGE_13) != 0) {
+		kWarning() << "ioctl FE_SET_VOLTAGE failed for frontend" << frontendPath;
 		return false;
 	}
 
@@ -357,14 +246,14 @@ bool DvbLinuxDevice::setVoltage(SecVoltage voltage)
 bool DvbLinuxDevice::sendMessage(const char *message, int length)
 {
 	Q_ASSERT((frontendFd >= 0) && (length >= 0) && (length <= 6));
-
 	dvb_diseqc_master_cmd cmd;
 	memset(&cmd, 0, sizeof(cmd));
 	memcpy(&cmd.msg, message, length);
 	cmd.msg_len = length;
 
 	if (ioctl(frontendFd, FE_DISEQC_SEND_MASTER_CMD, &cmd) != 0) {
-		kWarning() << "ioctl FE_DISEQC_SEND_MASTER_CMD failed for" << frontendPath;
+		kWarning() << "ioctl FE_DISEQC_SEND_MASTER_CMD failed for frontend" <<
+			frontendPath;
 		return false;
 	}
 
@@ -377,7 +266,7 @@ bool DvbLinuxDevice::sendBurst(SecBurst burst)
 
 	if (ioctl(frontendFd, FE_DISEQC_SEND_BURST,
 		  (burst == BurstMiniA) ? SEC_MINI_A : SEC_MINI_B) != 0) {
-		kWarning() << "ioctl FE_DISEQC_SEND_BURST failed for" << frontendPath;
+		kWarning() << "ioctl FE_DISEQC_SEND_BURST failed for frontend" << frontendPath;
 		return false;
 	}
 
@@ -453,10 +342,10 @@ static fe_code_rate convertDvbFecRate(DvbTransponderBase::FecRate fecRate)
 	switch (fecRate) {
 	case DvbTransponderBase::FecNone: return FEC_NONE;
 	case DvbTransponderBase::Fec1_2: return FEC_1_2;
-	case DvbTransponderBase::Fec1_3: return FEC_AUTO;
-	case DvbTransponderBase::Fec1_4: return FEC_AUTO;
+	case DvbTransponderBase::Fec1_3: return FEC_AUTO; // FIXME
+	case DvbTransponderBase::Fec1_4: return FEC_AUTO; // FIXME
 	case DvbTransponderBase::Fec2_3: return FEC_2_3;
-	case DvbTransponderBase::Fec2_5: return FEC_AUTO;
+	case DvbTransponderBase::Fec2_5: return FEC_AUTO; // FIXME
 	case DvbTransponderBase::Fec3_4: return FEC_3_4;
 	case DvbTransponderBase::Fec3_5: return FEC_3_5;
 	case DvbTransponderBase::Fec4_5: return FEC_4_5;
@@ -524,14 +413,13 @@ static fe_hierarchy convertDvbHierarchy(DvbTTransponder::Hierarchy hierarchy)
 bool DvbLinuxDevice::tune(const DvbTransponder &transponder)
 {
 	Q_ASSERT(frontendFd >= 0);
-
+	stopDvr();
 	dvb_frontend_parameters params;
-	memset(&params, 0, sizeof(params));
 
-	switch (transponder->getTransmissionType()) {
+	switch (transponder.getTransmissionType()) {
 	case DvbTransponderBase::DvbC: {
-		const DvbCTransponder *dvbCTransponder = transponder->getDvbCTransponder();
-
+		const DvbCTransponder *dvbCTransponder = transponder.as<DvbCTransponder>();
+		memset(&params, 0, sizeof(params));
 		params.frequency = dvbCTransponder->frequency;
 		params.inversion = INVERSION_AUTO;
 		params.u.qam.symbol_rate = dvbCTransponder->symbolRate;
@@ -539,23 +427,19 @@ bool DvbLinuxDevice::tune(const DvbTransponder &transponder)
 		params.u.qam.modulation = convertDvbModulation(dvbCTransponder->modulation);
 		break;
 	    }
-
 	case DvbTransponderBase::DvbS: {
-		const DvbSTransponder *dvbSTransponder = transponder->getDvbSTransponder();
-
+		const DvbSTransponder *dvbSTransponder = transponder.as<DvbSTransponder>();
+		memset(&params, 0, sizeof(params));
 		params.frequency = dvbSTransponder->frequency;
 		params.inversion = INVERSION_AUTO;
 		params.u.qpsk.symbol_rate = dvbSTransponder->symbolRate;
 		params.u.qpsk.fec_inner = convertDvbFecRate(dvbSTransponder->fecRate);
 		break;
 	    }
-
 	case DvbTransponderBase::DvbS2: {
-		const DvbS2Transponder *dvbS2Transponder = transponder->getDvbS2Transponder();
-
+		const DvbS2Transponder *dvbS2Transponder = transponder.as<DvbS2Transponder>();
 		dtv_property properties[9];
 		memset(properties, 0, sizeof(properties));
-
 		properties[0].cmd = DTV_DELIVERY_SYSTEM;
 		properties[0].u.data = SYS_DVBS2;
 		properties[1].cmd = DTV_FREQUENCY;
@@ -573,25 +457,22 @@ bool DvbLinuxDevice::tune(const DvbTransponder &transponder)
 		properties[7].cmd = DTV_INNER_FEC;
 		properties[7].u.data = convertDvbFecRate(dvbS2Transponder->fecRate);
 		properties[8].cmd = DTV_TUNE;
-
 		dtv_properties propertyList;
 		memset(&propertyList, 0, sizeof(propertyList));
-
-		propertyList.num = sizeof(properties) / sizeof(properties[0]);
 		propertyList.props = properties;
+		propertyList.num = (sizeof(properties) / sizeof(properties[0]));
 
 		if (ioctl(frontendFd, FE_SET_PROPERTY, &propertyList) != 0) {
-			kWarning() << "ioctl FE_SET_PROPERTY failed for" << frontendPath;
+			kWarning() << "ioctl FE_SET_PROPERTY failed for frontend" << frontendPath;
 			return false;
 		}
 
-		thread->discardBuffers();
+		startDvr();
 		return true;
 	    }
-
 	case DvbTransponderBase::DvbT: {
-		const DvbTTransponder *dvbTTransponder = transponder->getDvbTTransponder();
-
+		const DvbTTransponder *dvbTTransponder = transponder.as<DvbTTransponder>();
+		memset(&params, 0, sizeof(params));
 		params.frequency = dvbTTransponder->frequency;
 		params.inversion = INVERSION_AUTO;
 		params.u.ofdm.bandwidth = convertDvbBandwidth(dvbTTransponder->bandwidth);
@@ -606,38 +487,49 @@ bool DvbLinuxDevice::tune(const DvbTransponder &transponder)
 			convertDvbHierarchy(dvbTTransponder->hierarchy);
 		break;
 	    }
-
 	case DvbTransponderBase::Atsc: {
-		const AtscTransponder *atscTransponder = transponder->getAtscTransponder();
-
+		const AtscTransponder *atscTransponder = transponder.as<AtscTransponder>();
+		memset(&params, 0, sizeof(params));
 		params.frequency = atscTransponder->frequency;
 		params.inversion = INVERSION_AUTO;
 		params.u.vsb.modulation = convertDvbModulation(atscTransponder->modulation);
 		break;
 	    }
-
 	default:
-		kWarning() << "unknown transmission type" << transponder->getTransmissionType();
+		kWarning() << "unknown transmission type" << transponder.getTransmissionType();
 		return false;
 	}
 
 	if (ioctl(frontendFd, FE_SET_FRONTEND, &params) != 0) {
-		kWarning() << "ioctl FE_SET_FRONTEND failed for" << frontendPath;
+		kWarning() << "ioctl FE_SET_FRONTEND failed for frontend" << frontendPath;
 		return false;
 	}
 
-	thread->discardBuffers();
+	startDvr();
 	return true;
 }
 
+bool DvbLinuxDevice::isTuned()
+{
+	Q_ASSERT(frontendFd >= 0);
+	fe_status_t status;
+	memset(&status, 0, sizeof(status));
+
+	if (ioctl(frontendFd, FE_READ_STATUS, &status) != 0) {
+		kWarning() << "ioctl FE_READ_STATUS failed for frontend" << frontendPath;
+		return false;
+	}
+
+	return ((status & FE_HAS_LOCK) != 0);
+}
+
 int DvbLinuxDevice::getSignal()
 {
 	Q_ASSERT(frontendFd >= 0);
-
 	quint16 signal = 0;
 
 	if (ioctl(frontendFd, FE_READ_SIGNAL_STRENGTH, &signal) != 0) {
-		kWarning() << "ioctl FE_READ_SIGNAL_STRENGTH failed for" << frontendPath;
+		kWarning() << "ioctl FE_READ_SIGNAL_STRENGTH failed for frontend" << frontendPath;
 		return -1;
 	}
 
@@ -652,11 +544,10 @@ int DvbLinuxDevice::getSignal()
 int DvbLinuxDevice::getSnr()
 {
 	Q_ASSERT(frontendFd >= 0);
-
 	quint16 snr = 0;
 
 	if (ioctl(frontendFd, FE_READ_SNR, &snr) != 0) {
-		kWarning() << "ioctl FE_READ_SNR failed for" << frontendPath;
+		kWarning() << "ioctl FE_READ_SNR failed for frontend" << frontendPath;
 		return -1;
 	}
 
@@ -668,40 +559,24 @@ int DvbLinuxDevice::getSnr()
 	return ((snr * 100 + 0x8001) >> 16);
 }
 
-bool DvbLinuxDevice::isTuned()
-{
-	Q_ASSERT(frontendFd >= 0);
-
-	fe_status_t status;
-	memset(&status, 0, sizeof(status));
-
-	if (ioctl(frontendFd, FE_READ_STATUS, &status) != 0) {
-		kWarning() << "ioctl FE_READ_STATUS failed for" << frontendPath;
-		return false;
-	}
-
-	return ((status & FE_HAS_LOCK) != 0);
-}
-
 bool DvbLinuxDevice::addPidFilter(int pid)
 {
 	Q_ASSERT(frontendFd >= 0);
 
 	if (dmxFds.contains(pid)) {
-		kWarning() << "filter already set up for" << pid;
+		kWarning() << "pid filter already set up for pid" << pid;
 		return false;
 	}
 
 	int dmxFd = open(QFile::encodeName(demuxPath), O_RDONLY | O_NONBLOCK);
 
 	if (dmxFd < 0) {
-		kWarning() << "couldn't open" << demuxPath;
+		kWarning() << "cannot open demux" << demuxPath;
 		return false;
 	}
 
 	dmx_pes_filter_params pes_filter;
 	memset(&pes_filter, 0, sizeof(pes_filter));
-
 	pes_filter.pid = pid;
 	pes_filter.input = DMX_IN_FRONTEND;
 	pes_filter.output = DMX_OUT_TS_TAP;
@@ -709,7 +584,7 @@ bool DvbLinuxDevice::addPidFilter(int pid)
 	pes_filter.flags = DMX_IMMEDIATE_START;
 
 	if (ioctl(dmxFd, DMX_SET_PES_FILTER, &pes_filter) != 0) {
-		kWarning() << "couldn't set up filter for" << demuxPath;
+		kWarning() << "cannot set up pid filter for demux" << demuxPath;
 		close(dmxFd);
 		return false;
 	}
@@ -723,16 +598,16 @@ void DvbLinuxDevice::removePidFilter(int pid)
 	Q_ASSERT(frontendFd >= 0);
 
 	if (!dmxFds.contains(pid)) {
-		kWarning() << "no filter set up for" << pid;
+		kWarning() << "no pid filter set up for pid" << pid;
 		return;
 	}
 
 	close(dmxFds.take(pid));
 }
 
-void DvbLinuxDevice::startDescrambling(const DvbPmtSection &pmtSection)
+void DvbLinuxDevice::startDescrambling(const QByteArray &pmtSectionData)
 {
-	cam.startDescrambling(pmtSection);
+	cam.startDescrambling(pmtSectionData);
 }
 
 void DvbLinuxDevice::stopDescrambling(int serviceId)
@@ -740,178 +615,209 @@ void DvbLinuxDevice::stopDescrambling(int serviceId)
 	cam.stopDescrambling(serviceId);
 }
 
-void DvbLinuxDevice::execute(Command command, ReturnData returnData, Data data)
+void DvbLinuxDevice::release()
 {
-	DvbBackendAdapter<DvbLinuxDevice>::execute(this, command, returnData, data);
+	stopDvr();
+
+	if (dvrBuffer.data != NULL) {
+		dvrBuffer.dataSize = 0;
+		frontend->writeBuffer(dvrBuffer);
+		dvrBuffer.data = NULL;
+	}
+
+	if (dvrPipe[0] >= 0) {
+		close(dvrPipe[0]);
+		dvrPipe[0] = -1;
+	}
+
+	if (dvrPipe[1] >= 0) {
+		close(dvrPipe[1]);
+		dvrPipe[1] = -1;
+	}
+
+	if (dvrFd >= 0) {
+		close(dvrFd);
+		dvrFd = -1;
+	}
+
+	foreach (int dmxFd, dmxFds) {
+		close(dmxFd);
+	}
+
+	dmxFds.clear();
+
+	if (frontendFd >= 0) {
+		close(frontendFd);
+		frontendFd = -1;
+	}
 }
 
-static int DvbReadSysAttr(const QString &path)
+void DvbLinuxDevice::startDvr()
 {
-	QFile file(path);
+	Q_ASSERT((dvrFd >= 0) && !isRunning());
 
-	if (!file.open(QIODevice::ReadOnly)) {
-		return -1;
-	}
-
-	QByteArray data = file.read(16);
-
-	if ((data.size() == 0) || (data.size() == 16)) {
-		return -1;
-	}
-
-	bool ok;
-	int value = QString(data).toInt(&ok, 16);
-
-	if (!ok || (value >= (1 << 16))) {
-		return -1;
-	}
-
-	return value;
-}
-
-bool DvbLinuxDevice::identifyDevice()
-{
-	int fd = open(QFile::encodeName(frontendPath), O_RDONLY | O_NONBLOCK);
-
-	if (fd < 0) {
-		kWarning() << "couldn't open" << frontendPath;
-		return false;
-	}
-
-	dvb_frontend_info frontend_info;
-	memset(&frontend_info, 0, sizeof(frontend_info));
-
-	if (ioctl(fd, FE_GET_INFO, &frontend_info) != 0) {
-		kWarning() << "ioctl FE_GET_INFO failed for" << frontendPath;
-		close(fd);
-		return false;
-	}
-
-	close(fd);
-
-	switch (frontend_info.type) {
-	case FE_QPSK:
-		transmissionTypes = DvbS;
-
-		if (((frontend_info.caps & FE_CAN_2G_MODULATION) != 0) ||
-		    (strcmp(frontend_info.name, "Genpix 8psk-to-USB2 DVB-S") == 0) ||
-		    (strcmp(frontend_info.name, "Conexant CX24116/CX24118") == 0) ||
-		    (strcmp(frontend_info.name, "STB0899 Multistandard") == 0)) {
-			transmissionTypes |= DvbS2;
+	if ((dvrPipe[0] < 0) || (dvrPipe[1] < 0)) {
+		if (pipe(dvrPipe) != 0) {
+			dvrPipe[0] = -1;
+			dvrPipe[1] = -1;
 		}
 
-		break;
-	case FE_QAM:
-		transmissionTypes = DvbC;
-		break;
-	case FE_OFDM:
-		transmissionTypes = DvbT;
-		break;
-	case FE_ATSC:
-		transmissionTypes = Atsc;
-		break;
-	default:
-		kWarning() << "unknown frontend type" << frontend_info.type << "for" << frontendPath;
-		return false;
-	}
-
-	capabilities = 0;
-
-	if ((frontend_info.caps & FE_CAN_QAM_AUTO) != 0) {
-		capabilities |= DvbTModulationAuto;
-	}
-
-	if ((frontend_info.caps & FE_CAN_FEC_AUTO) != 0) {
-		capabilities |= DvbTFecAuto;
-	}
-
-	if ((frontend_info.caps & FE_CAN_TRANSMISSION_MODE_AUTO) != 0) {
-		capabilities |= DvbTTransmissionModeAuto;
-	}
-
-	if ((frontend_info.caps & FE_CAN_GUARD_INTERVAL_AUTO) != 0) {
-		capabilities |= DvbTGuardIntervalAuto;
-	}
-
-	deviceId.clear();
-	QString path = QString("/sys/class/dvb/dvb%1.frontend%2/").arg(adapter).arg(index);
-
-	if (QFile::exists(path + "device/vendor")) {
-		// PCI device
-		int vendor = DvbReadSysAttr(path + "device/vendor");
-		int device = DvbReadSysAttr(path + "device/device");
-		int subsystem_vendor = DvbReadSysAttr(path + "device/subsystem_vendor");
-		int subsystem_device = DvbReadSysAttr(path + "device/subsystem_device");
-
-		if ((vendor >= 0) && (device >= 0) && (subsystem_vendor >= 0) &&
-		    (subsystem_device >= 0)) {
-			deviceId = 'P';
-			deviceId += QString("%1").arg(vendor, 4, 16, QChar('0'));
-			deviceId += QString("%1").arg(device, 4, 16, QChar('0'));
-			deviceId += QString("%1").arg(subsystem_vendor, 4, 16, QChar('0'));
-			deviceId += QString("%1").arg(subsystem_device, 4, 16, QChar('0'));
-		}
-	} else if (QFile::exists(path + "device/idVendor")) {
-		// USB device
-		int vendor = DvbReadSysAttr(path + "device/idVendor");
-		int product = DvbReadSysAttr(path + "device/idProduct");
-
-		if ((vendor >= 0) && (product >= 0)) {
-			deviceId = 'U';
-			deviceId += QString("%1").arg(vendor, 4, 16, QChar('0'));
-			deviceId += QString("%1").arg(product, 4, 16, QChar('0'));
+		if ((dvrPipe[0] < 0) || (dvrPipe[1] < 0)) {
+			kWarning() << "cannot create pipe";
+			return;
 		}
 	}
 
-	if (deviceId.isEmpty()) {
-		kWarning() << "couldn't identify device";
+	if (dvrBuffer.data == NULL) {
+		dvrBuffer = frontend->getBuffer();
 	}
 
-	frontendName = QString::fromAscii(frontend_info.name);
-	kDebug() << "found dvb device" << deviceId << '/' << frontendName;
+	while (true) {
+		int bufferSize = dvrBuffer.bufferSize;
+		int dataSize = read(dvrFd, dvrBuffer.data, bufferSize);
 
-	return true;
+		if (dataSize < 0) {
+			if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
+				break;
+			}
+
+			if (errno == EINTR) {
+				continue;
+			}
+
+			dataSize = read(dvrFd, dvrBuffer.data, bufferSize);
+
+			if (dataSize < 0) {
+				if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
+					break;
+				}
+
+				if (errno == EINTR) {
+					continue;
+				}
+
+				kWarning() << "cannot read from dvr" << dvrPath;
+				return;
+			}
+		}
+
+		if (dataSize != bufferSize) {
+			break;
+		}
+	}
+
+	start();
 }
 
-DvbDeviceManager::DvbDeviceManager()
+void DvbLinuxDevice::stopDvr()
+{
+	if (isRunning()) {
+		Q_ASSERT((dvrPipe[0] >= 0) && (dvrPipe[1] >= 0));
+
+		if (write(dvrPipe[1], " ", 1) != 1) {
+			kWarning() << "cannot write to pipe";
+		}
+
+		wait();
+		char data;
+
+		if (read(dvrPipe[0], &data, 1) != 1) {
+			kWarning() << "cannot read from pipe";
+		}
+	}
+}
+
+void DvbLinuxDevice::run()
+{
+	Q_ASSERT((dvrFd >= 0) && (dvrPipe[0] >= 0) && (dvrBuffer.data != NULL));
+	pollfd pollFds[2];
+	memset(&pollFds, 0, sizeof(pollFds));
+	pollFds[0].fd = dvrPipe[0];
+	pollFds[0].events = POLLIN;
+	pollFds[1].fd = dvrFd;
+	pollFds[1].events = POLLIN;
+
+	while (true) {
+		if (poll(pollFds, 2, -1) < 0) {
+			if (errno == EINTR) {
+				continue;
+			}
+
+			kWarning() << "poll failed";
+			return;
+		}
+
+		if ((pollFds[0].revents & POLLIN) != 0) {
+			return;
+		}
+
+		while (true) {
+			int bufferSize = dvrBuffer.bufferSize;
+			int dataSize = read(dvrFd, dvrBuffer.data, bufferSize);
+
+			if (dataSize < 0) {
+				if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
+					break;
+				}
+
+				if (errno == EINTR) {
+					continue;
+				}
+
+				kWarning() << "cannot read from dvr" << dvrPath;
+				dataSize = read(dvrFd, dvrBuffer.data, bufferSize);
+
+				if (dataSize < 0) {
+					if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
+						break;
+					}
+
+					if (errno == EINTR) {
+						continue;
+					}
+
+					kWarning() << "cannot read from dvr" << dvrPath;
+					return;
+				}
+			}
+
+			if (dataSize > 0) {
+				dvrBuffer.dataSize = dataSize;
+				frontend->writeBuffer(dvrBuffer);
+				dvrBuffer = frontend->getBuffer();
+			}
+
+			if (dataSize != bufferSize) {
+				break;
+			}
+		}
+
+		msleep(10);
+	}
+}
+
+DvbLinuxDeviceManager::DvbLinuxDeviceManager(QObject *parent) : QObject(parent)
 {
 	QObject *notifier = Solid::DeviceNotifier::instance();
 	connect(notifier, SIGNAL(deviceAdded(QString)), this, SLOT(componentAdded(QString)));
 	connect(notifier, SIGNAL(deviceRemoved(QString)), this, SLOT(componentRemoved(QString)));
 }
 
-DvbDeviceManager::~DvbDeviceManager()
+DvbLinuxDeviceManager::~DvbLinuxDeviceManager()
 {
-	qDeleteAll(devices);
 }
 
-void DvbDeviceManager::doColdPlug()
+void DvbLinuxDeviceManager::doColdPlug()
 {
 	foreach (const Solid::Device &device,
 		 Solid::Device::listFromType(Solid::DeviceInterface::DvbInterface)) {
-		componentAdded(device);
+		componentAdded(device.udi());
 	}
 }
 
-void DvbDeviceManager::componentAdded(const QString &udi)
+void DvbLinuxDeviceManager::componentAdded(const QString &udi)
 {
-	componentAdded(Solid::Device(udi));
-}
-
-void DvbDeviceManager::componentRemoved(const QString &udi)
-{
-	DvbLinuxDevice *device = udis.value(udi, NULL);
-
-	if (device != NULL) {
-		if (device->componentRemoved(udi)) {
-			emit deviceRemoved(device);
-		}
-	}
-}
-
-void DvbDeviceManager::componentAdded(const Solid::Device &component)
-{
-	const Solid::DvbInterface *dvbInterface = component.as<Solid::DvbInterface>();
+	const Solid::DvbInterface *dvbInterface = Solid::Device(udi).as<Solid::DvbInterface>();
 
 	if (dvbInterface == NULL) {
 		return;
@@ -919,24 +825,169 @@ void DvbDeviceManager::componentAdded(const Solid::Device &component)
 
 	int adapter = dvbInterface->deviceAdapter();
 	int index = dvbInterface->deviceIndex();
+	QString devicePath = dvbInterface->device();
 
-	if ((adapter < 0) || (index < 0)) {
-		kWarning() << "couldn't determine adapter and/or index for" << component.udi();
+	if ((adapter < 0) || (adapter > 0x7fff) || (index < 0) || (index > 0x7fff)) {
+		kWarning() << "cannot determine adapter or index for device" << udi;
 		return;
 	}
 
-	int deviceIndex = (adapter << 16) | index;
+	if (devicePath.isEmpty()) {
+		kWarning() << "cannot determine path for device" << udi;
+		return;
+	}
 
-	DvbLinuxDevice *device = devices.value(deviceIndex, NULL);
+	int deviceIndex = ((adapter << 16) | index);
+	DvbLinuxDevice *device = devices.value(deviceIndex);
 
 	if (device == NULL) {
-		device = new DvbLinuxDevice(adapter, index);
+		device = new DvbLinuxDevice(this);
 		devices.insert(deviceIndex, device);
 	}
 
-	udis.insert(component.udi(), device);
+	bool addDevice = false;
 
-	if (device->componentAdded(component)) {
-		emit deviceAdded(device);
+	switch (dvbInterface->deviceType()) {
+	case Solid::DvbInterface::DvbCa:
+		if (device->caPath.isEmpty()) {
+			device->caPath = devicePath;
+			device->caUdi = udi;
+			udis.insert(udi, device);
+
+			if (device->isReady()) {
+				device->startCa();
+			}
+		}
+
+		break;
+	case Solid::DvbInterface::DvbDemux:
+		if (device->demuxPath.isEmpty()) {
+			device->demuxPath = devicePath;
+			device->demuxUdi = udi;
+			udis.insert(udi, device);
+			addDevice = true;
+		}
+
+		break;
+	case Solid::DvbInterface::DvbDvr:
+		if (device->dvrPath.isEmpty()) {
+			device->dvrPath = devicePath;
+			device->dvrUdi = udi;
+			udis.insert(udi, device);
+			addDevice = true;
+		}
+
+		break;
+	case Solid::DvbInterface::DvbFrontend:
+		if (device->frontendPath.isEmpty()) {
+			device->frontendPath = devicePath;
+			device->frontendUdi = udi;
+			udis.insert(udi, device);
+			addDevice = true;
+		}
+
+		break;
+	default:
+		break;
+	}
+
+	if (addDevice && !device->demuxPath.isEmpty() && !device->dvrPath.isEmpty() &&
+	    !device->frontendPath.isEmpty()) {
+		QString path = QString("/sys/class/dvb/dvb%1.frontend%2/").arg(adapter).arg(index);
+		QString deviceId;
+
+		if (QFile::exists(path + "device/vendor")) {
+			// PCI device
+			int vendor = readSysAttr(path + "device/vendor");
+			int device = readSysAttr(path + "device/device");
+			int subsystem_vendor = readSysAttr(path + "device/subsystem_vendor");
+			int subsystem_device = readSysAttr(path + "device/subsystem_device");
+
+			if ((vendor >= 0) && (device >= 0) && (subsystem_vendor >= 0) &&
+			    (subsystem_device >= 0)) {
+				deviceId = 'P';
+				deviceId += QString("%1").arg(vendor, 4, 16, QChar('0'));
+				deviceId += QString("%1").arg(device, 4, 16, QChar('0'));
+				deviceId += QString("%1").arg(subsystem_vendor, 4, 16, QChar('0'));
+				deviceId += QString("%1").arg(subsystem_device, 4, 16, QChar('0'));
+			}
+		} else if (QFile::exists(path + "device/idVendor")) {
+			// USB device
+			int vendor = readSysAttr(path + "device/idVendor");
+			int product = readSysAttr(path + "device/idProduct");
+
+			if ((vendor >= 0) && (product >= 0)) {
+				deviceId = 'U';
+				deviceId += QString("%1").arg(vendor, 4, 16, QChar('0'));
+				deviceId += QString("%1").arg(product, 4, 16, QChar('0'));
+			}
+		}
+
+		device->startDevice(deviceId);
+
+		if (device->isReady()) {
+			emit deviceAdded(device);
+		}
 	}
 }
+
+void DvbLinuxDeviceManager::componentRemoved(const QString &udi)
+{
+	DvbLinuxDevice *device = udis.take(udi);
+
+	if (device == NULL) {
+		return;
+	}
+
+	bool removeDevice = false;
+
+	if (udi == device->caUdi) {
+		device->caPath.clear();
+		device->caUdi.clear();
+
+		if (device->isReady()) {
+			device->stopCa();
+		}
+	} else if (udi == device->demuxUdi) {
+		device->demuxPath.clear();
+		device->demuxUdi.clear();
+		removeDevice = true;
+	} else if (udi == device->dvrUdi) {
+		device->dvrPath.clear();
+		device->dvrUdi.clear();
+		removeDevice = true;
+	} else if (udi == device->frontendUdi) {
+		device->frontendPath.clear();
+		device->frontendUdi.clear();
+		removeDevice = true;
+	}
+
+	if (removeDevice && device->isReady()) {
+		emit deviceRemoved(device);
+		device->stopDevice();
+	}
+}
+
+int DvbLinuxDeviceManager::readSysAttr(const QString &path)
+{
+	QFile file(path);
+
+	if (!file.open(QIODevice::ReadOnly)) {
+		return -1;
+	}
+
+	QByteArray data = file.read(8);
+
+	if ((data.size() == 0) || (data.size() == 8)) {
+		return -1;
+	}
+
+	bool ok = false;
+	int value = data.simplified().toInt(&ok, 0);
+
+	if (!ok || (value < 0) || (value > 0xffff)) {
+		return -1;
+	}
+
+	return value;
+}
