@@ -21,102 +21,87 @@
 #ifndef DVBDEVICE_LINUX_H
 #define DVBDEVICE_LINUX_H
 
-#include <Solid/Device>
+#include <QThread>
 #include "dvbbackenddevice.h"
 #include "dvbcam_linux.h"
 
-class DvbDeviceThread;
-
-class DvbLinuxDevice : public DvbBackendDevice
+class DvbLinuxDevice : public QThread, public DvbBackendDevice
 {
 public:
-	DvbLinuxDevice(int adapter_, int index_);
+	explicit DvbLinuxDevice(QObject *parent);
 	~DvbLinuxDevice();
 
-	bool componentAdded(const Solid::Device &component);
-	bool componentRemoved(const QString &udi);
+	bool isReady() const;
+	void startDevice(const QString &deviceId_);
+	void startCa();
+	void stopCa();
+	void stopDevice();
 
-	void setBuffer(DvbAbstractDeviceBuffer *buffer);
+	QString caPath;
+	QString caUdi;
+	QString demuxPath;
+	QString demuxUdi;
+	QString dvrPath;
+	QString dvrUdi;
+	QString frontendPath;
+	QString frontendUdi;
+
+protected:
 	QString getDeviceId();
 	QString getFrontendName();
 	TransmissionTypes getTransmissionTypes();
 	Capabilities getCapabilities();
+	void setFrontendDevice(DvbFrontendDevice *frontend_);
+	void setDeviceEnabled(bool enabled_);
 	bool acquire();
 	bool setTone(SecTone tone);
 	bool setVoltage(SecVoltage voltage);
 	bool sendMessage(const char *message, int length);
 	bool sendBurst(SecBurst burst);
-	bool tune(const DvbTransponder &transponder);
-	int getSignal(); // 0 - 100 ; -1 = unsupported
-	int getSnr(); // 0 - 100 ; -1 = unsupported
+	bool tune(const DvbTransponder &transponder); // discards obsolete data
 	bool isTuned();
+	int getSignal(); // 0 - 100 [%] or -1 = not supported
+	int getSnr(); // 0 - 100 [%] or -1 = not supported
 	bool addPidFilter(int pid);
 	void removePidFilter(int pid);
-	void startDescrambling(const DvbPmtSection &pmtSection);
+	void startDescrambling(const QByteArray &pmtSectionData);
 	void stopDescrambling(int serviceId);
 	void release();
 
-	int adapter;
-	int index;
-
 private:
-	Q_DISABLE_COPY(DvbLinuxDevice)
+	void startDvr();
+	void stopDvr();
+	void run();
 
-	enum stateFlag {
-		CaPresent	= (1 << 0),
-		DemuxPresent	= (1 << 1),
-		DvrPresent	= (1 << 2),
-		FrontendPresent	= (1 << 3),
-
-		DevicePresent	= (DemuxPresent | DvrPresent | FrontendPresent)
-	};
-
-	Q_DECLARE_FLAGS(stateFlags, stateFlag)
-
-	void execute(Command command, ReturnData returnData, Data data);
-	bool identifyDevice();
-
-	Solid::Device caComponent;
-	Solid::Device demuxComponent;
-	Solid::Device dvrComponent;
-	Solid::Device frontendComponent;
-
-	QString caPath;
-	QString demuxPath;
-	QString dvrPath;
-	QString frontendPath;
-
-	DvbAbstractDeviceBuffer *buffer;
-	stateFlags internalState;
-	TransmissionTypes transmissionTypes;
-	Capabilities capabilities;
+	bool ready;
 	QString deviceId;
 	QString frontendName;
-	bool ready;
-	DvbDeviceThread *thread;
+	TransmissionTypes transmissionTypes;
+	Capabilities capabilities;
+	DvbFrontendDevice *frontend;
+	bool enabled;
 	int frontendFd;
-	int dvrFd;
 	QMap<int, int> dmxFds;
+
+	int dvrFd;
+	int dvrPipe[2];
+	DvbDataBuffer dvrBuffer;
+
 	DvbLinuxCam cam;
 };
 
-class DvbDeviceManager : public QObject
+class DvbLinuxDeviceManager : public QObject
 {
 	Q_OBJECT
-	Q_PROPERTY(int backendMagic READ backendMagic)
 public:
-	DvbDeviceManager();
-	~DvbDeviceManager();
-
-	int backendMagic()
-	{
-		return dvbBackendMagic;
-	}
+	explicit DvbLinuxDeviceManager(QObject *parent);
+	~DvbLinuxDeviceManager();
 
 public slots:
 	void doColdPlug();
 
 signals:
+	void requestBuiltinDeviceManager(QObject *&bultinDeviceManager);
 	void deviceAdded(DvbBackendDevice *device);
 	void deviceRemoved(DvbBackendDevice *device);
 
@@ -125,7 +110,7 @@ private slots:
 	void componentRemoved(const QString &udi);
 
 private:
-	void componentAdded(const Solid::Device &component);
+	int readSysAttr(const QString &path);
 
 	QMap<int, DvbLinuxDevice *> devices;
 	QMap<QString, DvbLinuxDevice *> udis;
