@@ -21,18 +21,28 @@
 #include "dvbliveview.h"
 #include "dvbliveview_p.h"
 
+#include <QDir>
 #include <QPainter>
+#include <KDebug>
 #include <KLocale>
 #include <KMessageBox>
 #include "../mediawidget.h"
+#include "dvbdevice.h"
 #include "dvbmanager.h"
 
 void DvbOsd::init(OsdLevel level_, const QString &channelName_,
-	const QList<DvbEpgEntry> &epgEntries_)
+	const QList<DvbSharedEpgEntry> &epgEntries)
 {
 	level = level_;
 	channelName = channelName_;
-	epgEntries = epgEntries_;
+
+	if (epgEntries.size() >= 1) {
+		firstEntry = *epgEntries.at(0);
+	}
+
+	if (epgEntries.size() >= 2) {
+		secondEntry = *epgEntries.at(1);
+	}
 }
 
 QPixmap DvbOsd::paintOsd(QRect &rect, const QFont &font, Qt::LayoutDirection)
@@ -45,28 +55,17 @@ QPixmap DvbOsd::paintOsd(QRect &rect, const QFont &font, Qt::LayoutDirection)
 	int elapsedTime = 0;
 	int totalTime = 0;
 
-	const DvbEpgEntry *firstEntry = NULL;
-	const DvbEpgEntry *secondEntry = NULL;
-
-	if (!epgEntries.isEmpty()) {
-		firstEntry = &epgEntries.at(0);
+	if (firstEntry.channel.isValid()) {
+		entryString = KGlobal::locale()->formatTime(firstEntry.begin.toLocalTime().time())
+			+ ' ' + firstEntry.title;
+		elapsedTime = firstEntry.begin.secsTo(QDateTime::currentDateTime());
+		totalTime = QTime().secsTo(firstEntry.duration);
 	}
 
-	if (epgEntries.size() >= 2) {
-		secondEntry = &epgEntries.at(1);
-	}
-
-	if (firstEntry != NULL) {
-		entryString = KGlobal::locale()->formatTime(firstEntry->begin.toLocalTime().time())
-			+ ' ' + firstEntry->title;
-		elapsedTime = firstEntry->begin.secsTo(QDateTime::currentDateTime());
-		totalTime = QTime().secsTo(firstEntry->duration);
-	}
-
-	if ((level == ShortOsd) && (secondEntry != NULL)) {
+	if ((level == ShortOsd) && secondEntry.channel.isValid()) {
 		entryString = entryString + '\n' +
-			KGlobal::locale()->formatTime(secondEntry->begin.toLocalTime().time()) +
-			' ' + secondEntry->title;
+			KGlobal::locale()->formatTime(secondEntry.begin.toLocalTime().time()) +
+			' ' + secondEntry.title;
 	}
 
 	int lineHeight = QFontMetrics(osdFont).height();
@@ -104,17 +103,17 @@ QPixmap DvbOsd::paintOsd(QRect &rect, const QFont &font, Qt::LayoutDirection)
 		if (level == LongOsd) {
 			QRect boundingRect = entryRect;
 
-			if ((firstEntry != NULL) && !firstEntry->subheading.isEmpty()) {
+			if (!firstEntry.subheading.isEmpty()) {
 				painter.drawText(entryRect.x(), boundingRect.bottom() + 1,
 					entryRect.width(), lineHeight, Qt::AlignLeft,
-					firstEntry->subheading, &boundingRect);
+					firstEntry.subheading, &boundingRect);
 			}
 
-			if ((firstEntry != NULL) && !firstEntry->details.isEmpty()) {
+			if (!firstEntry.details.isEmpty()) {
 				painter.drawText(entryRect.x(), boundingRect.bottom() + 1,
 					entryRect.width(),
 					rect.height() - boundingRect.bottom() - 1,
-					Qt::AlignLeft | Qt::TextWordWrap, firstEntry->details,
+					Qt::AlignLeft | Qt::TextWordWrap, firstEntry.details,
 					&boundingRect);
 			}
 
@@ -129,7 +128,7 @@ QPixmap DvbOsd::paintOsd(QRect &rect, const QFont &font, Qt::LayoutDirection)
 
 void DvbLiveViewInternal::processData(const char data[188])
 {
-	buffer.append(QByteArray::fromRawData(data, 188));
+	buffer.append(data, 188);
 
 	if (buffer.size() < (87 * 188)) {
 		return;
@@ -145,18 +144,17 @@ void DvbLiveViewInternal::processData(const char data[188])
 	buffer.reserve(87 * 188);
 }
 
-DvbLiveView::DvbLiveView(DvbManager *manager_) : manager(manager_), device(NULL), videoPid(-1),
-	audioPid(-1), subtitlePid(-1)
+DvbLiveView::DvbLiveView(DvbManager *manager_, QObject *parent) : QObject(parent),
+	manager(manager_), device(NULL), videoPid(-1), audioPid(-1), subtitlePid(-1)
 {
 	mediaWidget = manager->getMediaWidget();
 	osdWidget = mediaWidget->getOsdWidget();
 
 	internal = new DvbLiveViewInternal();
-	internal->eitFilter.setManager(manager);
 	internal->mediaWidget = mediaWidget;
 
-	connect(&internal->pmtFilter, SIGNAL(pmtSectionChanged(DvbPmtSection)),
-		this, SLOT(pmtSectionChanged(DvbPmtSection)));
+	connect(&internal->pmtFilter, SIGNAL(pmtSectionChanged(QByteArray)),
+		this, SLOT(pmtSectionChanged(QByteArray)));
 	connect(&patPmtTimer, SIGNAL(timeout()), this, SLOT(insertPatPmt()));
 	connect(&osdTimer, SIGNAL(timeout()), this, SLOT(osdTimeout()));
 
@@ -173,9 +171,9 @@ DvbLiveView::~DvbLiveView()
 	delete internal;
 }
 
-const DvbChannel *DvbLiveView::getChannel() const
+const DvbSharedChannel &DvbLiveView::getChannel() const
 {
-	return channel.constData();
+	return channel;
 }
 
 DvbDevice *DvbLiveView::getDevice() const
@@ -183,12 +181,12 @@ DvbDevice *DvbLiveView::getDevice() const
 	return device;
 }
 
-void DvbLiveView::playChannel(const DvbChannel *channel_)
+void DvbLiveView::playChannel(const DvbSharedChannel &channel_)
 {
 	DvbDevice *newDevice = NULL;
 
 	if ((channel.constData() != NULL) && (channel->source == channel_->source) &&
-	    (channel->transponder->corresponds(channel_->transponder))) {
+	    (channel->transponder.corresponds(channel_->transponder))) {
 		newDevice = manager->requestDevice(channel->source, channel->transponder,
 			DvbManager::Shared);
 	}
@@ -203,26 +201,31 @@ void DvbLiveView::playChannel(const DvbChannel *channel_)
 	}
 
 	if (device == NULL) {
-		channel = NULL;
+		channel = DvbSharedChannel();
 		mediaWidget->stop();
-		KMessageBox::sorry(manager->getParentWidget(),
-			i18nc("message box", "No available device found."));
-		// FIXME better error message if devices are blocked by recordings
+
+		if (manager->getRecordingModel()->hasActiveRecordings()) {
+			KMessageBox::sorry(manager->getParentWidget(),
+				i18nc("@info", "All devices are used for recordings."));
+		} else {
+			KMessageBox::information(manager->getParentWidget(),
+				i18nc("@info", "No device found."));
+		}
+
 		return;
 	}
 
 	mediaWidget->playDvb(channel->name);
 
-	internal->eitFilter.setSource(channel->source);
-	internal->pmtFilter.setProgramNumber(channel->getServiceId());
+	internal->pmtFilter.setProgramNumber(channel->serviceId);
 	startDevice();
 
-	internal->patGenerator.initPat(channel->transportStreamId, channel->getServiceId(),
+	internal->patGenerator.initPat(channel->transportStreamId, channel->serviceId,
 		channel->pmtPid);
 	videoPid = -1;
 	audioPid = channel->audioPid;
 	subtitlePid = -1;
-	pmtSectionChanged(DvbPmtSection(channel->pmtSection));
+	pmtSectionChanged(channel->pmtSectionData);
 	patPmtTimer.start(500);
 
 	internal->buffer.reserve(87 * 188);
@@ -239,7 +242,7 @@ void DvbLiveView::toggleOsd()
 	case DvbOsd::Off:
 		internal->dvbOsd.init(DvbOsd::ShortOsd,
 			QString("%1 - %2").arg(channel->number).arg(channel->name),
-			manager->getEpgModel()->getCurrentNext(channel->name));
+			manager->getEpgModel()->getCurrentNext(channel));
 		osdWidget->showObject(&internal->dvbOsd, 2500);
 		osdTimer.start(2500);
 		break;
@@ -256,11 +259,10 @@ void DvbLiveView::toggleOsd()
 	}
 }
 
-void DvbLiveView::pmtSectionChanged(const DvbPmtSection &section)
+void DvbLiveView::pmtSectionChanged(const QByteArray &pmtSectionData)
 {
-	internal->pmtSection = section;
-
-	DvbPmtParser pmtParser(section);
+	internal->pmtSectionData = pmtSectionData;
+	DvbPmtParser pmtParser(DvbPmtSection(internal->pmtSectionData));
 	videoPid = pmtParser.videoPid;
 
 	for (int i = 0;; ++i) {
@@ -293,7 +295,7 @@ void DvbLiveView::pmtSectionChanged(const DvbPmtSection &section)
 	updatePids(true);
 
 	if (channel->isScrambled) {
-		device->startDescrambling(section, this);
+		device->startDescrambling(internal->pmtSectionData, this);
 	}
 
 	if (internal->timeShiftFile.isOpen()) {
@@ -390,9 +392,16 @@ void DvbLiveView::prepareTimeShift()
 
 	if (internal->timeShiftFile.exists() ||
 	    !internal->timeShiftFile.open(QIODevice::WriteOnly)) {
-		// FIXME error message
-		mediaWidget->stop();
-		return;
+		kWarning() << "cannot open file" << internal->timeShiftFile.fileName();
+		internal->timeShiftFile.setFileName(QDir::homePath() + "/TimeShift-" +
+			QDateTime::currentDateTime().toString("yyyyMMddThhmmss") + ".m2t");
+
+		if (internal->timeShiftFile.exists() ||
+		    !internal->timeShiftFile.open(QIODevice::WriteOnly)) {
+			kWarning() << "cannot open file" << internal->timeShiftFile.fileName();
+			mediaWidget->stop();
+			return;
+		}
 	}
 
 	updatePids();
@@ -428,17 +437,17 @@ void DvbLiveView::liveStopped()
 		device = NULL;
 	}
 
-	channel = NULL;
+	channel = DvbSharedChannel();
 	pids.clear();
 	patPmtTimer.stop();
 	osdTimer.stop();
 
-	internal->pmtSection = DvbPmtSection(QByteArray());
+	internal->pmtSectionData.clear();
 	internal->patGenerator = DvbSectionGenerator();
 	internal->pmtGenerator = DvbSectionGenerator();
 	internal->buffer.clear();
 	internal->timeShiftFile.close();
-	internal->dvbOsd.init(DvbOsd::Off, QString(), QList<DvbEpgEntry>());
+	internal->dvbOsd.init(DvbOsd::Off, QString(), QList<DvbSharedEpgEntry>());
 	osdWidget->hideObject();
 }
 
@@ -448,36 +457,36 @@ void DvbLiveView::startDevice()
 		device->addPidFilter(pid, internal);
 	}
 
-	device->addPidFilter(0x12, &internal->eitFilter);
-	device->addPidFilter(channel->pmtPid, &internal->pmtFilter);
+	device->addSectionFilter(channel->pmtPid, &internal->pmtFilter);
 	connect(device, SIGNAL(stateChanged()), this, SLOT(deviceStateChanged()));
 
-	if (channel->isScrambled && internal->pmtSection.isValid()) {
-		device->startDescrambling(internal->pmtSection, this);
+	if (channel->isScrambled && !internal->pmtSectionData.isEmpty()) {
+		device->startDescrambling(internal->pmtSectionData, this);
 	}
+
+	manager->getEpgModel()->startEventFilter(device, channel);
 }
 
 void DvbLiveView::stopDevice()
 {
-	if (channel->isScrambled && internal->pmtSection.isValid()) {
-		device->stopDescrambling(internal->pmtSection.programNumber(), this);
+	manager->getEpgModel()->stopEventFilter(device, channel);
+
+	if (channel->isScrambled && !internal->pmtSectionData.isEmpty()) {
+		device->stopDescrambling(internal->pmtSectionData, this);
 	}
 
 	foreach (int pid, pids) {
 		device->removePidFilter(pid, internal);
 	}
 
-	device->removePidFilter(0x12, &internal->eitFilter);
-	device->removePidFilter(channel->pmtPid, &internal->pmtFilter);
+	device->removeSectionFilter(channel->pmtPid, &internal->pmtFilter);
 	disconnect(device, SIGNAL(stateChanged()), this, SLOT(deviceStateChanged()));
-
-	internal->eitFilter.resetFilter();
-	internal->pmtFilter.resetFilter();
 }
 
 void DvbLiveView::updatePids(bool forcePatPmtUpdate)
 {
-	DvbPmtParser pmtParser(internal->pmtSection);
+	DvbPmtSection pmtSection(internal->pmtSectionData);
+	DvbPmtParser pmtParser(pmtSection);
 	QSet<int> newPids;
 	bool updatePatPmt = forcePatPmtUpdate;
 	bool isTimeShifting = internal->timeShiftFile.isOpen();
@@ -528,7 +537,7 @@ void DvbLiveView::updatePids(bool forcePatPmtUpdate)
 	}
 
 	if (updatePatPmt) {
-		internal->pmtGenerator.initPmt(channel->pmtPid, internal->pmtSection, pids);
+		internal->pmtGenerator.initPmt(channel->pmtPid, pmtSection, pids);
 		insertPatPmt();
 	}
 }
