@@ -22,23 +22,27 @@
 #include "dvbmanager_p.h"
 
 #include <QDir>
+#include <QPluginLoader>
+#include <QWidget>
+#include <KConfigGroup>
 #include <KDebug>
 #include <KLocale>
 #include <KStandardDirs>
 #include <config-kaffeine.h>
-#include "dvbchannelui.h"
 #include "dvbconfig.h"
+#include "dvbdevice.h"
 #include "dvbdevice_linux.h"
 #include "dvbepg.h"
 #include "dvbliveview.h"
+#include "dvbsi.h"
 
 DvbManager::DvbManager(MediaWidget *mediaWidget_, QWidget *parent_) : QObject(parent_),
 	parent(parent_), mediaWidget(mediaWidget_), channelView(NULL), dvbDumpEnabled(false)
 {
-	channelModel = new DvbSqlChannelModel(this);
+	channelModel = DvbChannelModel::createSqlModel(this);
 	recordingModel = new DvbRecordingModel(this, this);
-	epgModel = new DvbEpgModel(this);
-	liveView = new DvbLiveView(this);
+	epgModel = new DvbEpgModel(this, this);
+	liveView = new DvbLiveView(this, this);
 
 	readDeviceConfigs();
 	updateSourceMapping();
@@ -54,6 +58,7 @@ DvbManager::~DvbManager()
 
 	// we need an explicit deletion order (device users ; devices ; device manager)
 
+	delete epgModel;
 	delete recordingModel;
 
 	foreach (const DvbDeviceConfig &deviceConfig, deviceConfigs) {
@@ -65,6 +70,7 @@ DvbDevice *DvbManager::requestDevice(const QString &source, const DvbTransponder
 	DvbManager::RequestType requestType)
 {
 	Q_ASSERT(requestType != Exclusive);
+	// FIXME call DvbEpgModel::startEventFilter / DvbEpgModel::stopEventFilter here?
 
 	for (int i = 0; i < deviceConfigs.size(); ++i) {
 		const DvbDeviceConfig &it = deviceConfigs.at(i);
@@ -73,7 +79,7 @@ DvbDevice *DvbManager::requestDevice(const QString &source, const DvbTransponder
 			continue;
 		}
 
-		if ((it.source == source) && it.transponder->corresponds(transponder)) {
+		if ((it.source == source) && it.transponder.corresponds(transponder)) {
 			++deviceConfigs[i].useCount;
 
 			if (requestType == Prioritized) {
@@ -161,7 +167,6 @@ DvbDevice *DvbManager::requestExclusiveDevice(const QString &source)
 
 				deviceConfigs[i].useCount = -1;
 				deviceConfigs[i].source.clear();
-				deviceConfigs[i].transponder = NULL;
 				return device;
 			}
 		}
@@ -288,7 +293,7 @@ QList<DvbTransponder> DvbManager::getTransponders(DvbDevice *device, const QStri
 	}
 
 	if ((scanSource.first == DvbS) &&
-	    ((device->getTransmissionTypes() & DvbBackendDevice::DvbS2) != 0)) {
+	    ((device->getTransmissionTypes() & DvbDevice::DvbS2) != 0)) {
 		scanSource.first = DvbS2;
 	}
 
@@ -323,28 +328,14 @@ bool DvbManager::updateScanData(const QByteArray &data)
 	return true;
 }
 
-QString DvbManager::getRecordingFolder()
+QString DvbManager::getRecordingFolder() const
 {
-	QString path = KGlobal::config()->group("DVB").readEntry("RecordingFolder");
-
-	if (path.isEmpty() || !QDir(path).exists()) {
-		path = QDir::homePath();
-		setRecordingFolder(path);
-	}
-
-	return path;
+	return KGlobal::config()->group("DVB").readEntry("RecordingFolder", QDir::homePath());
 }
 
-QString DvbManager::getTimeShiftFolder()
+QString DvbManager::getTimeShiftFolder() const
 {
-	QString path = KGlobal::config()->group("DVB").readEntry("TimeShiftFolder");
-
-	if (path.isEmpty() || !QDir(path).exists()) {
-		path = QDir::homePath();
-		setTimeShiftFolder(path);
-	}
-
-	return path;
+	return KGlobal::config()->group("DVB").readEntry("TimeShiftFolder", QDir::homePath());
 }
 
 int DvbManager::getBeginMargin() const
@@ -423,6 +414,11 @@ void DvbManager::enableDvbDump()
 	}
 }
 
+void DvbManager::requestBuiltinDeviceManager(QObject *&builtinDeviceManager)
+{
+	builtinDeviceManager = new DvbLinuxDeviceManager(this);
+}
+
 void DvbManager::deviceAdded(DvbBackendDevice *backendDevice)
 {
 	DvbDevice *device = new DvbDevice(backendDevice, this);
@@ -468,10 +464,9 @@ void DvbManager::deviceRemoved(DvbBackendDevice *backendDevice)
 
 void DvbManager::loadDeviceManager()
 {
-	/*
-	QDir dir(QString::fromUtf8(KAFFEINE_PLUGIN_INSTALL_DIR "/module"));
-	QStringList entries = dir.entryList(QStringList("kaffeinedvb*"));
-	qSort(entries.begin(), entries.end(), qGreater<QString>());
+	QDir dir(QString::fromUtf8(KAFFEINE_LIB_INSTALL_DIR "/"));
+	QStringList entries = dir.entryList(QStringList("*kaffeinedvb*"), QDir::NoFilter,
+		QDir::Name | QDir::Reversed);
 
 	foreach (const QString &entry, entries) {
 		QString path = dir.filePath(entry);
@@ -480,44 +475,32 @@ void DvbManager::loadDeviceManager()
 			continue;
 		}
 
-		typedef QObject *(*funcPointer)();
+		QObject *deviceManager = QPluginLoader(path).instance();
 
-		funcPointer func = (funcPointer) QLibrary::resolve(path, "create_device_manager");
-
-		if (func == NULL) {
-			kError() << "couldn't load dvb module" << path;
-			return;
-		}
-
-		QObject *deviceManager = func();
-
-		if (deviceManager->property("backendMagic") != dvbBackendMagic) {
-			kError() << "invalid magic number for dvb module" << path;
-			return;
+		if (deviceManager == NULL) {
+			kWarning() << "cannot load dvb device manager" << path;
+			break;
 		}
 
+		kDebug() << "using dvb device manager" << path;
 		deviceManager->setParent(this);
+		connect(deviceManager, SIGNAL(requestBuiltinDeviceManager(QObject*&)),
+			this, SLOT(requestBuiltinDeviceManager(QObject*&)));
 		connect(deviceManager, SIGNAL(deviceAdded(DvbBackendDevice*)),
 			this, SLOT(deviceAdded(DvbBackendDevice*)));
 		connect(deviceManager, SIGNAL(deviceRemoved(DvbBackendDevice*)),
 			this, SLOT(deviceRemoved(DvbBackendDevice*)));
 		QMetaObject::invokeMethod(deviceManager, "doColdPlug");
-
-		kDebug() << "successfully loaded" << path;
 		return;
 	}
 
-	kError() << "no dvb module found";
-	*/
-
-	QObject *deviceManager = new DvbDeviceManager();
-	deviceManager->setParent(this);
+	kDebug() << "using built-in dvb device manager";
+	DvbLinuxDeviceManager *deviceManager = new DvbLinuxDeviceManager(this);
 	connect(deviceManager, SIGNAL(deviceAdded(DvbBackendDevice*)),
 		this, SLOT(deviceAdded(DvbBackendDevice*)));
 	connect(deviceManager, SIGNAL(deviceRemoved(DvbBackendDevice*)),
 		this, SLOT(deviceRemoved(DvbBackendDevice*)));
-	QMetaObject::invokeMethod(deviceManager, "doColdPlug");
-	return;
+	deviceManager->doColdPlug();
 }
 
 void DvbManager::readDeviceConfigs()
@@ -759,34 +742,18 @@ bool DvbManager::readScanSources(DvbScanData &data, const char *tag, Transmissio
 			}
 
 			line = data.readLine();
-			DvbTransponderBase *transponder = NULL;
+			DvbTransponder transponder =
+				DvbTransponder::fromString(QString::fromAscii(line));
 
-			switch (type) {
-			case DvbC:
-				transponder = new DvbCTransponder;
-				break;
-			case DvbS:
-			case DvbS2:
-				if (line[1] == '2') {
-					transponder = new DvbS2Transponder;
-				} else {
-					transponder = new DvbSTransponder;
+			if (!transponder.isValid()) {
+				parseError = true;
+			} else {
+				transponders.append(transponder);
+
+				if (transponder.getTransmissionType() ==
+				    DvbTransponderBase::DvbS) {
 					containsDvbS1 = true;
 				}
-				break;
-			case DvbT:
-				transponder = new DvbTTransponder;
-				break;
-			case Atsc:
-				transponder = new AtscTransponder;
-				break;
-			}
-
-			if (!transponder->fromString(QString::fromAscii(line))) {
-				parseError = true;
-				delete transponder;
-			} else {
-				transponders.append(DvbTransponder(transponder));
 			}
 		}
 
@@ -799,7 +766,7 @@ bool DvbManager::readScanSources(DvbScanData &data, const char *tag, Transmissio
 
 			if (containsDvbS1) {
 				for (int i = 0; i < transponders.size(); ++i) {
-					if (transponders.at(i)->getTransmissionType() ==
+					if (transponders.at(i).getTransmissionType() ==
 					    DvbTransponderBase::DvbS2) {
 						transponders.removeAt(i);
 						--i;
