@@ -25,6 +25,7 @@
 #include <QMap>
 #include <QPair>
 #include <QStringList>
+#include "dvbtransponder.h"
 
 class QTreeView;
 class DvbBackendDevice;
@@ -37,8 +38,6 @@ class DvbEpgModel;
 class DvbLiveView;
 class DvbRecordingModel;
 class DvbScanData;
-class DvbTransponder;
-class DvbTransponderBase;
 class MediaWidget;
 
 class DvbManager : public QObject
@@ -121,8 +120,8 @@ public:
 	QList<DvbTransponder> getTransponders(DvbDevice *device, const QString &source);
 	bool updateScanData(const QByteArray &data);
 
-	QString getRecordingFolder();
-	QString getTimeShiftFolder();
+	QString getRecordingFolder() const;
+	QString getTimeShiftFolder() const;
 	int getBeginMargin() const; // seconds
 	int getEndMargin() const; // seconds
 	bool override6937Charset() const;
@@ -140,6 +139,7 @@ public:
 	void enableDvbDump();
 
 private slots:
+	void requestBuiltinDeviceManager(QObject *&builtinDeviceManager);
 	void deviceAdded(DvbBackendDevice *backendDevice);
 	void deviceRemoved(DvbBackendDevice *backendDevice);
 
@@ -186,7 +186,7 @@ public:
 	int useCount; // -1 means exclusive use
 	int prioritizedUseCount;
 	QString source;
-	QExplicitlySharedDataPointer<const DvbTransponderBase> transponder;
+	DvbTransponder transponder;
 };
 
 class DvbDeviceConfigUpdate
