@@ -22,6 +22,7 @@
 
 #include <QBitArray>
 #include <KDebug>
+#include "dvbdevice.h"
 #include "dvbsi.h"
 
 class DvbPatEntry
@@ -34,6 +35,8 @@ public:
 	int pid;
 };
 
+Q_DECLARE_TYPEINFO(DvbPatEntry, Q_MOVABLE_TYPE);
+
 class DvbSdtEntry
 {
 public:
@@ -68,7 +71,7 @@ public:
 
 private:
 	bool checkMultipleSection(const DvbStandardSection &section);
-	void processSection(const QByteArray &data);
+	void processSection(const char *data, int size);
 	void timerEvent(QTimerEvent *);
 
 	DvbScan *scan;
@@ -83,13 +86,11 @@ bool DvbScanFilter::startFilter(int pid_, DvbScan::FilterType type_)
 {
 	Q_ASSERT(pid == -1);
 
-	resetFilter();
-
 	pid = pid_;
 	type = type_;
 	multipleSections.clear();
 
-	if (!scan->device->addPidFilter(pid, this)) {
+	if (!scan->device->addSectionFilter(pid, this)) {
 		pid = -1;
 		return false;
 	}
@@ -108,7 +109,7 @@ void DvbScanFilter::stopFilter()
 {
 	if (pid != -1) {
 		killTimer(timerId);
-		scan->device->removePidFilter(pid, this);
+		scan->device->removeSectionFilter(pid, this);
 
 		pid = -1;
 	}
@@ -143,11 +144,11 @@ bool DvbScanFilter::checkMultipleSection(const DvbStandardSection &section)
 	return true;
 }
 
-void DvbScanFilter::processSection(const QByteArray &data)
+void DvbScanFilter::processSection(const char *data, int size)
 {
 	switch (type) {
 	case DvbScan::PatFilter: {
-		DvbPatSection patSection(data);
+		DvbPatSection patSection(data, size);
 
 		if (!patSection.isValid() || (patSection.tableId() != 0x0)) {
 			return;
@@ -159,12 +160,10 @@ void DvbScanFilter::processSection(const QByteArray &data)
 		}
 
 		scan->processPat(patSection);
-
 		break;
 	    }
-
 	case DvbScan::PmtFilter: {
-		DvbPmtSection pmtSection(data);
+		DvbPmtSection pmtSection(data, size);
 
 		if (!pmtSection.isValid() || (pmtSection.tableId() != 0x2)) {
 			return;
@@ -176,12 +175,10 @@ void DvbScanFilter::processSection(const QByteArray &data)
 		}
 
 		scan->processPmt(pmtSection, pid);
-
 		break;
 	    }
-
 	case DvbScan::SdtFilter: {
-		DvbSdtSection sdtSection(data);
+		DvbSdtSection sdtSection(data, size);
 
 		if (!sdtSection.isValid() || (sdtSection.tableId() != 0x42)) {
 			// there are also other tables in the SDT
@@ -194,12 +191,10 @@ void DvbScanFilter::processSection(const QByteArray &data)
 		}
 
 		scan->processSdt(sdtSection);
-
 		break;
 	    }
-
 	case DvbScan::VctFilter: {
-		AtscVctSection vctSection(data);
+		AtscVctSection vctSection(data, size);
 
 		if (!vctSection.isValid() ||
 		    ((vctSection.tableId() != 0xc8) && (vctSection.tableId() != 0xc9))) {
@@ -213,12 +208,10 @@ void DvbScanFilter::processSection(const QByteArray &data)
 		}
 
 		scan->processVct(vctSection);
-
 		break;
 	    }
-
 	case DvbScan::NitFilter: {
-		DvbNitSection nitSection(data);
+		DvbNitSection nitSection(data, size);
 
 		if (!nitSection.isValid() || (nitSection.tableId() != 0x40)) {
 			// we are only interested in the current network
@@ -231,7 +224,6 @@ void DvbScanFilter::processSection(const QByteArray &data)
 		}
 
 		scan->processNit(nitSection);
-
 		break;
 	    }
 	}
@@ -268,16 +260,17 @@ DvbScan::DvbScan(DvbDevice *device_, const QString &source_, const QString &auto
 		bool offsets = (autoScanSource == "AUTO-Offsets");
 
 		for (int frequency = 177500000; frequency <= 226500000; frequency += 7000000) {
-			DvbTTransponder *transponder = new DvbTTransponder;
-			transponder->frequency = frequency;
-			transponder->bandwidth = DvbTTransponder::Bandwidth7MHz;
-			transponder->modulation = DvbTTransponder::ModulationAuto;
-			transponder->fecRateHigh = DvbTTransponder::FecAuto;
-			transponder->fecRateLow = DvbTTransponder::FecNone;
-			transponder->transmissionMode = DvbTTransponder::TransmissionModeAuto;
-			transponder->guardInterval = DvbTTransponder::GuardIntervalAuto;
-			transponder->hierarchy = DvbTTransponder::HierarchyNone;
-			transponders.append(DvbTransponder(transponder));
+			DvbTransponder transponder(DvbTransponderBase::DvbT);
+			DvbTTransponder *dvbTTransponder = transponder.as<DvbTTransponder>();
+			dvbTTransponder->frequency = frequency;
+			dvbTTransponder->bandwidth = DvbTTransponder::Bandwidth7MHz;
+			dvbTTransponder->modulation = DvbTTransponder::ModulationAuto;
+			dvbTTransponder->fecRateHigh = DvbTTransponder::FecAuto;
+			dvbTTransponder->fecRateLow = DvbTTransponder::FecNone;
+			dvbTTransponder->transmissionMode = DvbTTransponder::TransmissionModeAuto;
+			dvbTTransponder->guardInterval = DvbTTransponder::GuardIntervalAuto;
+			dvbTTransponder->hierarchy = DvbTTransponder::HierarchyNone;
+			transponders.append(transponder);
 		}
 
 		for (int frequency = 474000000; frequency <= 858000000; frequency += 8000000) {
@@ -294,16 +287,20 @@ DvbScan::DvbScan(DvbDevice *device_, const QString &source_, const QString &auto
 					offset = 167000;
 				}
 
-				DvbTTransponder *transponder = new DvbTTransponder;
-				transponder->frequency = frequency + offset;
-				transponder->bandwidth = DvbTTransponder::Bandwidth8MHz;
-				transponder->modulation = DvbTTransponder::ModulationAuto;
-				transponder->fecRateHigh = DvbTTransponder::FecAuto;
-				transponder->fecRateLow = DvbTTransponder::FecNone;
-				transponder->transmissionMode = DvbTTransponder::TransmissionModeAuto;
-				transponder->guardInterval = DvbTTransponder::GuardIntervalAuto;
-				transponder->hierarchy = DvbTTransponder::HierarchyNone;
-				transponders.append(DvbTransponder(transponder));
+				DvbTransponder transponder(DvbTransponderBase::DvbT);
+				DvbTTransponder *dvbTTransponder =
+					transponder.as<DvbTTransponder>();
+				dvbTTransponder->frequency = frequency + offset;
+				dvbTTransponder->bandwidth = DvbTTransponder::Bandwidth8MHz;
+				dvbTTransponder->modulation = DvbTTransponder::ModulationAuto;
+				dvbTTransponder->fecRateHigh = DvbTTransponder::FecAuto;
+				dvbTTransponder->fecRateLow = DvbTTransponder::FecNone;
+				dvbTTransponder->transmissionMode =
+					DvbTTransponder::TransmissionModeAuto;
+				dvbTTransponder->guardInterval =
+					DvbTTransponder::GuardIntervalAuto;
+				dvbTTransponder->hierarchy = DvbTTransponder::HierarchyNone;
+				transponders.append(transponder);
 			}
 		}
 	} else if (autoScanSource == "AUTO-Australia") {
@@ -315,16 +312,20 @@ DvbScan::DvbScan(DvbDevice *device_, const QString &source_, const QString &auto
 					offset = 125000;
 				}
 
-				DvbTTransponder *transponder = new DvbTTransponder;
-				transponder->frequency = frequency + offset;
-				transponder->bandwidth = DvbTTransponder::Bandwidth7MHz;
-				transponder->modulation = DvbTTransponder::ModulationAuto;
-				transponder->fecRateHigh = DvbTTransponder::FecAuto;
-				transponder->fecRateLow = DvbTTransponder::FecNone;
-				transponder->transmissionMode = DvbTTransponder::TransmissionModeAuto;
-				transponder->guardInterval = DvbTTransponder::GuardIntervalAuto;
-				transponder->hierarchy = DvbTTransponder::HierarchyNone;
-				transponders.append(DvbTransponder(transponder));
+				DvbTransponder transponder(DvbTransponderBase::DvbT);
+				DvbTTransponder *dvbTTransponder =
+					transponder.as<DvbTTransponder>();
+				dvbTTransponder->frequency = frequency + offset;
+				dvbTTransponder->bandwidth = DvbTTransponder::Bandwidth7MHz;
+				dvbTTransponder->modulation = DvbTTransponder::ModulationAuto;
+				dvbTTransponder->fecRateHigh = DvbTTransponder::FecAuto;
+				dvbTTransponder->fecRateLow = DvbTTransponder::FecNone;
+				dvbTTransponder->transmissionMode =
+					DvbTTransponder::TransmissionModeAuto;
+				dvbTTransponder->guardInterval =
+					DvbTTransponder::GuardIntervalAuto;
+				dvbTTransponder->hierarchy = DvbTTransponder::HierarchyNone;
+				transponders.append(transponder);
 			}
 		}
 
@@ -336,16 +337,20 @@ DvbScan::DvbScan(DvbDevice *device_, const QString &source_, const QString &auto
 					offset = 125000;
 				}
 
-				DvbTTransponder *transponder = new DvbTTransponder;
-				transponder->frequency = frequency + offset;
-				transponder->bandwidth = DvbTTransponder::Bandwidth7MHz;
-				transponder->modulation = DvbTTransponder::ModulationAuto;
-				transponder->fecRateHigh = DvbTTransponder::FecAuto;
-				transponder->fecRateLow = DvbTTransponder::FecNone;
-				transponder->transmissionMode = DvbTTransponder::TransmissionModeAuto;
-				transponder->guardInterval = DvbTTransponder::GuardIntervalAuto;
-				transponder->hierarchy = DvbTTransponder::HierarchyNone;
-				transponders.append(DvbTransponder(transponder));
+				DvbTransponder transponder(DvbTransponderBase::DvbT);
+				DvbTTransponder *dvbTTransponder =
+					transponder.as<DvbTTransponder>();
+				dvbTTransponder->frequency = frequency + offset;
+				dvbTTransponder->bandwidth = DvbTTransponder::Bandwidth7MHz;
+				dvbTTransponder->modulation = DvbTTransponder::ModulationAuto;
+				dvbTTransponder->fecRateHigh = DvbTTransponder::FecAuto;
+				dvbTTransponder->fecRateLow = DvbTTransponder::FecNone;
+				dvbTTransponder->transmissionMode =
+					DvbTTransponder::TransmissionModeAuto;
+				dvbTTransponder->guardInterval =
+					DvbTTransponder::GuardIntervalAuto;
+				dvbTTransponder->hierarchy = DvbTTransponder::HierarchyNone;
+				transponders.append(transponder);
 			}
 		}
 	} else if (autoScanSource == "AUTO-Italy") {
@@ -354,43 +359,53 @@ DvbScan::DvbScan(DvbDevice *device_, const QString &source_, const QString &auto
 
 		for (unsigned i = 0; i < (sizeof(italyVhf) / sizeof(italyVhf[0])); ++i) {
 			for (int j = 0; j < 2; ++j) {
-				DvbTTransponder *transponder = new DvbTTransponder;
-				transponder->frequency = italyVhf[i];
-				transponder->bandwidth = (j == 0) ? DvbTTransponder::Bandwidth7MHz : DvbTTransponder::Bandwidth8MHz;
-				transponder->modulation = DvbTTransponder::ModulationAuto;
-				transponder->fecRateHigh = DvbTTransponder::FecAuto;
-				transponder->fecRateLow = DvbTTransponder::FecNone;
-				transponder->transmissionMode = DvbTTransponder::TransmissionModeAuto;
-				transponder->guardInterval = DvbTTransponder::GuardIntervalAuto;
-				transponder->hierarchy = DvbTTransponder::HierarchyNone;
-				transponders.append(DvbTransponder(transponder));
+				DvbTransponder transponder(DvbTransponderBase::DvbT);
+				DvbTTransponder *dvbTTransponder =
+					transponder.as<DvbTTransponder>();
+				dvbTTransponder->frequency = italyVhf[i];
+				dvbTTransponder->bandwidth = ((j == 0) ?
+					DvbTTransponder::Bandwidth7MHz :
+					DvbTTransponder::Bandwidth8MHz);
+				dvbTTransponder->modulation = DvbTTransponder::ModulationAuto;
+				dvbTTransponder->fecRateHigh = DvbTTransponder::FecAuto;
+				dvbTTransponder->fecRateLow = DvbTTransponder::FecNone;
+				dvbTTransponder->transmissionMode =
+					DvbTTransponder::TransmissionModeAuto;
+				dvbTTransponder->guardInterval =
+					DvbTTransponder::GuardIntervalAuto;
+				dvbTTransponder->hierarchy = DvbTTransponder::HierarchyNone;
+				transponders.append(transponder);
 			}
 		}
 
 		for (int frequency = 474000000; frequency <= 858000000; frequency += 8000000) {
-			DvbTTransponder *transponder = new DvbTTransponder;
-			transponder->frequency = frequency;
-			transponder->bandwidth = DvbTTransponder::Bandwidth8MHz;
-			transponder->modulation = DvbTTransponder::ModulationAuto;
-			transponder->fecRateHigh = DvbTTransponder::FecAuto;
-			transponder->fecRateLow = DvbTTransponder::FecNone;
-			transponder->transmissionMode = DvbTTransponder::TransmissionModeAuto;
-			transponder->guardInterval = DvbTTransponder::GuardIntervalAuto;
-			transponder->hierarchy = DvbTTransponder::HierarchyNone;
-			transponders.append(DvbTransponder(transponder));
+			DvbTransponder transponder(DvbTransponderBase::DvbT);
+			DvbTTransponder *dvbTTransponder =
+				transponder.as<DvbTTransponder>();
+			dvbTTransponder->frequency = frequency;
+			dvbTTransponder->bandwidth = DvbTTransponder::Bandwidth8MHz;
+			dvbTTransponder->modulation = DvbTTransponder::ModulationAuto;
+			dvbTTransponder->fecRateHigh = DvbTTransponder::FecAuto;
+			dvbTTransponder->fecRateLow = DvbTTransponder::FecNone;
+			dvbTTransponder->transmissionMode = DvbTTransponder::TransmissionModeAuto;
+			dvbTTransponder->guardInterval = DvbTTransponder::GuardIntervalAuto;
+			dvbTTransponder->hierarchy = DvbTTransponder::HierarchyNone;
+			transponders.append(transponder);
 		}
 	} else if (autoScanSource == "AUTO-Taiwan") {
 		for (int frequency = 527000000; frequency <= 599000000; frequency += 6000000) {
-			DvbTTransponder *transponder = new DvbTTransponder;
-			transponder->frequency = frequency;
-			transponder->bandwidth = DvbTTransponder::Bandwidth6MHz;
-			transponder->modulation = DvbTTransponder::ModulationAuto;
-			transponder->fecRateHigh = DvbTTransponder::FecAuto;
-			transponder->fecRateLow = DvbTTransponder::FecNone;
-			transponder->transmissionMode = DvbTTransponder::TransmissionModeAuto;
-			transponder->guardInterval = DvbTTransponder::GuardIntervalAuto;
-			transponder->hierarchy = DvbTTransponder::HierarchyNone;
-			transponders.append(DvbTransponder(transponder));
+			DvbTransponder transponder(DvbTransponderBase::DvbT);
+			DvbTTransponder *dvbTTransponder =
+				transponder.as<DvbTTransponder>();
+			dvbTTransponder->frequency = frequency;
+			dvbTTransponder->bandwidth = DvbTTransponder::Bandwidth6MHz;
+			dvbTTransponder->modulation = DvbTTransponder::ModulationAuto;
+			dvbTTransponder->fecRateHigh = DvbTTransponder::FecAuto;
+			dvbTTransponder->fecRateLow = DvbTTransponder::FecNone;
+			dvbTTransponder->transmissionMode = DvbTTransponder::TransmissionModeAuto;
+			dvbTTransponder->guardInterval = DvbTTransponder::GuardIntervalAuto;
+			dvbTTransponder->hierarchy = DvbTTransponder::HierarchyNone;
+			transponders.append(transponder);
 		}
 	}
 }
@@ -465,7 +480,7 @@ void DvbScan::updateState()
 			// fall through
 		case ScanNit: {
 			if (!isLive && !isAuto &&
-			    (transponder->getTransmissionType() != DvbTransponderBase::Atsc)) {
+			    (transponder.getTransmissionType() != DvbTransponderBase::Atsc)) {
 				if (!startFilter(0x10, NitFilter)) {
 					return;
 				}
@@ -475,7 +490,7 @@ void DvbScan::updateState()
 		    }
 			// fall through
 		case ScanSdt: {
-			if (transponder->getTransmissionType() != DvbTransponderBase::Atsc) {
+			if (transponder.getTransmissionType() != DvbTransponderBase::Atsc) {
 				if (!startFilter(0x11, SdtFilter)) {
 					return;
 				}
@@ -505,7 +520,7 @@ void DvbScan::updateState()
 				DvbPreviewChannel &channel = channels[i];
 
 				foreach (const DvbSdtEntry &sdtEntry, sdtEntries) {
-					if (channel.getServiceId() == sdtEntry.serviceId) {
+					if (channel.serviceId == sdtEntry.serviceId) {
 						channel.name = sdtEntry.name;
 						channel.networkId = sdtEntry.networkId;
 						channel.isScrambled = sdtEntry.scrambled;
@@ -517,7 +532,7 @@ void DvbScan::updateState()
 				if (channel.name.isEmpty()) {
 					channel.name = QString("#0 %1:%2").
 						       arg(channel.transportStreamId).
-						       arg(channel.getServiceId());
+						       arg(channel.serviceId);
 				}
 			}
 
@@ -570,7 +585,8 @@ void DvbScan::updateState()
 
 			case DvbDevice::DeviceTuned:
 				if (isAuto) {
-					transponders[transponderIndex - 1] = device->getAutoTransponder();
+					transponders[transponderIndex - 1] =
+						device->getAutoTransponder();
 				}
 
 				state = ScanPat;
@@ -605,16 +621,17 @@ void DvbScan::processPmt(const DvbPmtSection &section, int pid)
 	DvbPmtParser parser(section);
 	channel.hasVideo = (parser.videoPid >= 0);
 
-	for (int i = 0; i < parser.audioPids.size(); ++i) {
-		channel.audioPids.append(parser.audioPids.at(i).first);
+	if (!parser.audioPids.isEmpty()) {
+		channel.audioPid = parser.audioPids.at(0).first;
 	}
 
-	if (channel.hasVideo || !channel.audioPids.isEmpty()) {
+	if (channel.hasVideo || (channel.audioPid >= 0)) {
 		channel.source = source;
 		channel.transponder = transponder;
 		channel.transportStreamId = transportStreamId;
 		channel.pmtPid = pid;
-		channel.pmtSection = section.toByteArray();
+		channel.pmtSectionData = section.toByteArray();
+		channel.serviceId = section.programNumber();
 		channel.snr = snr;
 		channels.append(channel);
 	}
@@ -653,7 +670,8 @@ void DvbScan::processVct(const AtscVctSection &section)
 
 	for (AtscVctSectionEntry entry = section.entries(); (i > 0) && (entry.isValid());
 	     --i, entry.advance()) {
-		QString majorminor = QString("%1-%2 ").arg(entry.majorNumber(), 3, 10, QLatin1Char('0')).arg(entry.minorNumber());
+		QString majorminor = QString("%1-%2 ").
+			arg(entry.majorNumber(), 3, 10, QLatin1Char('0')).arg(entry.minorNumber());
 
 		DvbSdtEntry sdtEntry(entry.programNumber(), entry.sourceId(), entry.isScrambled());
 
@@ -931,7 +949,9 @@ void DvbScan::processNitDescriptor(const DvbDescriptor &descriptor)
 {
 	DvbTransponder newTransponder;
 
-	switch (transponder->getTransmissionType()) {
+	switch (transponder.getTransmissionType()) {
+	case DvbTransponderBase::Invalid:
+		break;
 	case DvbTransponderBase::DvbC: {
 		if (descriptor.descriptorTag() != 0x44) {
 			break;
@@ -943,14 +963,14 @@ void DvbScan::processNitDescriptor(const DvbDescriptor &descriptor)
 			break;
 		}
 
-		DvbCTransponder *dvbCTransponder = new DvbCTransponder();
+		newTransponder = DvbTransponder(DvbTransponderBase::DvbC);
+		DvbCTransponder *dvbCTransponder = newTransponder.as<DvbCTransponder>();
 		dvbCTransponder->frequency =
 			DvbDescriptor::bcdToInt(cableDescriptor.frequency(), 100);
 		dvbCTransponder->symbolRate =
 			DvbDescriptor::bcdToInt(cableDescriptor.symbolRate(), 100);
 		dvbCTransponder->modulation = extractDvbCModulation(cableDescriptor);
 		dvbCTransponder->fecRate = extractDvbCFecRate(cableDescriptor);
-		newTransponder = DvbTransponder(dvbCTransponder);
 		break;
 	    }
 	case DvbTransponderBase::DvbS:
@@ -968,13 +988,15 @@ void DvbScan::processNitDescriptor(const DvbDescriptor &descriptor)
 		DvbSTransponder *dvbSTransponder;
 
 		if (!satelliteDescriptor.isDvbS2()) {
-			dvbSTransponder = new DvbSTransponder();
+			newTransponder = DvbTransponder(DvbTransponderBase::DvbS);
+			dvbSTransponder = newTransponder.as<DvbSTransponder>();
 		} else {
-			if ((device->getTransmissionTypes() & DvbBackendDevice::DvbS2) == 0) {
+			if ((device->getTransmissionTypes() & DvbDevice::DvbS2) == 0) {
 				break;
 			}
 
-			DvbS2Transponder *dvbS2Transponder = new DvbS2Transponder();
+			newTransponder = DvbTransponder(DvbTransponderBase::DvbS2);
+			DvbS2Transponder *dvbS2Transponder = newTransponder.as<DvbS2Transponder>();
 			dvbS2Transponder->modulation = extractDvbS2Modulation(satelliteDescriptor);
 			dvbS2Transponder->rollOff = extractDvbS2RollOff(satelliteDescriptor);
 			dvbSTransponder = dvbS2Transponder;
@@ -986,7 +1008,6 @@ void DvbScan::processNitDescriptor(const DvbDescriptor &descriptor)
 		dvbSTransponder->symbolRate =
 			DvbDescriptor::bcdToInt(satelliteDescriptor.symbolRate(), 100);
 		dvbSTransponder->fecRate = extractDvbSFecRate(satelliteDescriptor);
-		newTransponder = DvbTransponder(dvbSTransponder);
 		break;
 	    }
 	case DvbTransponderBase::DvbT: {
@@ -1000,7 +1021,8 @@ void DvbScan::processNitDescriptor(const DvbDescriptor &descriptor)
 			break;
 		}
 
-		DvbTTransponder *dvbTTransponder = new DvbTTransponder();
+		newTransponder = DvbTransponder(DvbTransponderBase::DvbT);
+		DvbTTransponder *dvbTTransponder = newTransponder.as<DvbTTransponder>();
 		dvbTTransponder->frequency = terrestrialDescriptor.frequency() * 10;
 		dvbTTransponder->bandwidth = extractDvbTBandwidth(terrestrialDescriptor);
 		dvbTTransponder->modulation = extractDvbTModulation(terrestrialDescriptor);
@@ -1015,18 +1037,17 @@ void DvbScan::processNitDescriptor(const DvbDescriptor &descriptor)
 			dvbTTransponder->fecRateLow = DvbTTransponder::FecNone;
 		}
 
-		newTransponder = DvbTransponder(dvbTTransponder);
 		break;
 	    }
 	case DvbTransponderBase::Atsc:
 		break;
 	}
 
-	if (newTransponder.constData() != NULL) {
+	if (newTransponder.isValid()) {
 		bool duplicate = false;
 
 		foreach (const DvbTransponder &existingTransponder, transponders) {
-			if (existingTransponder->corresponds(newTransponder)) {
+			if (existingTransponder.corresponds(newTransponder)) {
 				duplicate = true;
 				break;
 			}
