@@ -21,7 +21,6 @@
 #ifndef DVBSCAN_H
 #define DVBSCAN_H
 
-#include <QObject>
 #include "dvbchannel.h"
 
 class AtscVctSection;
@@ -49,9 +48,10 @@ public:
 	// DvbTransponder transponder;
 	// int transportStreamId;
 	// int pmtPid;
-	// QByteArray pmtSection;
+	// QByteArray pmtSectionData;
+	// int serviceId;
+	// int audioPid; // may be -1 (not present)
 	// bool hasVideo;
-	QList<int> audioPids;
 	int snr; // percent
 
 	/*
@@ -68,7 +68,6 @@ public:
 	 */
 
 	// int number;
-	// int audioPid; // may be -1 (not present)
 };
 
 class DvbScan : public QObject
