@@ -33,7 +33,7 @@
 #include <KLed>
 #include <KLocale>
 #include <KMessageBox>
-#include "dvbchannelui.h"
+#include "dvbchanneldialog.h"
 #include "dvbdevice.h"
 #include "dvbliveview.h"
 #include "dvbmanager.h"
@@ -79,11 +79,11 @@ void DvbGradProgress::paintEvent(QPaintEvent *event)
 	QLabel::paintEvent(event);
 }
 
-class DvbPreviewChannelModel : public QAbstractTableModel
+class DvbPreviewChannelTableModel : public QAbstractTableModel
 {
 public:
-	explicit DvbPreviewChannelModel(QObject *parent) : QAbstractTableModel(parent) { }
-	~DvbPreviewChannelModel() { }
+	explicit DvbPreviewChannelTableModel(QObject *parent) : QAbstractTableModel(parent) { }
+	~DvbPreviewChannelTableModel() { }
 
 	enum ItemDataRole
 	{
@@ -126,7 +126,7 @@ private:
 
 Q_DECLARE_METATYPE(const DvbPreviewChannel *)
 
-QAbstractItemModel *DvbPreviewChannelModel::createProxyModel(QObject *parent)
+QAbstractItemModel *DvbPreviewChannelTableModel::createProxyModel(QObject *parent)
 {
 	QSortFilterProxyModel *proxyModel = new QSortFilterProxyModel(parent);
 	proxyModel->setDynamicSortFilter(true);
@@ -136,7 +136,7 @@ QAbstractItemModel *DvbPreviewChannelModel::createProxyModel(QObject *parent)
 	return proxyModel;
 }
 
-int DvbPreviewChannelModel::columnCount(const QModelIndex &parent) const
+int DvbPreviewChannelTableModel::columnCount(const QModelIndex &parent) const
 {
 	if (parent.isValid()) {
 		return 0;
@@ -145,7 +145,7 @@ int DvbPreviewChannelModel::columnCount(const QModelIndex &parent) const
 	return 3;
 }
 
-int DvbPreviewChannelModel::rowCount(const QModelIndex &parent) const
+int DvbPreviewChannelTableModel::rowCount(const QModelIndex &parent) const
 {
 	if (parent.isValid()) {
 		return 0;
@@ -154,7 +154,7 @@ int DvbPreviewChannelModel::rowCount(const QModelIndex &parent) const
 	return channels.size();
 }
 
-QVariant DvbPreviewChannelModel::data(const QModelIndex &index, int role) const
+QVariant DvbPreviewChannelTableModel::data(const QModelIndex &index, int role) const
 {
 	const DvbPreviewChannel &channel = channels.at(index.row());
 
@@ -195,8 +195,8 @@ QVariant DvbPreviewChannelModel::data(const QModelIndex &index, int role) const
 	return QVariant();
 }
 
-QVariant DvbPreviewChannelModel::headerData(int section, Qt::Orientation orientation, int role)
-	const
+QVariant DvbPreviewChannelTableModel::headerData(int section, Qt::Orientation orientation,
+	int role) const
 {
 	if (orientation != Qt::Horizontal || role != Qt::DisplayRole) {
 		return QVariant();
@@ -204,7 +204,7 @@ QVariant DvbPreviewChannelModel::headerData(int section, Qt::Orientation orienta
 
 	switch (section) {
 	case 0:
-		return i18n("Name");
+		return i18nc("@title:column tv show", "Channel");
 	case 1:
 		return i18n("Provider");
 	case 2:
@@ -228,18 +228,20 @@ DvbScanDialog::DvbScanDialog(DvbManager *manager_, QWidget *parent) : KDialog(pa
 
 	channelModel = new DvbChannelModel(this);
 	channelModel->cloneFrom(manager->getChannelModel());
+	DvbChannelTableModel *channelTableModel = new DvbChannelTableModel(this);
 
 	DvbChannelView *channelView = new DvbChannelView(groupBox);
 	channelView->setContextMenuPolicy(Qt::ActionsContextMenu);
 	channelView->setDragDropMode(QAbstractItemView::InternalMove);
-	channelView->setModel(channelModel->createProxyModel(channelView));
+	channelView->setModel(channelTableModel);
 	channelView->setRootIsDecorated(false);
 	channelView->setSelectionMode(QAbstractItemView::ExtendedSelection);
 	QHeaderView *header = manager->getChannelView()->header();
 	channelView->sortByColumn(header->sortIndicatorSection(), header->sortIndicatorOrder());
 	channelView->setSortingEnabled(true);
-	connect(channelModel, SIGNAL(checkInternalMove(bool*)),
-		channelView, SLOT(checkInternalMove(bool*)));
+	channelTableModel->setChannelModel(channelModel);
+	connect(channelTableModel, SIGNAL(checkChannelDragAndDrop(bool*)),
+		channelView, SLOT(checkChannelDragAndDrop(bool*)));
 
 	KAction *action = channelView->addEditAction();
 	QPushButton *pushButton = new QPushButton(action->icon(), action->text(), groupBox);
@@ -338,7 +340,7 @@ DvbScanDialog::DvbScanDialog(DvbManager *manager_, QWidget *parent) : KDialog(pa
 	groupBox = new QGroupBox(i18n("Scan Results"), mainWidget);
 	groupLayout = new QVBoxLayout(groupBox);
 
-	previewModel = new DvbPreviewChannelModel(this);
+	previewModel = new DvbPreviewChannelTableModel(this);
 	scanResultsView = new QTreeView(groupBox);
 	scanResultsView->setModel(previewModel->createProxyModel(groupBox));
 	scanResultsView->setRootIsDecorated(false);
@@ -400,12 +402,12 @@ void DvbScanDialog::scanButtonClicked(bool checked)
 	// start scan
 	Q_ASSERT(internal == NULL);
 
-	if (manager->getLiveView()->getChannel() == NULL) {
+	if (!manager->getLiveView()->getChannel().isValid()) {
 		isLive = false; // FIXME workaround
 	}
 
 	if (isLive) {
-		const DvbChannel *channel = manager->getLiveView()->getChannel();
+		const DvbSharedChannel &channel = manager->getLiveView()->getChannel();
 		internal = new DvbScan(device, channel->source, channel->transponder);
 	} else {
 		QString source = sourceBox->currentText();
@@ -423,7 +425,8 @@ void DvbScanDialog::scanButtonClicked(bool checked)
 			}
 		} else {
 			scanButton->setChecked(false);
-			KMessageBox::sorry(this, i18nc("message box", "No available device found."));
+			KMessageBox::sorry(this,
+				i18nc("message box", "No available device found."));
 			return;
 		}
 	}
@@ -437,7 +440,8 @@ void DvbScanDialog::scanButtonClicked(bool checked)
 		this, SLOT(foundChannels(QList<DvbPreviewChannel>)));
 	connect(internal, SIGNAL(scanProgress(int)), progressBar, SLOT(setValue(int)));
 	// calling scanFinished() will delete internal, so we have to queue the signal!
-	connect(internal, SIGNAL(scanFinished()), this, SLOT(scanFinished()), Qt::QueuedConnection);
+	connect(internal, SIGNAL(scanFinished()),
+		this, SLOT(scanFinished()), Qt::QueuedConnection);
 
 	internal->start();
 }
@@ -501,10 +505,10 @@ void DvbScanDialog::addSelectedChannels()
 		if (!selectedRows.contains(modelIndex.row())) {
 			selectedRows.insert(modelIndex.row());
 			const DvbChannel *channel = scanResultsView->model()->data(modelIndex,
-				DvbPreviewChannelModel::DvbPreviewChannelRole).
+				DvbPreviewChannelTableModel::DvbPreviewChannelRole).
 				value<const DvbPreviewChannel *>();
-			// if the index is invalid, the channel is added
-			channelModel->setData(QModelIndex(), QVariant::fromValue(channel));
+			DvbChannel newChannel(*channel);
+			channelModel->addChannel(newChannel);
 		}
 	}
 }
@@ -542,9 +546,8 @@ void DvbScanDialog::addFilteredChannels()
 			}
 		}
 
-		const DvbChannel *constChannel = &channel;
-		// if the index is invalid, the channel is added
-		channelModel->setData(QModelIndex(), QVariant::fromValue(constChannel));
+		DvbChannel newChannel(channel);
+		channelModel->addChannel(newChannel);
 	}
 }
 
