@@ -1,7 +1,7 @@
 /*
  * dvbsi.cpp
  *
- * Copyright (C) 2008-2009 Christoph Pfister <christophpfister@gmail.com>
+ * Copyright (C) 2008-2010 Christoph Pfister <christophpfister@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,157 +23,31 @@
 #include <QTextCodec>
 #include <KDebug>
 
-// FIXME some debug messages may be printed too often
-
-void DvbSectionFilter::processData(const char data[188])
+void DvbSection::initSection(const char *data, int size)
 {
-	if ((data[3] & 0x10) == 0) {
-		// no payload
-		kDebug() << "no payload";
+	if (size < 3) {
+		initSectionData();
 		return;
 	}
 
-	unsigned char continuity = (data[3] & 0x0f);
+	int sectionLength = ((((static_cast<unsigned char>(data[1]) & 0xf) << 8) |
+		static_cast<unsigned char>(data[2])) + 3);
 
-	if (bufferValid) {
-		// check continuity
-		if (continuity == continuityCounter) {
-			// duplicate packets
-			kDebug() << "duplicate packets";
-			return;
-		}
-
-		if (continuity != ((continuityCounter + 1) & 0x0f)) {
-			// discontinuity
-			kDebug() << "discontinuity";
-		}
-	}
-
-	continuityCounter = continuity;
-
-	bool sectionStart = ((data[1] & 0x40) != 0);
-	const char *payload;
-	int payloadLength;
-
-	if ((data[3] & 0x20) == 0) {
-		// adaptation field not present
-		payload = data + 4;
-		payloadLength = 188 - 4;
-	} else {
-		// adaptation field present
-		unsigned char length = data[4];
-
-		if (length > 182) {
-			kDebug() << "no payload or corrupt";
-			return;
-		}
-
-		payload = data + 5 + length;
-		payloadLength = 188 - 5 - length;
-	}
-
-	// be careful that playloadLength is > 0 at this point
-
-	if (sectionStart) {
-		unsigned char pointer = payload[0];
-
-		if (pointer >= payloadLength) {
-			kDebug() << "invalid pointer";
-			pointer = payloadLength - 1;
-		}
-
-		if (bufferValid) {
-			appendData(payload + 1, pointer);
-			processSections(true);
-			// be aware that the filter might have been reset
-			// (buffer cleared, bufferValid set to false)
-			// in this case don't poison the filter with new data
-			// --> check bufferValid before calling appendData()
-		} else {
-			bufferValid = true;
-		}
-
-		payload += pointer + 1;
-		payloadLength -= pointer + 1;
-	}
-
-	if (bufferValid) {
-		appendData(payload, payloadLength);
-		processSections();
-	}
-}
-
-void DvbSectionFilter::appendData(const char *data, int length)
-{
-	int size = buffer.size();
-	buffer.resize(size + length);
-	memcpy(buffer.data() + size, data, length);
-}
-
-void DvbSectionFilter::processSections(bool force)
-{
-	while (buffer.size() >= 3) {
-		int tableId = static_cast<unsigned char>(buffer.at(0));
-
-		if (tableId == 0xff) {
-			// padding
-			buffer.clear();
-			break;
-		}
-
-		int sectionLength = (((static_cast<unsigned char>(buffer.at(1)) & 0x0f) << 8) |
-				     static_cast<unsigned char>(buffer.at(2))) + 3;
-
-		if (sectionLength <= buffer.size()) {
-			QByteArray section = buffer.left(sectionLength);
-			buffer.remove(0, sectionLength);
-			processSection(section);
-			continue;
-		}
-
-		if (force) {
-			kDebug() << "short section";
-			QByteArray section = buffer;
-			buffer.clear();
-			processSection(section);
-		}
-
-		break;
-	}
-
-	if (force && !buffer.isEmpty()) {
-		int tableId = static_cast<unsigned char>(buffer.at(0));
-
-		if (tableId != 0xff) {
-			kDebug() << "stray data";
-		}
-
-		buffer.clear();
-	}
-}
-
-DvbSection::DvbSection(const QByteArray &data) : DvbSectionData(data)
-{
-	if (length < 3) {
-		length = 0;
-		return;
-	}
-
-	int sectionLength = (((at(1) & 0xf) << 8) | at(2)) + 3;
-
-	if (sectionLength <= length) {
-		length = sectionLength;
-	} else {
+	if (sectionLength > size) {
 		kDebug() << "adjusting length";
+		sectionLength = size;
 	}
+
+	initSectionData(data, sectionLength, size);
 }
 
-int DvbStandardSection::verifyCrc32() const
+int DvbStandardSection::verifyCrc32(const char *data, int size)
 {
 	unsigned int crc32 = 0xffffffff;
 
-	for (int i = 0; i < length; ++i) {
-		crc32 = (crc32 << 8) ^ crc32Table[(crc32 >> 24) ^ at(i)];
+	for (int i = 0; i < size; ++i) {
+		crc32 = ((crc32 << 8) ^
+			crc32Table[(crc32 >> 24) ^ static_cast<unsigned char>(data[i])]);
 	}
 
 	return crc32;
@@ -264,6 +138,16 @@ const unsigned int DvbStandardSection::crc32Table[] =
 	0xbcb4666d, 0xb8757bda, 0xb5365d03, 0xb1f740b4
 };
 
+void DvbStandardSection::initStandardSection(const char *data, int size)
+{
+	if (size < 12) {
+		initSectionData();
+		return;
+	}
+
+	initSection(data, size);
+}
+
 class Iso6937Codec : public QTextCodec
 {
 public:
@@ -353,20 +237,17 @@ const unsigned short Iso6937Codec::table[] = {
 	0x0142, 0x00f8, 0x0153, 0x00df, 0x00fe, 0x0167, 0x014b, 0x00ad
 };
 
-QString DvbSiText::convertText(const DvbSectionData &text)
+QString DvbSiText::convertText(const char *data, int size)
 {
-	const char *data = text.getData();
-	int length = text.getLength();
-
-	if (length < 1) {
+	if (size < 1) {
 		return QString();
 	}
 
 	// determine encoding
 	TextEncoding encoding = (override6937 ? Iso8859_1 : Iso6937);
 
-	if (text.at(0) < 0x20) {
-		switch (text.at(0)) {
+	if (static_cast<unsigned char>(data[0]) < 0x20) {
+		switch (data[0]) {
 		case 0x01: encoding = Iso8859_5; break;
 		case 0x02: encoding = Iso8859_6; break;
 		case 0x03: encoding = Iso8859_7; break;
@@ -374,25 +255,22 @@ QString DvbSiText::convertText(const DvbSectionData &text)
 		case 0x05: encoding = Iso8859_9; break;
 		case 0x06: encoding = Iso8859_10; break;
 		case 0x07: encoding = Iso8859_11; break;
-		case 0x08: encoding = Iso8859_12; break;
 		case 0x09: encoding = Iso8859_13; break;
 		case 0x0a: encoding = Iso8859_14; break;
 		case 0x0b: encoding = Iso8859_15; break;
-
 		case 0x13: encoding = Gb2312; break;
 		case 0x14: encoding = Big5; break;
 		case 0x15: encoding = Utf_8; break;
-
 		case 0x10: {
-			if (length < 3) {
+			if (size < 3) {
 				return QString();
 			}
 
-			if (text.at(1) != 0) {
+			if (data[1] != 0) {
 				return QString();
 			}
 
-			switch (text.at(2)) {
+			switch (data[2]) {
 			case 0x01: encoding = Iso8859_1; break;
 			case 0x02: encoding = Iso8859_2; break;
 			case 0x03: encoding = Iso8859_3; break;
@@ -404,27 +282,23 @@ QString DvbSiText::convertText(const DvbSectionData &text)
 			case 0x09: encoding = Iso8859_9; break;
 			case 0x0a: encoding = Iso8859_10; break;
 			case 0x0b: encoding = Iso8859_11; break;
-			case 0x0c: encoding = Iso8859_12; break;
 			case 0x0d: encoding = Iso8859_13; break;
 			case 0x0e: encoding = Iso8859_14; break;
 			case 0x0f: encoding = Iso8859_15; break;
-
 			default:
 				return QString();
 			}
 
 			data += 2;
-			length -= 2;
-
+			size -= 2;
 			break;
 		    }
-
 		default:
 			return QString();
 		}
 
 		data++;
-		length--;
+		size--;
 	}
 
 	if (codecTable[encoding] == NULL) {
@@ -443,7 +317,6 @@ QString DvbSiText::convertText(const DvbSectionData &text)
 		case Iso8859_9: codec = QTextCodec::codecForName("ISO 8859-9"); break;
 		case Iso8859_10: codec = QTextCodec::codecForName("ISO 8859-10"); break;
 		case Iso8859_11: codec = QTextCodec::codecForName("ISO 8859-11"); break;
-		case Iso8859_12: codec = QTextCodec::codecForName("ISO 8859-12"); break;
 		case Iso8859_13: codec = QTextCodec::codecForName("ISO 8859-13"); break;
 		case Iso8859_14: codec = QTextCodec::codecForName("ISO 8859-14"); break;
 		case Iso8859_15: codec = QTextCodec::codecForName("ISO 8859-15"); break;
@@ -459,10 +332,10 @@ QString DvbSiText::convertText(const DvbSectionData &text)
 	if (encoding <= Iso8859_15) {
 		// only strip control codes for one-byte character tables
 
-		char *dest = new char[length];
+		char *dest = new char[size];
 		char *destIt = dest;
 
-		for (const char *it = data; it != (data + length); ++it) {
+		for (const char *it = data; it != (data + size); ++it) {
 			unsigned char value = *it;
 
 			if ((value < 0x80) || (value > 0x9f)) {
@@ -476,7 +349,7 @@ QString DvbSiText::convertText(const DvbSectionData &text)
 		return result;
 	}
 
-	return codecTable[encoding]->toUnicode(data, length);
+	return codecTable[encoding]->toUnicode(data, size);
 }
 
 void DvbSiText::setOverride6937(bool override)
@@ -487,24 +360,25 @@ void DvbSiText::setOverride6937(bool override)
 QTextCodec *DvbSiText::codecTable[EncodingTypeMax + 1] = { NULL };
 bool DvbSiText::override6937 = false;
 
-DvbDescriptor::DvbDescriptor(const DvbSectionData &data) : DvbSectionData(data)
+void DvbDescriptor::initDescriptor(const char *data, int size)
 {
-	if (length < 2) {
-		if (length != 0) {
+	if (size < 2) {
+		if (size != 0) {
 			kDebug() << "invalid descriptor";
 		}
 
-		length = 0;
+		initSectionData();
 		return;
 	}
 
-	int descriptorLength = at(1) + 2;
+	int descriptorLength = (static_cast<unsigned char>(data[1]) + 2);
 
-	if (descriptorLength <= length) {
-		length = descriptorLength;
-	} else {
+	if (descriptorLength > size) {
 		kDebug() << "adjusting length";
+		descriptorLength = size;
 	}
+
+	initSectionData(data, descriptorLength, size);
 }
 
 QString AtscPsipText::interpretTextData(const char *data, unsigned int len,
@@ -539,15 +413,16 @@ QString AtscPsipText::interpretTextData(const char *data, unsigned int len,
 	return result;
 }
 
-QString AtscPsipText::convertText(const DvbSectionData &text)
+QString AtscPsipText::convertText(const char *data, int size)
 {
-	int length = text.getLength();
 	QString result;
 
-	if (length < 1) {
+	if (size < 1) {
 		return QString();
 	}
-	unsigned int num_strings = text.at(0);
+
+	unsigned int num_strings = static_cast<unsigned char>(data[0]);
+
 	if (num_strings == 0) {
 		return result;
 	}
@@ -558,25 +433,31 @@ QString AtscPsipText::convertText(const DvbSectionData &text)
 
 	// First three bytes are the ISO 639 language code
 	offset += 3;
-	if (offset > length) {
-		kWarning() << "Unexpected end";
+
+	if (offset > size) {
+		kDebug() << "adjusting length";
 		return result;
 	}
-	int num_segments = text.at(offset++);
+
+	int num_segments = static_cast<unsigned char>(data[offset++]);
+
 	for (int j = 0; j < num_segments; j++) {
-		if (offset + 3 > length) {
-			kWarning() << "Unexpected end";
+		if ((offset + 3) > size) {
+			kDebug() << "adjusting length";
 			return result;
 		}
-		int comp_type = text.at(offset++);
-		int mode = text.at(offset++);
-		int num_bytes = text.at(offset++);
-		if (offset + num_bytes > length)
-		{
-			kWarning() << "Unexpected end";
+
+		int comp_type = static_cast<unsigned char>(data[offset++]);
+		int mode = static_cast<unsigned char>(data[offset++]);
+		int num_bytes = static_cast<unsigned char>(data[offset++]);
+
+		if ((offset + num_bytes) > size) {
+			kDebug() << "adjusting length";
 			return result;
 		}
-		const char *comp_string = text.getData() + offset;
+
+		const char *comp_string = (data + offset);
+
 		if (comp_type == 0x00) {
 			// Uncompressed
 			result += interpretTextData(comp_string,
@@ -585,13 +466,15 @@ QString AtscPsipText::convertText(const DvbSectionData &text)
 			   mode == 0x00) {
 			// Huffman Compression
 			// As per the spec, only mode 0x00 supports Huffman coding
-			result += AtscHuffmanString::convertText(comp_string, num_bytes, comp_type);
+			result +=
+				AtscHuffmanString::convertText(comp_string, num_bytes, comp_type);
 		} else {
-			kWarning() << "Unsupported compression type:" 
-				   << comp_type << "mode:" << mode;
+			kWarning() << "Unsupported compression / mode:" << comp_type << mode;
 		}
+
 		offset += num_bytes;
 	}
+
 	return result;
 }
 
@@ -643,7 +526,8 @@ unsigned char AtscHuffmanString::getByte()
 		// Note this takes advantage of the fact that there is always
 		// at least one byte remaining (to avoid a read-past end
 		// condition)
-		unsigned char value = (((data[0] << 8) | ((unsigned char) data[1])) >> (shift + 1)) & 0xff;
+		unsigned char value =
+			((((data[0] << 8) | ((unsigned char) data[1])) >> (shift + 1)) & 0xff);
 
 		bitsLeft -= 8;
 		data++;
@@ -1131,6 +1015,29 @@ const unsigned char AtscHuffmanString::Huffman2Tables[] = {
 	0x02, 0x03, 0x04, 0x05, 0x9b, 0x9b, 0x9b, 0x9b,
 	0x9b, 0x9b, 0x9b, 0x9b, 0x9b, 0x9b };
 
+void DvbPmtFilter::processSection(const char *data, int size)
+{
+	unsigned char tableId = data[0];
+
+	if (tableId != 0x02) {
+		return;
+	}
+
+	DvbPmtSection pmtSection(data, size);
+
+	if (!pmtSection.isValid() || (pmtSection.programNumber() != programNumber)) {
+		return;
+	}
+
+	if ((size == lastPmtSectionData.size()) &&
+	    (memcmp(data, lastPmtSectionData.constData(), size) == 0)) {
+		return;
+	}
+
+	lastPmtSectionData = pmtSection.toByteArray();
+	emit pmtSectionChanged(lastPmtSectionData);
+}
+
 void DvbSectionGenerator::initPat(int transportStreamId, int programNumber, int pmtPid)
 {
 	Q_ASSERT((pmtPid >= 0) && (pmtPid <= 0x1fff));
@@ -1157,14 +1064,12 @@ void DvbSectionGenerator::initPmt(int pmtPid, const DvbPmtSection &section, cons
 	int size = entry.getData() - section.getData();
 
 	while (entry.isValid()) {
-		const char *begin = entry.getData();
-		bool selected = pids.contains(entry.pid());
-		entry.advance();
-
-		if (selected) {
-			memcpy(data + size, begin, entry.getData() - begin);
-			size += entry.getData() - begin;
+		if (pids.contains(entry.pid())) {
+			memcpy(data + size, entry.getData(), entry.getLength());
+			size += entry.getLength();
 		}
+
+		entry.advance();
 	}
 
 	endSection(size + 4, pmtPid);
@@ -1238,20 +1143,6 @@ void DvbSectionGenerator::endSection(int sectionLength, int pid)
 	versionNumber = (versionNumber + 1) & 0x1f;
 }
 
-void DvbPmtFilter::processSection(const QByteArray &data)
-{
-	DvbPmtSection pmtSection(data);
-
-	if (!pmtSection.isValid() || (pmtSection.tableId() != 0x2) ||
-	    (pmtSection.programNumber() != programNumber) ||
-	    (pmtSection.versionNumber() == versionNumber)) {
-		return;
-	}
-
-	versionNumber = pmtSection.versionNumber();
-	emit pmtSectionChanged(pmtSection);
-}
-
 DvbPmtParser::DvbPmtParser(const DvbPmtSection &section) : videoPid(-1), teletextPid(-1)
 {
 	for (DvbPmtSectionEntry entry = section.entries(); entry.isValid(); entry.advance()) {
@@ -1315,11 +1206,12 @@ DvbPmtParser::DvbPmtParser(const DvbPmtSection &section) : videoPid(-1), teletex
 		case 0x02: // MPEG2 video
 		case 0x10: // MPEG4 video
 		case 0x1b: // H264 video
-			if (videoPid != -1) {
+			if (videoPid < 0) {
+				videoPid = entry.pid();
+			} else {
 				kDebug() << "more than one video pid";
 			}
 
-			videoPid = entry.pid();
 			break;
 
 		case 0x03: // MPEG1 audio
@@ -1333,11 +1225,11 @@ DvbPmtParser::DvbPmtParser(const DvbPmtSection &section) : videoPid(-1), teletex
 
 		case 0x06: // private data - can be teletext, subtitle, ac3 or something else
 			if (teletextPresent) {
-				if (teletextPid != -1) {
+				if (teletextPid < 0) {
+					teletextPid = entry.pid();
+				} else {
 					kDebug() << "more than one teletext pid";
 				}
-
-				teletextPid = entry.pid();
 			}
 
 			if (!subtitleLanguage.isEmpty()) {
@@ -1368,310 +1260,406 @@ DvbPmtParser::DvbPmtParser(const DvbPmtSection &section) : videoPid(-1), teletex
 	}
 }
 
+void AtscEitSectionEntry::initEitSectionEntry(const char *data, int size)
+{
+	if (size < 12) {
+		if (size != 0) {
+			kDebug() << "invalid entry";
+		}
+
+		initSectionData();
+		return;
+	}
+
+	titleLength = static_cast<unsigned char>(data[9]);
+
+	if (titleLength > (size - 12)) {
+		kDebug() << "adjusting length";
+		titleLength = (size - 12);
+	}
+
+	// too ugly to be automatically generated
+	int entryLength = ((((static_cast<unsigned char>(data[10 + titleLength]) & 0xf) << 8) |
+		static_cast<unsigned char>(data[11 + titleLength])) + 12 + titleLength);
+
+	if (entryLength > size) {
+		kDebug() << "adjusting length";
+		entryLength = size;
+	}
+
+	initSectionData(data, entryLength, size);
+}
+
 // everything below this line is automatically generated
 
 DvbLanguageDescriptor::DvbLanguageDescriptor(const DvbDescriptor &descriptor) : DvbDescriptor(descriptor)
 {
-	if (length < 6) {
+	if (getLength() < 6) {
 		kDebug() << "invalid descriptor";
-		length = 0;
+		initSectionData();
 		return;
 	}
 }
 
 DvbSubtitleDescriptor::DvbSubtitleDescriptor(const DvbDescriptor &descriptor) : DvbDescriptor(descriptor)
 {
-	if (length < 10) {
+	if (getLength() < 10) {
 		kDebug() << "invalid descriptor";
-		length = 0;
+		initSectionData();
 		return;
 	}
 }
 
 DvbServiceDescriptor::DvbServiceDescriptor(const DvbDescriptor &descriptor) : DvbDescriptor(descriptor)
 {
-	if (length < 5) {
+	if (getLength() < 5) {
 		kDebug() << "invalid descriptor";
-		length = 0;
+		initSectionData();
 		return;
 	}
 
 	providerNameLength = at(3);
 
-	if (providerNameLength > (length - 5)) {
+	if (providerNameLength > (getLength() - 5)) {
 		kDebug() << "adjusting length";
-		providerNameLength = length - 5;
+		providerNameLength = (getLength() - 5);
 	}
 
 	serviceNameLength = at(4 + providerNameLength);
 
-	if (serviceNameLength > (length - (5 + providerNameLength))) {
+	if (serviceNameLength > (getLength() - (5 + providerNameLength))) {
 		kDebug() << "adjusting length";
-		serviceNameLength = length - (5 + providerNameLength);
+		serviceNameLength = (getLength() - (5 + providerNameLength));
 	}
 }
 
 DvbShortEventDescriptor::DvbShortEventDescriptor(const DvbDescriptor &descriptor) : DvbDescriptor(descriptor)
 {
-	if (length < 7) {
+	if (getLength() < 7) {
 		kDebug() << "invalid descriptor";
-		length = 0;
+		initSectionData();
 		return;
 	}
 
 	eventNameLength = at(5);
 
-	if (eventNameLength > (length - 7)) {
+	if (eventNameLength > (getLength() - 7)) {
 		kDebug() << "adjusting length";
-		eventNameLength = length - 7;
+		eventNameLength = (getLength() - 7);
 	}
 
 	textLength = at(6 + eventNameLength);
 
-	if (textLength > (length - (7 + eventNameLength))) {
+	if (textLength > (getLength() - (7 + eventNameLength))) {
 		kDebug() << "adjusting length";
-		textLength = length - (7 + eventNameLength);
+		textLength = (getLength() - (7 + eventNameLength));
 	}
 }
 
 DvbExtendedEventDescriptor::DvbExtendedEventDescriptor(const DvbDescriptor &descriptor) : DvbDescriptor(descriptor)
 {
-	if (length < 8) {
+	if (getLength() < 8) {
 		kDebug() << "invalid descriptor";
-		length = 0;
+		initSectionData();
 		return;
 	}
 
 	itemsLength = at(6);
 
-	if (itemsLength > (length - 8)) {
+	if (itemsLength > (getLength() - 8)) {
 		kDebug() << "adjusting length";
-		itemsLength = length - 8;
+		itemsLength = (getLength() - 8);
 	}
 
 	textLength = at(7 + itemsLength);
 
-	if (textLength > (length - (8 + itemsLength))) {
+	if (textLength > (getLength() - (8 + itemsLength))) {
 		kDebug() << "adjusting length";
-		textLength = length - (8 + itemsLength);
+		textLength = (getLength() - (8 + itemsLength));
 	}
 }
 
 DvbCableDescriptor::DvbCableDescriptor(const DvbDescriptor &descriptor) : DvbDescriptor(descriptor)
 {
-	if (length < 13) {
+	if (getLength() < 13) {
 		kDebug() << "invalid descriptor";
-		length = 0;
+		initSectionData();
 		return;
 	}
 }
 
 DvbSatelliteDescriptor::DvbSatelliteDescriptor(const DvbDescriptor &descriptor) : DvbDescriptor(descriptor)
 {
-	if (length < 13) {
+	if (getLength() < 13) {
 		kDebug() << "invalid descriptor";
-		length = 0;
+		initSectionData();
 		return;
 	}
 }
 
 DvbTerrestrialDescriptor::DvbTerrestrialDescriptor(const DvbDescriptor &descriptor) : DvbDescriptor(descriptor)
 {
-	if (length < 13) {
+	if (getLength() < 13) {
 		kDebug() << "invalid descriptor";
-		length = 0;
+		initSectionData();
 		return;
 	}
 }
 
 AtscChannelNameDescriptor::AtscChannelNameDescriptor(const DvbDescriptor &descriptor) : DvbDescriptor(descriptor)
 {
-	if (length < 2) {
+	if (getLength() < 2) {
 		kDebug() << "invalid descriptor";
-		length = 0;
+		initSectionData();
 		return;
 	}
 }
 
-DvbPatSectionEntry::DvbPatSectionEntry(const DvbSectionData &data_) : DvbSectionData(data_)
+void DvbPatSectionEntry::initPatSectionEntry(const char *data, int size)
 {
-	if (length < 4) {
-		if (length != 0) {
+	if (size < 4) {
+		if (size != 0) {
 			kDebug() << "invalid entry";
 		}
 
-		length = 0;
+		initSectionData();
 		return;
 	}
 
-	length = 4;
+	initSectionData(data, 4, size);
 }
 
-DvbPmtSectionEntry::DvbPmtSectionEntry(const DvbSectionData &data_) : DvbSectionData(data_)
+void DvbPmtSectionEntry::initPmtSectionEntry(const char *data, int size)
 {
-	if (length < 5) {
-		if (length != 0) {
+	if (size < 5) {
+		if (size != 0) {
 			kDebug() << "invalid entry";
 		}
 
-		length = 0;
+		initSectionData();
 		return;
 	}
 
-	int entryLength = (((at(3) & 0xf) << 8) | at(4)) + 5;
+	int entryLength = ((((static_cast<unsigned char>(data[3]) & 0xf) << 8) | static_cast<unsigned char>(data[4])) + 5);
 
-	if (entryLength <= length) {
-		length = entryLength;
-	} else {
+	if (entryLength > size) {
 		kDebug() << "adjusting length";
+		entryLength = size;
 	}
+
+	initSectionData(data, entryLength, size);
 }
 
-DvbSdtSectionEntry::DvbSdtSectionEntry(const DvbSectionData &data_) : DvbSectionData(data_)
+void DvbSdtSectionEntry::initSdtSectionEntry(const char *data, int size)
 {
-	if (length < 5) {
-		if (length != 0) {
+	if (size < 5) {
+		if (size != 0) {
 			kDebug() << "invalid entry";
 		}
 
-		length = 0;
+		initSectionData();
 		return;
 	}
 
-	int entryLength = (((at(3) & 0xf) << 8) | at(4)) + 5;
+	int entryLength = ((((static_cast<unsigned char>(data[3]) & 0xf) << 8) | static_cast<unsigned char>(data[4])) + 5);
 
-	if (entryLength <= length) {
-		length = entryLength;
-	} else {
+	if (entryLength > size) {
 		kDebug() << "adjusting length";
+		entryLength = size;
 	}
+
+	initSectionData(data, entryLength, size);
 }
 
-DvbEitSectionEntry::DvbEitSectionEntry(const DvbSectionData &data_) : DvbSectionData(data_)
+void DvbEitSectionEntry::initEitSectionEntry(const char *data, int size)
 {
-	if (length < 12) {
-		if (length != 0) {
+	if (size < 12) {
+		if (size != 0) {
 			kDebug() << "invalid entry";
 		}
 
-		length = 0;
+		initSectionData();
 		return;
 	}
 
-	int entryLength = (((at(10) & 0xf) << 8) | at(11)) + 12;
+	int entryLength = ((((static_cast<unsigned char>(data[10]) & 0xf) << 8) | static_cast<unsigned char>(data[11])) + 12);
 
-	if (entryLength <= length) {
-		length = entryLength;
-	} else {
+	if (entryLength > size) {
 		kDebug() << "adjusting length";
+		entryLength = size;
 	}
+
+	initSectionData(data, entryLength, size);
 }
 
-DvbNitSectionEntry::DvbNitSectionEntry(const DvbSectionData &data_) : DvbSectionData(data_)
+void DvbNitSectionEntry::initNitSectionEntry(const char *data, int size)
 {
-	if (length < 6) {
-		if (length != 0) {
+	if (size < 6) {
+		if (size != 0) {
 			kDebug() << "invalid entry";
 		}
 
-		length = 0;
+		initSectionData();
 		return;
 	}
 
-	int entryLength = (((at(4) & 0xf) << 8) | at(5)) + 6;
+	int entryLength = ((((static_cast<unsigned char>(data[4]) & 0xf) << 8) | static_cast<unsigned char>(data[5])) + 6);
 
-	if (entryLength <= length) {
-		length = entryLength;
-	} else {
+	if (entryLength > size) {
 		kDebug() << "adjusting length";
+		entryLength = size;
 	}
+
+	initSectionData(data, entryLength, size);
 }
 
-AtscVctSectionEntry::AtscVctSectionEntry(const DvbSectionData &data_) : DvbSectionData(data_)
+void AtscMgtSectionEntry::initMgtSectionEntry(const char *data, int size)
 {
-	if (length < 32) {
-		if (length != 0) {
+	if (size < 11) {
+		if (size != 0) {
 			kDebug() << "invalid entry";
 		}
 
-		length = 0;
+		initSectionData();
 		return;
 	}
 
-	int entryLength = (((at(30) & 0x3) << 8) | at(31)) + 32;
+	int entryLength = ((((static_cast<unsigned char>(data[9]) & 0xf) << 8) | static_cast<unsigned char>(data[10])) + 11);
 
-	if (entryLength <= length) {
-		length = entryLength;
-	} else {
+	if (entryLength > size) {
 		kDebug() << "adjusting length";
+		entryLength = size;
 	}
+
+	initSectionData(data, entryLength, size);
 }
 
-DvbPatSection::DvbPatSection(const QByteArray &data) : DvbStandardSection(data)
+void AtscVctSectionEntry::initVctSectionEntry(const char *data, int size)
 {
-	if (length < 12) {
-		length = 0;
-		return;
-	}
-}
+	if (size < 32) {
+		if (size != 0) {
+			kDebug() << "invalid entry";
+		}
 
-DvbPmtSection::DvbPmtSection(const QByteArray &data) : DvbStandardSection(data)
-{
-	if (length < 16) {
-		length = 0;
+		initSectionData();
 		return;
 	}
 
+	int entryLength = ((((static_cast<unsigned char>(data[30]) & 0x3) << 8) | static_cast<unsigned char>(data[31])) + 32);
+
+	if (entryLength > size) {
+		kDebug() << "adjusting length";
+		entryLength = size;
+	}
+
+	initSectionData(data, entryLength, size);
+}
+
+void DvbPatSection::initPatSection(const char *data, int size)
+{
+	if (size < 12) {
+		initSectionData();
+		return;
+	}
+
+	initStandardSection(data, size);
+}
+
+void DvbPmtSection::initPmtSection(const char *data, int size)
+{
+	if (size < 16) {
+		initSectionData();
+		return;
+	}
+
+	initStandardSection(data, size);
 	descriptorsLength = ((at(10) & 0xf) << 8) | at(11);
 
-	if (descriptorsLength > (length - 16)) {
+	if (descriptorsLength > (getLength() - 16)) {
 		kDebug() << "adjusting length";
-		descriptorsLength = length - 16;
+		descriptorsLength = (getLength() - 16);
 	}
 }
 
-DvbSdtSection::DvbSdtSection(const QByteArray &data) : DvbStandardSection(data)
+void DvbSdtSection::initSdtSection(const char *data, int size)
 {
-	if (length < 15) {
-		length = 0;
+	if (size < 15) {
+		initSectionData();
 		return;
 	}
+
+	initStandardSection(data, size);
 }
 
-DvbEitSection::DvbEitSection(const QByteArray &data) : DvbStandardSection(data)
+void DvbEitSection::initEitSection(const char *data, int size)
 {
-	if (length < 18) {
-		length = 0;
+	if (size < 18) {
+		initSectionData();
 		return;
 	}
+
+	initStandardSection(data, size);
 }
 
-DvbNitSection::DvbNitSection(const QByteArray &data) : DvbStandardSection(data)
+void DvbNitSection::initNitSection(const char *data, int size)
 {
-	if (length < 16) {
-		length = 0;
+	if (size < 16) {
+		initSectionData();
 		return;
 	}
 
+	initStandardSection(data, size);
 	descriptorsLength = ((at(8) & 0xf) << 8) | at(9);
 
-	if (descriptorsLength > (length - 16)) {
+	if (descriptorsLength > (getLength() - 16)) {
 		kDebug() << "adjusting length";
-		descriptorsLength = length - 16;
+		descriptorsLength = (getLength() - 16);
 	}
 
 	entriesLength = ((at(10 + descriptorsLength) & 0xf) << 8) | at(11 + descriptorsLength);
 
-	if (entriesLength > (length - (16 + descriptorsLength))) {
+	if (entriesLength > (getLength() - (16 + descriptorsLength))) {
 		kDebug() << "adjusting length";
-		entriesLength = length - (16 + descriptorsLength);
+		entriesLength = (getLength() - (16 + descriptorsLength));
 	}
 }
 
-AtscVctSection::AtscVctSection(const QByteArray &data) : DvbStandardSection(data)
+void AtscMgtSection::initMgtSection(const char *data, int size)
 {
-	if (length < 14) {
-		length = 0;
+	if (size < 15) {
+		initSectionData();
 		return;
 	}
+
+	initStandardSection(data, size);
+}
+
+void AtscVctSection::initVctSection(const char *data, int size)
+{
+	if (size < 14) {
+		initSectionData();
+		return;
+	}
+
+	initStandardSection(data, size);
+}
+
+void AtscEitSection::initEitSection(const char *data, int size)
+{
+	if (size < 14) {
+		initSectionData();
+		return;
+	}
+
+	initStandardSection(data, size);
+}
+
+void AtscEttSection::initEttSection(const char *data, int size)
+{
+	if (size < 17) {
+		initSectionData();
+		return;
+	}
+
+	initStandardSection(data, size);
 }
