@@ -1,7 +1,7 @@
 /*
  * dvbsi.h
  *
- * Copyright (C) 2008-2009 Christoph Pfister <christophpfister@gmail.com>
+ * Copyright (C) 2008-2010 Christoph Pfister <christophpfister@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,47 +22,18 @@
 #define DVBSI_H
 
 #include <QPair>
-#include "dvbdevice.h"
+#include "dvbbackenddevice.h"
 
-class DvbSectionFilter : public DvbPidFilter
-{
-public:
-	DvbSectionFilter() : continuityCounter(-1), bufferValid(false) { }
-	~DvbSectionFilter() { }
-
-	void resetFilter()
-	{
-		buffer.clear();
-		continuityCounter = -1;
-		bufferValid = false;
-	}
-
-protected:
-	virtual void processSection(const QByteArray &data) = 0;
-
-private:
-	void processData(const char data[188]);
-
-	void appendData(const char *data, int length);
-	void processSections(bool force = false);
-
-	QByteArray buffer;
-	int continuityCounter;
-	bool bufferValid;
-};
+class DvbPmtSection;
 
 class DvbSectionData
 {
 public:
-	explicit DvbSectionData(const QByteArray &byteArray_) : byteArray(byteArray_)
+	bool isValid() const
 	{
-		data = byteArray.constData();
-		size = byteArray.size();
-		length = size;
+		return (length > 0);
 	}
 
-	~DvbSectionData() { }
-
 	unsigned char at(int index) const
 	{
 		return data[index];
@@ -78,50 +49,46 @@ public:
 		return length;
 	}
 
-	bool isValid() const
-	{
-		return (length > 0);
-	}
-
 	QByteArray toByteArray() const
 	{
 		return QByteArray(data, length);
 	}
 
 protected:
-	DvbSectionData next() const
+	DvbSectionData() { }
+	~DvbSectionData() { }
+
+	void initSectionData()
 	{
-		return DvbSectionData(byteArray, data + length, size - length);
+		data = NULL;
+		length = 0;
+		size = 0;
 	}
 
-	DvbSectionData subArray(int index, int size) const
+	void initSectionData(const char *data_, int length_, int size_)
 	{
-		return DvbSectionData(byteArray, data + index, size);
+		data = data_;
+		length = length_;
+		size = size_;
 	}
 
-	int length;
+	int getSize() const
+	{
+		return size;
+	}
 
 private:
-	DvbSectionData(const QByteArray &byteArray_, const char *data_, int size_) :
-		size(size_), data(data_), byteArray(byteArray_)
-	{
-		length = size;
-	}
-
-	int size;
 	const char *data;
-	QByteArray byteArray;
+	int length;
+	int size;
 };
 
 class DvbSection : public DvbSectionData
 {
 public:
-	explicit DvbSection(const QByteArray &data);
-	~DvbSection() { }
-
 	int getSectionLength() const
 	{
-		return length;
+		return getLength();
 	}
 
 	int tableId() const
@@ -133,20 +100,20 @@ public:
 	{
 		return (at(1) & 0x80) != 0;
 	}
+
+protected:
+	DvbSection() { }
+	~DvbSection() { }
+
+	void initSection(const char *data, int size);
+
+private:
+	Q_DISABLE_COPY(DvbSection)
 };
 
 class DvbStandardSection : public DvbSection
 {
 public:
-	explicit DvbStandardSection(const QByteArray &data) : DvbSection(data)
-	{
-		if (length < 12) {
-			length = 0;
-		}
-	}
-
-	~DvbStandardSection() { }
-
 	int versionNumber() const
 	{
 		return (at(5) >> 1) & ((1 << 5) - 1);
@@ -167,15 +134,23 @@ public:
 		return at(7);
 	}
 
-	int verifyCrc32() const;
-
+	static int verifyCrc32(const char *data, int size);
 	static const unsigned int crc32Table[];
+
+protected:
+	DvbStandardSection() { }
+	~DvbStandardSection() { }
+
+	void initStandardSection(const char *data, int size);
+
+private:
+	Q_DISABLE_COPY(DvbStandardSection)
 };
 
 class DvbSiText
 {
 public:
-	static QString convertText(const DvbSectionData &text);
+	static QString convertText(const char *data, int size);
 	static void setOverride6937(bool override);
 
 private:
@@ -193,15 +168,13 @@ private:
 		Iso8859_9	=  9,
 		Iso8859_10	= 10,
 		Iso8859_11	= 11,
-		Iso8859_12	= 12,
-		Iso8859_13	= 13,
-		Iso8859_14	= 14,
-		Iso8859_15	= 15,
-		Gb2312		= 16,
-		Big5		= 17,
-		Utf_8		= 18,
-
-		EncodingTypeMax	= 18
+		Iso8859_13	= 12,
+		Iso8859_14	= 13,
+		Iso8859_15	= 14,
+		Gb2312		= 15,
+		Big5		= 16,
+		Utf_8		= 17,
+		EncodingTypeMax	= 17
 	};
 
 	static QTextCodec *codecTable[EncodingTypeMax + 1];
@@ -211,7 +184,11 @@ private:
 class DvbDescriptor : public DvbSectionData
 {
 public:
-	explicit DvbDescriptor(const DvbSectionData &data);
+	DvbDescriptor(const char *data, int size)
+	{
+		initDescriptor(data, size);
+	}
+
 	~DvbDescriptor() { }
 
 	int descriptorTag() const
@@ -221,7 +198,7 @@ public:
 
 	void advance()
 	{
-		*this = DvbDescriptor(next());
+		initDescriptor(getData() + getLength(), getSize() - getLength());
 	}
 
 	static int bcdToInt(unsigned int bcd, int multiplier)
@@ -236,13 +213,17 @@ public:
 
 		return value;
 	}
+
+private:
+	void initDescriptor(const char *data, int size);
 };
 
 // ATSC "Multiple String Structure".  See A/65C Section 6.10
 class AtscPsipText
 {
 public:
-	static QString convertText(const DvbSectionData &text);
+	static QString convertText(const char *data, int size);
+
 private:
 	static QString interpretTextData(const char *data, unsigned int len,
 					 unsigned int mode);
@@ -275,6 +256,28 @@ private:
 	static const unsigned char Huffman2Tables[];
 };
 
+class DvbPmtFilter : public QObject, public DvbSectionFilter
+{
+	Q_OBJECT
+public:
+	DvbPmtFilter() : programNumber(-1) { }
+	~DvbPmtFilter() { }
+
+	void setProgramNumber(int programNumber_)
+	{
+		programNumber = programNumber_;
+	}
+
+signals:
+	void pmtSectionChanged(const QByteArray &pmtSectionData);
+
+private:
+	void processSection(const char *data, int size);
+
+	int programNumber;
+	QByteArray lastPmtSectionData;
+};
+
 class DvbSectionGenerator
 {
 public:
@@ -302,35 +305,6 @@ private:
 	int continuityCounter;
 };
 
-class DvbPmtFilter : public QObject, public DvbSectionFilter
-{
-	Q_OBJECT
-public:
-	DvbPmtFilter() : programNumber(-1), versionNumber(-1) { }
-	~DvbPmtFilter() { }
-
-	void reset()
-	{
-		DvbSectionFilter::resetFilter();
-		programNumber = -1;
-		versionNumber = -1;
-	}
-
-	void setProgramNumber(int programNumber_)
-	{
-		programNumber = programNumber_;
-	}
-
-signals:
-	void pmtSectionChanged(const DvbPmtSection &section);
-
-private:
-	void processSection(const QByteArray &data);
-
-	int programNumber;
-	int versionNumber;
-};
-
 class DvbPmtParser
 {
 public:
@@ -343,6 +317,47 @@ public:
 	int teletextPid;
 };
 
+class AtscEitSectionEntry : public DvbSectionData
+{
+public:
+	AtscEitSectionEntry(const char *data, int size)
+	{
+		initEitSectionEntry(data, size);
+	}
+
+	~AtscEitSectionEntry() { }
+
+	void advance()
+	{
+		initEitSectionEntry(getData() + getLength(), getSize() - getLength());
+	}
+
+	int eventId() const
+	{
+		return ((at(0) & 0x3f) << 8) | at(1);
+	}
+
+	int startTime() const
+	{
+		return (at(2) << 24) | (at(3) << 16) | (at(4) << 8) | at(5);
+	}
+
+	int duration() const
+	{
+		return ((at(6) & 0xf) << 16) | (at(7) << 8) | at(8);
+	}
+
+	QString title() const
+	{
+		return AtscPsipText::convertText(getData() + 10, titleLength);
+	}
+
+private:
+	void initEitSectionEntry(const char *data, int size);
+
+	int titleLength;
+};
+
 // everything below this line is automatically generated
 
 class DvbLanguageDescriptor : public DvbDescriptor
@@ -365,6 +380,9 @@ public:
 	{
 		return at(4);
 	}
+
+private:
+	Q_DISABLE_COPY(DvbLanguageDescriptor)
 };
 
 class DvbSubtitleDescriptor : public DvbDescriptor
@@ -392,6 +410,9 @@ public:
 	{
 		return at(5);
 	}
+
+private:
+	Q_DISABLE_COPY(DvbSubtitleDescriptor)
 };
 
 class DvbServiceDescriptor : public DvbDescriptor
@@ -402,15 +423,17 @@ public:
 
 	QString providerName() const
 	{
-		return DvbSiText::convertText(subArray(4, providerNameLength));
+		return DvbSiText::convertText(getData() + 4, providerNameLength);
 	}
 
 	QString serviceName() const
 	{
-		return DvbSiText::convertText(subArray(5 + providerNameLength, serviceNameLength));
+		return DvbSiText::convertText(getData() + 5 + providerNameLength, serviceNameLength);
 	}
 
 private:
+	Q_DISABLE_COPY(DvbServiceDescriptor)
+
 	int providerNameLength;
 	int serviceNameLength;
 };
@@ -423,15 +446,17 @@ public:
 
 	QString eventName() const
 	{
-		return DvbSiText::convertText(subArray(6, eventNameLength));
+		return DvbSiText::convertText(getData() + 6, eventNameLength);
 	}
 
 	QString text() const
 	{
-		return DvbSiText::convertText(subArray(7 + eventNameLength, textLength));
+		return DvbSiText::convertText(getData() + 7 + eventNameLength, textLength);
 	}
 
 private:
+	Q_DISABLE_COPY(DvbShortEventDescriptor)
+
 	int eventNameLength;
 	int textLength;
 };
@@ -444,10 +469,12 @@ public:
 
 	QString text() const
 	{
-		return DvbSiText::convertText(subArray(8 + itemsLength, textLength));
+		return DvbSiText::convertText(getData() + 8 + itemsLength, textLength);
 	}
 
 private:
+	Q_DISABLE_COPY(DvbExtendedEventDescriptor)
+
 	int itemsLength;
 	int textLength;
 };
@@ -477,6 +504,9 @@ public:
 	{
 		return (at(12) & 0xf);
 	}
+
+private:
+	Q_DISABLE_COPY(DvbCableDescriptor)
 };
 
 class DvbSatelliteDescriptor : public DvbDescriptor
@@ -519,6 +549,9 @@ public:
 	{
 		return (at(12) & 0xf);
 	}
+
+private:
+	Q_DISABLE_COPY(DvbSatelliteDescriptor)
 };
 
 class DvbTerrestrialDescriptor : public DvbDescriptor
@@ -566,6 +599,9 @@ public:
 	{
 		return ((at(8) & 0x7) >> 1);
 	}
+
+private:
+	Q_DISABLE_COPY(DvbTerrestrialDescriptor)
 };
 
 class AtscChannelNameDescriptor : public DvbDescriptor
@@ -576,19 +612,26 @@ public:
 
 	QString name() const
 	{
-		return AtscPsipText::convertText(subArray(2, length - 2));
+		return AtscPsipText::convertText(getData() + 2, getLength() - 2);
 	}
+
+private:
+	Q_DISABLE_COPY(AtscChannelNameDescriptor)
 };
 
 class DvbPatSectionEntry : public DvbSectionData
 {
 public:
-	explicit DvbPatSectionEntry(const DvbSectionData &data_);
+	DvbPatSectionEntry(const char *data, int size)
+	{
+		initPatSectionEntry(data, size);
+	}
+
 	~DvbPatSectionEntry() { }
 
 	void advance()
 	{
-		*this = DvbPatSectionEntry(next());
+		initPatSectionEntry(getData() + getLength(), getSize() - getLength());
 	}
 
 	int programNumber() const
@@ -600,17 +643,24 @@ public:
 	{
 		return ((at(2) & 0x1f) << 8) | at(3);
 	}
+
+private:
+	void initPatSectionEntry(const char *data, int size);
 };
 
 class DvbPmtSectionEntry : public DvbSectionData
 {
 public:
-	explicit DvbPmtSectionEntry(const DvbSectionData &data_);
+	DvbPmtSectionEntry(const char *data, int size)
+	{
+		initPmtSectionEntry(data, size);
+	}
+
 	~DvbPmtSectionEntry() { }
 
 	void advance()
 	{
-		*this = DvbPmtSectionEntry(next());
+		initPmtSectionEntry(getData() + getLength(), getSize() - getLength());
 	}
 
 	int streamType() const
@@ -625,19 +675,26 @@ public:
 
 	DvbDescriptor descriptors() const
 	{
-		return DvbDescriptor(subArray(5, length - 5));
+		return DvbDescriptor(getData() + 5, getLength() - 5);
 	}
+
+private:
+	void initPmtSectionEntry(const char *data, int size);
 };
 
 class DvbSdtSectionEntry : public DvbSectionData
 {
 public:
-	explicit DvbSdtSectionEntry(const DvbSectionData &data_);
+	DvbSdtSectionEntry(const char *data, int size)
+	{
+		initSdtSectionEntry(data, size);
+	}
+
 	~DvbSdtSectionEntry() { }
 
 	void advance()
 	{
-		*this = DvbSdtSectionEntry(next());
+		initSdtSectionEntry(getData() + getLength(), getSize() - getLength());
 	}
 
 	int serviceId() const
@@ -652,19 +709,26 @@ public:
 
 	DvbDescriptor descriptors() const
 	{
-		return DvbDescriptor(subArray(5, length - 5));
+		return DvbDescriptor(getData() + 5, getLength() - 5);
 	}
+
+private:
+	void initSdtSectionEntry(const char *data, int size);
 };
 
 class DvbEitSectionEntry : public DvbSectionData
 {
 public:
-	explicit DvbEitSectionEntry(const DvbSectionData &data_);
+	DvbEitSectionEntry(const char *data, int size)
+	{
+		initEitSectionEntry(data, size);
+	}
+
 	~DvbEitSectionEntry() { }
 
 	void advance()
 	{
-		*this = DvbEitSectionEntry(next());
+		initEitSectionEntry(getData() + getLength(), getSize() - getLength());
 	}
 
 	int startDate() const
@@ -684,36 +748,84 @@ public:
 
 	DvbDescriptor descriptors() const
 	{
-		return DvbDescriptor(subArray(12, length - 12));
+		return DvbDescriptor(getData() + 12, getLength() - 12);
 	}
+
+private:
+	void initEitSectionEntry(const char *data, int size);
 };
 
 class DvbNitSectionEntry : public DvbSectionData
 {
 public:
-	explicit DvbNitSectionEntry(const DvbSectionData &data_);
+	DvbNitSectionEntry(const char *data, int size)
+	{
+		initNitSectionEntry(data, size);
+	}
+
 	~DvbNitSectionEntry() { }
 
 	void advance()
 	{
-		*this = DvbNitSectionEntry(next());
+		initNitSectionEntry(getData() + getLength(), getSize() - getLength());
 	}
 
 	DvbDescriptor descriptors() const
 	{
-		return DvbDescriptor(subArray(6, length - 6));
+		return DvbDescriptor(getData() + 6, getLength() - 6);
 	}
+
+private:
+	void initNitSectionEntry(const char *data, int size);
+};
+
+class AtscMgtSectionEntry : public DvbSectionData
+{
+public:
+	AtscMgtSectionEntry(const char *data, int size)
+	{
+		initMgtSectionEntry(data, size);
+	}
+
+	~AtscMgtSectionEntry() { }
+
+	void advance()
+	{
+		initMgtSectionEntry(getData() + getLength(), getSize() - getLength());
+	}
+
+	int tableType() const
+	{
+		return (at(0) << 8) | at(1);
+	}
+
+	int pid() const
+	{
+		return ((at(2) & 0x1f) << 8) | at(3);
+	}
+
+	DvbDescriptor descriptors() const
+	{
+		return DvbDescriptor(getData() + 11, getLength() - 11);
+	}
+
+private:
+	void initMgtSectionEntry(const char *data, int size);
 };
 
 class AtscVctSectionEntry : public DvbSectionData
 {
 public:
-	explicit AtscVctSectionEntry(const DvbSectionData &data_);
+	AtscVctSectionEntry(const char *data, int size)
+	{
+		initVctSectionEntry(data, size);
+	}
+
 	~AtscVctSectionEntry() { }
 
 	void advance()
 	{
-		*this = AtscVctSectionEntry(next());
+		initVctSectionEntry(getData() + getLength(), getSize() - getLength());
 	}
 
 	int shortName1() const
@@ -778,14 +890,26 @@ public:
 
 	DvbDescriptor descriptors() const
 	{
-		return DvbDescriptor(subArray(32, length - 32));
+		return DvbDescriptor(getData() + 32, getLength() - 32);
 	}
+
+private:
+	void initVctSectionEntry(const char *data, int size);
 };
 
 class DvbPatSection : public DvbStandardSection
 {
 public:
-	explicit DvbPatSection(const QByteArray &data);
+	DvbPatSection(const char *data, int size)
+	{
+		initPatSection(data, size);
+	}
+
+	explicit DvbPatSection(const QByteArray &byteArray)
+	{
+		initPatSection(byteArray.constData(), byteArray.size());
+	}
+
 	~DvbPatSection() { }
 
 	int transportStreamId() const
@@ -795,14 +919,27 @@ public:
 
 	DvbPatSectionEntry entries() const
 	{
-		return DvbPatSectionEntry(subArray(8, length - 12));
+		return DvbPatSectionEntry(getData() + 8, getLength() - 12);
 	}
+
+private:
+	Q_DISABLE_COPY(DvbPatSection)
+	void initPatSection(const char *data, int size);
 };
 
 class DvbPmtSection : public DvbStandardSection
 {
 public:
-	explicit DvbPmtSection(const QByteArray &data);
+	DvbPmtSection(const char *data, int size)
+	{
+		initPmtSection(data, size);
+	}
+
+	explicit DvbPmtSection(const QByteArray &byteArray)
+	{
+		initPmtSection(byteArray.constData(), byteArray.size());
+	}
+
 	~DvbPmtSection() { }
 
 	int programNumber() const
@@ -812,22 +949,34 @@ public:
 
 	DvbDescriptor descriptors() const
 	{
-		return DvbDescriptor(subArray(12, descriptorsLength));
+		return DvbDescriptor(getData() + 12, descriptorsLength);
 	}
 
 	DvbPmtSectionEntry entries() const
 	{
-		return DvbPmtSectionEntry(subArray(12 + descriptorsLength, length - (16 + descriptorsLength)));
+		return DvbPmtSectionEntry(getData() + 12 + descriptorsLength, getLength() - (16 + descriptorsLength));
 	}
 
 private:
+	Q_DISABLE_COPY(DvbPmtSection)
+	void initPmtSection(const char *data, int size);
+
 	int descriptorsLength;
 };
 
 class DvbSdtSection : public DvbStandardSection
 {
 public:
-	explicit DvbSdtSection(const QByteArray &data);
+	DvbSdtSection(const char *data, int size)
+	{
+		initSdtSection(data, size);
+	}
+
+	explicit DvbSdtSection(const QByteArray &byteArray)
+	{
+		initSdtSection(byteArray.constData(), byteArray.size());
+	}
+
 	~DvbSdtSection() { }
 
 	int originalNetworkId() const
@@ -837,14 +986,27 @@ public:
 
 	DvbSdtSectionEntry entries() const
 	{
-		return DvbSdtSectionEntry(subArray(11, length - 15));
+		return DvbSdtSectionEntry(getData() + 11, getLength() - 15);
 	}
+
+private:
+	Q_DISABLE_COPY(DvbSdtSection)
+	void initSdtSection(const char *data, int size);
 };
 
 class DvbEitSection : public DvbStandardSection
 {
 public:
-	explicit DvbEitSection(const QByteArray &data);
+	DvbEitSection(const char *data, int size)
+	{
+		initEitSection(data, size);
+	}
+
+	explicit DvbEitSection(const QByteArray &byteArray)
+	{
+		initEitSection(byteArray.constData(), byteArray.size());
+	}
+
 	~DvbEitSection() { }
 
 	int serviceId() const
@@ -864,35 +1026,90 @@ public:
 
 	DvbEitSectionEntry entries() const
 	{
-		return DvbEitSectionEntry(subArray(14, length - 18));
+		return DvbEitSectionEntry(getData() + 14, getLength() - 18);
 	}
+
+private:
+	Q_DISABLE_COPY(DvbEitSection)
+	void initEitSection(const char *data, int size);
 };
 
 class DvbNitSection : public DvbStandardSection
 {
 public:
-	explicit DvbNitSection(const QByteArray &data);
+	DvbNitSection(const char *data, int size)
+	{
+		initNitSection(data, size);
+	}
+
+	explicit DvbNitSection(const QByteArray &byteArray)
+	{
+		initNitSection(byteArray.constData(), byteArray.size());
+	}
+
 	~DvbNitSection() { }
 
 	DvbDescriptor descriptors() const
 	{
-		return DvbDescriptor(subArray(10, descriptorsLength));
+		return DvbDescriptor(getData() + 10, descriptorsLength);
 	}
 
 	DvbNitSectionEntry entries() const
 	{
-		return DvbNitSectionEntry(subArray(12 + descriptorsLength, entriesLength));
+		return DvbNitSectionEntry(getData() + 12 + descriptorsLength, entriesLength);
 	}
 
 private:
+	Q_DISABLE_COPY(DvbNitSection)
+	void initNitSection(const char *data, int size);
+
 	int descriptorsLength;
 	int entriesLength;
 };
 
+class AtscMgtSection : public DvbStandardSection
+{
+public:
+	AtscMgtSection(const char *data, int size)
+	{
+		initMgtSection(data, size);
+	}
+
+	explicit AtscMgtSection(const QByteArray &byteArray)
+	{
+		initMgtSection(byteArray.constData(), byteArray.size());
+	}
+
+	~AtscMgtSection() { }
+
+	int entryCount() const
+	{
+		return (at(9) << 8) | at(10);
+	}
+
+	AtscMgtSectionEntry entries() const
+	{
+		return AtscMgtSectionEntry(getData() + 11, getLength() - 15);
+	}
+
+private:
+	Q_DISABLE_COPY(AtscMgtSection)
+	void initMgtSection(const char *data, int size);
+};
+
 class AtscVctSection : public DvbStandardSection
 {
 public:
-	explicit AtscVctSection(const QByteArray &data);
+	AtscVctSection(const char *data, int size)
+	{
+		initVctSection(data, size);
+	}
+
+	explicit AtscVctSection(const QByteArray &byteArray)
+	{
+		initVctSection(byteArray.constData(), byteArray.size());
+	}
+
 	~AtscVctSection() { }
 
 	int entryCount() const
@@ -902,8 +1119,87 @@ public:
 
 	AtscVctSectionEntry entries() const
 	{
-		return AtscVctSectionEntry(subArray(10, length - 14));
+		return AtscVctSectionEntry(getData() + 10, getLength() - 14);
 	}
+
+private:
+	Q_DISABLE_COPY(AtscVctSection)
+	void initVctSection(const char *data, int size);
+};
+
+class AtscEitSection : public DvbStandardSection
+{
+public:
+	AtscEitSection(const char *data, int size)
+	{
+		initEitSection(data, size);
+	}
+
+	explicit AtscEitSection(const QByteArray &byteArray)
+	{
+		initEitSection(byteArray.constData(), byteArray.size());
+	}
+
+	~AtscEitSection() { }
+
+	int sourceId() const
+	{
+		return (at(3) << 8) | at(4);
+	}
+
+	int entryCount() const
+	{
+		return at(9);
+	}
+
+	AtscEitSectionEntry entries() const
+	{
+		return AtscEitSectionEntry(getData() + 10, getLength() - 14);
+	}
+
+private:
+	Q_DISABLE_COPY(AtscEitSection)
+	void initEitSection(const char *data, int size);
+};
+
+class AtscEttSection : public DvbStandardSection
+{
+public:
+	AtscEttSection(const char *data, int size)
+	{
+		initEttSection(data, size);
+	}
+
+	explicit AtscEttSection(const QByteArray &byteArray)
+	{
+		initEttSection(byteArray.constData(), byteArray.size());
+	}
+
+	~AtscEttSection() { }
+
+	int sourceId() const
+	{
+		return (at(9) << 8) | at(10);
+	}
+
+	int eventId() const
+	{
+		return (at(11) << 6) | (at(12) >> 2);
+	}
+
+	int messageType() const
+	{
+		return (at(12) & 0x3);
+	}
+
+	QString text() const
+	{
+		return AtscPsipText::convertText(getData() + 13, getLength() - 17);
+	}
+
+private:
+	Q_DISABLE_COPY(AtscEttSection)
+	void initEttSection(const char *data, int size);
 };
 
 #endif /* DVBSI_H */
