@@ -20,7 +20,6 @@
 
 #include "dvbtab.h"
 
-#include <QAbstractProxyModel>
 #include <QBoxLayout>
 #include <QDir>
 #include <QHeaderView>
@@ -33,13 +32,16 @@
 #include <KLineEdit>
 #include <KLocale>
 #include <KMenu>
+#include <KMessageBox>
 #include "../mediawidget.h"
 #include "../osdwidget.h"
-#include "dvbchannelui.h"
+#include "dvbchanneldialog.h"
 #include "dvbconfigdialog.h"
 #include "dvbepg.h"
+#include "dvbepgdialog.h"
 #include "dvbliveview.h"
 #include "dvbmanager.h"
+#include "dvbrecordingdialog.h"
 #include "dvbscandialog.h"
 
 class DvbTimeShiftCleaner : public QThread
@@ -98,7 +100,7 @@ DvbTab::DvbTab(KMenu *menu, KActionCollection *collection, MediaWidget *mediaWid
 	menu->addAction(collection->addAction("dvb_osd", osdAction));
 
 	KAction *recordingsAction = new KAction(KIcon("view-pim-calendar"),
-						i18nc("dialog", "Recording Schedule"), this);
+		i18nc("dialog", "Recording Schedule"), this);
 	recordingsAction->setShortcut(Qt::Key_R);
 	connect(recordingsAction, SIGNAL(triggered(bool)), this, SLOT(showRecordingDialog()));
 	menu->addAction(collection->addAction("dvb_recordings", recordingsAction));
@@ -112,15 +114,16 @@ DvbTab::DvbTab(KMenu *menu, KActionCollection *collection, MediaWidget *mediaWid
 
 	menu->addSeparator();
 
-	KAction *configureAction = new KAction(KIcon("configure"), i18n("Configure Television"), this);
-	connect(configureAction, SIGNAL(triggered(bool)), this, SLOT(configureDvb()));
+	KAction *configureAction = new KAction(KIcon("configure"),
+		i18nc("@action:inmenu", "Configure Television..."), this);
+	connect(configureAction, SIGNAL(triggered()), this, SLOT(configureDvb()));
 	menu->addAction(collection->addAction("settings_dvb", configureAction));
 
 	connect(mediaWidget, SIGNAL(previousDvbChannel()), this, SLOT(previousChannel()));
 	connect(mediaWidget, SIGNAL(nextDvbChannel()), this, SLOT(nextChannel()));
 
-	connect(manager->getRecordingModel(), SIGNAL(programRemoved(DvbRecordingKey)),
-		this, SLOT(programRemoved(DvbRecordingKey)));
+	connect(manager->getRecordingModel(), SIGNAL(recordingRemoved(DvbSharedRecording)),
+		this, SLOT(recordingRemoved(DvbSharedRecording)));
 
 	QBoxLayout *boxLayout = new QHBoxLayout(this);
 	boxLayout->setMargin(0);
@@ -141,7 +144,7 @@ DvbTab::DvbTab(KMenu *menu, KActionCollection *collection, MediaWidget *mediaWid
 
 	channelView = new DvbChannelView(leftWidget);
 	channelView->setContextMenuPolicy(Qt::ActionsContextMenu);
-	channelProxyModel = manager->getChannelModel()->createProxyModel(channelView);
+	channelProxyModel = new DvbChannelTableModel(this);
 	channelView->setModel(channelProxyModel);
 	channelView->setRootIsDecorated(false);
 
@@ -153,8 +156,9 @@ DvbTab::DvbTab(KMenu *menu, KActionCollection *collection, MediaWidget *mediaWid
 	channelView->setSortingEnabled(true);
 	channelView->addEditAction();
 	connect(channelView, SIGNAL(activated(QModelIndex)), this, SLOT(playChannel(QModelIndex)));
+	channelProxyModel->setChannelModel(manager->getChannelModel());
 	connect(lineEdit, SIGNAL(textChanged(QString)),
-		channelProxyModel, SLOT(setFilterRegExp(QString)));
+		channelProxyModel, SLOT(setFilter(QString)));
 	manager->setChannelView(channelView);
 	leftLayout->addWidget(channelView);
 
@@ -223,37 +227,32 @@ DvbTab::~DvbTab()
 void DvbTab::playChannel(const QString &nameOrNumber)
 {
 	DvbChannelModel *channelModel = manager->getChannelModel();
-	QModelIndex index;
+	DvbSharedChannel channel;
 	int number = nameOrNumber.toInt();
 
 	if (number > 0) {
-		index = channelModel->findChannelByNumber(number);
+		channel = channelModel->findChannelByNumber(number);
 	}
 
-	if (!index.isValid()) {
-		index = channelModel->findChannelByName(nameOrNumber);
+	if (!channel.isValid()) {
+		channel = channelModel->findChannelByName(nameOrNumber);
 	}
 
-	if (index.isValid()) {
-		const DvbChannel *channel = channelModel->data(index,
-			DvbChannelModel::DvbChannelRole).value<const DvbChannel *>();
-		playChannel(channel, channelProxyModel->mapFromSource(index));
+	if (channel.isValid()) {
+		playChannel(channel, channelProxyModel->find(channel));
 	}
 }
 
 void DvbTab::playLastChannel()
 {
-	if ((manager->getLiveView()->getChannel() == NULL) && !currentChannel.isEmpty()) {
+	if (!manager->getLiveView()->getChannel().isValid() && !currentChannel.isEmpty()) {
 		lastChannel = currentChannel;
 	}
 
-	DvbChannelModel *channelModel = manager->getChannelModel();
-	QModelIndex index = channelModel->findChannelByName(lastChannel);
+	DvbSharedChannel channel = manager->getChannelModel()->findChannelByName(lastChannel);
 
-	if (index.isValid()) {
-		const DvbChannel *channel = channelModel->data(index,
-			DvbChannelModel::DvbChannelRole).value<const DvbChannel *>();
-		playChannel(channel, channelProxyModel->mapFromSource(index));
+	if (channel.isValid()) {
+		playChannel(channel, channelProxyModel->find(channel));
 	}
 }
 
@@ -277,13 +276,38 @@ void DvbTab::osdKeyPressed(int key)
 	if ((key >= Qt::Key_0) && (key <= Qt::Key_9)) {
 		osdChannel += QString::number(key - Qt::Key_0);
 		osdChannelTimer.start(1500);
-		mediaWidget->getOsdWidget()->showText(i18nc("osd", "Channel: %1_", osdChannel), 1500);
+		mediaWidget->getOsdWidget()->showText(i18nc("osd", "Channel: %1_", osdChannel),
+			1500);
 	}
 }
 
 void DvbTab::mayCloseApplication(bool *ok, QWidget *parent)
 {
-	manager->getRecordingModel()->mayCloseApplication(ok, parent);
+	if (*ok) {
+		DvbRecordingModel *recordingModel = manager->getRecordingModel();
+
+		if (recordingModel->hasActiveRecordings()) {
+			if (KMessageBox::warningYesNo(parent, i18nc("message box",
+			    "Kaffeine is currently recording programs.\n"
+			    "Do you really want to close the application?")) != KMessageBox::Yes) {
+				*ok = false;
+			}
+
+			return;
+		}
+
+		if (recordingModel->hasRecordings()) {
+			if (KMessageBox::questionYesNo(parent, i18nc("message box",
+			    "Kaffeine has scheduled recordings.\n"
+			    "Do you really want to close the application?"), QString(),
+			    KStandardGuiItem::yes(), KStandardGuiItem::no(),
+			    "ScheduledRecordings") != KMessageBox::Yes) {
+				*ok = false;
+			}
+
+			return;
+		}
+	}
 }
 
 void DvbTab::showChannelDialog()
@@ -296,20 +320,14 @@ void DvbTab::showChannelDialog()
 
 void DvbTab::showRecordingDialog()
 {
-	manager->getRecordingModel()->showDialog(this);
+	DvbRecordingDialog::showDialog(manager, this);
 }
 
 void DvbTab::showEpgDialog()
 {
-	const DvbChannel *channel = manager->getLiveView()->getChannel();
-	QString channelName;
-
-	if (channel != NULL) {
-		channelName = channel->name;
-	}
-
-	KDialog *dialog = new DvbEpgDialog(manager, channelName, this);
+	DvbEpgDialog *dialog = new DvbEpgDialog(manager, this);
 	dialog->setAttribute(Qt::WA_DeleteOnClose, true);
+	dialog->setCurrentChannel(manager->getLiveView()->getChannel());
 	dialog->setModal(true);
 	dialog->show();
 }
@@ -317,34 +335,46 @@ void DvbTab::showEpgDialog()
 void DvbTab::instantRecord(bool checked)
 {
 	if (checked) {
-		const DvbChannel *channel = manager->getLiveView()->getChannel();
+		const DvbSharedChannel &channel = manager->getLiveView()->getChannel();
 
-		if (channel == NULL) {
+		if (!channel.isValid()) {
 			instantRecordAction->setChecked(false);
 			return;
 		}
 
-		DvbRecordingEntry recordingEntry;
-		// FIXME use epg for name
-		recordingEntry.name = (channel->name + QTime::currentTime().toString("-hhmmss"));
-		recordingEntry.channelName = channel->name;
-		recordingEntry.begin = QDateTime::currentDateTime().toUTC();
-		recordingEntry.duration = QTime(12, 0);
-		instantRecordingKey =
-			manager->getRecordingModel()->scheduleProgram(recordingEntry);
-		mediaWidget->getOsdWidget()->showText(i18nc("osd", "Instant Record Started"), 1500);
+		DvbRecording recording;
+		QList<DvbSharedEpgEntry> epgEntries =
+			manager->getEpgModel()->getCurrentNext(channel);
+
+		if (!epgEntries.isEmpty()) {
+			recording.name = epgEntries.at(0)->title;
+		}
+
+		if (recording.name.isEmpty()) {
+			recording.name =
+				(channel->name + QTime::currentTime().toString("-hhmmss"));
+		}
+
+		recording.channel = channel;
+		recording.begin = QDateTime::currentDateTime().toUTC();
+		recording.duration = QTime(12, 0);
+		instantRecording = manager->getRecordingModel()->addRecording(recording);
+		mediaWidget->getOsdWidget()->showText(i18nc("osd", "Instant Record Started"),
+			1500);
 	} else {
-		manager->getRecordingModel()->removeProgram(instantRecordingKey);
-		mediaWidget->getOsdWidget()->showText(i18nc("osd", "Instant Record Stopped"), 1500);
+		manager->getRecordingModel()->removeRecording(instantRecording);
+		mediaWidget->getOsdWidget()->showText(i18nc("osd", "Instant Record Stopped"),
+			1500);
 	}
 }
 
-void DvbTab::programRemoved(const DvbRecordingKey &recordingKey)
+void DvbTab::recordingRemoved(const DvbSharedRecording &recording)
 {
-	if (instantRecordingKey == recordingKey) {
-		instantRecordingKey = DvbRecordingKey();
+	if (instantRecording == recording) {
+		instantRecording = DvbSharedRecording();
 		instantRecordAction->setChecked(false);
-		mediaWidget->getOsdWidget()->showText(i18nc("osd", "Instant Record Stopped"), 1500);
+		mediaWidget->getOsdWidget()->showText(i18nc("osd", "Instant Record Stopped"),
+			1500);
 	}
 }
 
@@ -362,22 +392,17 @@ void DvbTab::tuneOsdChannel()
 	osdChannel.clear();
 	osdChannelTimer.stop();
 
-	DvbChannelModel *channelModel = manager->getChannelModel();
-	QModelIndex index = channelModel->findChannelByNumber(number);
+	DvbSharedChannel channel = manager->getChannelModel()->findChannelByNumber(number);
 
-	if (index.isValid()) {
-		const DvbChannel *channel = channelModel->data(index,
-			DvbChannelModel::DvbChannelRole).value<const DvbChannel *>();
-		playChannel(channel, channelProxyModel->mapFromSource(index));
+	if (channel.isValid()) {
+		playChannel(channel, channelProxyModel->find(channel));
 	}
 }
 
 void DvbTab::playChannel(const QModelIndex &index)
 {
 	if (index.isValid()) {
-		const DvbChannel *channel = channelProxyModel->data(index,
-			DvbChannelModel::DvbChannelRole).value<const DvbChannel *>();
-		playChannel(channel, index);
+		playChannel(channelProxyModel->value(index), index);
 	}
 }
 
@@ -406,7 +431,8 @@ void DvbTab::cleanTimeShiftFiles()
 	}
 
 	QDir dir(manager->getTimeShiftFolder());
-	QStringList entries = dir.entryList(QStringList("TimeShift-*.m2t"), QDir::Files, QDir::Name);
+	QStringList entries =
+		dir.entryList(QStringList("TimeShift-*.m2t"), QDir::Files, QDir::Name);
 
 	if (entries.count() < 2) {
 		return;
@@ -423,9 +449,9 @@ void DvbTab::activate()
 	mediaWidget->setFocus();
 }
 
-void DvbTab::playChannel(const DvbChannel *channel, const QModelIndex &index)
+void DvbTab::playChannel(const DvbSharedChannel &channel, const QModelIndex &index)
 {
-	if (channel == NULL) {
+	if (!channel.isValid()) {
 		kWarning() << "channel is invalid";
 		return;
 	}
