@@ -26,12 +26,12 @@
 #include "../tabbase.h"
 #include "dvbrecording.h"
 
-class QAbstractProxyModel;
+class QModelIndex;
 class QSplitter;
 class KAction;
 class KActionCollection;
 class KMenu;
-class DvbChannel;
+class DvbChannelTableModel;
 class DvbChannelView;
 class DvbTimeShiftCleaner;
 class MediaWidget;
@@ -49,6 +49,11 @@ public:
 	void toggleOsd();
 	void toggleInstantRecord();
 
+	DvbManager *getManager() const
+	{
+		return manager;
+	}
+
 	void enableDvbDump();
 
 public slots:
@@ -60,7 +65,7 @@ private slots:
 	void showEpgDialog();
 	void showRecordingDialog();
 	void instantRecord(bool checked);
-	void programRemoved(const DvbRecordingKey &recordingKey);
+	void recordingRemoved(const DvbSharedRecording &recording);
 	void configureDvb();
 	void tuneOsdChannel();
 	void playChannel(const QModelIndex &index);
@@ -70,14 +75,14 @@ private slots:
 
 private:
 	void activate();
-	void playChannel(const DvbChannel *channel, const QModelIndex &index);
+	void playChannel(const DvbSharedChannel &channel, const QModelIndex &index);
 
 	MediaWidget *mediaWidget;
 	DvbManager *manager;
 	KAction *instantRecordAction;
-	DvbRecordingKey instantRecordingKey;
+	DvbSharedRecording instantRecording;
 	QSplitter *splitter;
-	QAbstractProxyModel *channelProxyModel;
+	DvbChannelTableModel *channelProxyModel;
 	DvbChannelView *channelView;
 	QLayout *mediaLayout;
 	QString osdChannel;
