@@ -27,6 +27,7 @@
 #include <QStackedLayout>
 #include <KActionCollection>
 #include <KCmdLineOptions>
+#include <KComboBox>
 #include <KFileDialog>
 #include <KInputDialog>
 #include <kio/deletejob.h>
@@ -34,13 +35,31 @@
 #include <KMenuBar>
 #include <KRecentFilesAction>
 #include <KShortcutsDialog>
-#include <KSystemTrayIcon>
+#include <KStatusNotifierItem>
 #include <KTabBar>
 #include <KToolBar>
 #include "dvb/dvbtab.h"
 #include "playlist/playlisttab.h"
 #include "dbusobjects.h"
 
+class StackedLayout : public QStackedLayout
+{
+public:
+	explicit StackedLayout(QWidget *parent) : QStackedLayout(parent) { }
+	~StackedLayout() { }
+
+	QSize minimumSize() const
+	{
+		QWidget *widget = currentWidget();
+
+		if (widget != NULL) {
+			return widget->minimumSizeHint();
+		}
+
+		return QSize();
+	}
+};
+
 class StartTab : public TabBase
 {
 public:
@@ -63,7 +82,8 @@ StartTab::StartTab(MainWindow *mainWindow)
 	gridLayout->setMargin(10);
 	gridLayout->setSpacing(15);
 
-	QAbstractButton *button = addShortcut(i18n("&1 Play File"), KIcon("video-x-generic"), this);
+	QAbstractButton *button =
+		addShortcut(i18n("&1 Play File"), KIcon("video-x-generic"), this);
 	button->setShortcut(Qt::Key_1);
 	connect(button, SIGNAL(clicked()), mainWindow, SLOT(open()));
 	gridLayout->addWidget(button, 0, 0);
@@ -113,6 +133,7 @@ public:
 	void activate()
 	{
 		layout()->addWidget(mediaWidget);
+		mediaWidget->setFocus();
 	}
 
 private:
@@ -128,6 +149,16 @@ PlayerTab::PlayerTab(MediaWidget *mediaWidget_) : mediaWidget(mediaWidget_)
 class ConfigurationDialog : public KDialog
 {
 public:
+	enum StartupDisplayMode {
+		StartupNormalMode,
+		StartupMinimalMode,
+		StartupFullScreenMode,
+		StartupRememberLastSetting
+	};
+
+	static StartupDisplayMode getStartupDisplayMode();
+	static void setStartupDisplayMode(StartupDisplayMode startupDisplayMode);
+
 	ConfigurationDialog(MediaWidget *mediaWidget_, QWidget *parent);
 	~ConfigurationDialog();
 
@@ -135,10 +166,39 @@ public:
 
 private:
 	MediaWidget *mediaWidget;
+	KComboBox *startupDisplayModeBox;
 	QSpinBox *shortSkipBox;
 	QSpinBox *longSkipBox;
 };
 
+ConfigurationDialog::StartupDisplayMode ConfigurationDialog::getStartupDisplayMode()
+{
+	StartupDisplayMode startupDisplayMode = StartupNormalMode;
+
+	switch (KGlobal::config()->group("MainWindow").readEntry("StartupDisplayMode", 0)) {
+	case 0: startupDisplayMode = StartupNormalMode; break;
+	case 1: startupDisplayMode = StartupMinimalMode; break;
+	case 2: startupDisplayMode = StartupFullScreenMode; break;
+	case 3: startupDisplayMode = StartupRememberLastSetting; break;
+	}
+
+	return startupDisplayMode;
+}
+
+void ConfigurationDialog::setStartupDisplayMode(StartupDisplayMode startupDisplayMode)
+{
+	int value = 0;
+
+	switch (startupDisplayMode) {
+	case StartupNormalMode: value = 0; break;
+	case StartupMinimalMode: value = 1; break;
+	case StartupFullScreenMode: value = 2; break;
+	case StartupRememberLastSetting: value = 3; break;
+	}
+
+	KGlobal::config()->group("MainWindow").writeEntry("StartupDisplayMode", value);
+}
+
 ConfigurationDialog::ConfigurationDialog(MediaWidget *mediaWidget_, QWidget *parent) :
 	KDialog(parent), mediaWidget(mediaWidget_)
 {
@@ -147,21 +207,45 @@ ConfigurationDialog::ConfigurationDialog(MediaWidget *mediaWidget_, QWidget *par
 	QWidget *widget = new QWidget(this);
 	QGridLayout *gridLayout = new QGridLayout(widget);
 
+	gridLayout->addWidget(new QLabel(i18nc("@label:listbox", "Startup display mode:"), widget),
+		0, 0);
+
+	int index = 0;
+
+	switch (getStartupDisplayMode()) {
+	case StartupNormalMode: index = 0; break;
+	case StartupMinimalMode: index = 1; break;
+	case StartupFullScreenMode: index = 2; break;
+	case StartupRememberLastSetting: index = 3; break;
+	}
+
+	startupDisplayModeBox = new KComboBox(widget);
+	startupDisplayModeBox->addItem(i18nc("@item:inlistbox 'Startup display mode:'",
+		"Normal Mode"));
+	startupDisplayModeBox->addItem(i18nc("@item:inlistbox 'Startup display mode:'",
+		"Minimal Mode"));
+	startupDisplayModeBox->addItem(i18nc("@item:inlistbox 'Startup display mode:'",
+		"Full Screen Mode"));
+	startupDisplayModeBox->addItem(i18nc("@item:inlistbox 'Startup display mode:'",
+		"Remember Last Setting"));
+	startupDisplayModeBox->setCurrentIndex(index);
+	gridLayout->addWidget(startupDisplayModeBox, 0, 1);
+
 	gridLayout->addWidget(new QLabel(i18nc("'Configure Kaffeine' dialog",
-		"Short skip duration:"), widget), 0, 0);
+		"Short skip duration:"), widget), 1, 0);
 
 	shortSkipBox = new QSpinBox(widget);
 	shortSkipBox->setRange(1, 600);
 	shortSkipBox->setValue(mediaWidget->getShortSkipDuration());
-	gridLayout->addWidget(shortSkipBox, 0, 1);
+	gridLayout->addWidget(shortSkipBox, 1, 1);
 
 	gridLayout->addWidget(new QLabel(i18nc("'Configure Kaffeine' dialog",
-		"Long skip duration:"), widget), 1, 0);
+		"Long skip duration:"), widget), 2, 0);
 
 	longSkipBox = new QSpinBox(widget);
 	longSkipBox->setRange(1, 600);
 	longSkipBox->setValue(mediaWidget->getLongSkipDuration());
-	gridLayout->addWidget(longSkipBox, 1, 1);
+	gridLayout->addWidget(longSkipBox, 2, 1);
 
 	setMainWidget(widget);
 }
@@ -172,6 +256,16 @@ ConfigurationDialog::~ConfigurationDialog()
 
 void ConfigurationDialog::accept()
 {
+	StartupDisplayMode displayMode = StartupNormalMode;
+
+	switch (startupDisplayModeBox->currentIndex()) {
+	case 0: displayMode = StartupNormalMode; break;
+	case 1: displayMode = StartupMinimalMode; break;
+	case 2: displayMode = StartupFullScreenMode; break;
+	case 3: displayMode = StartupRememberLastSetting; break;
+	}
+
+	setStartupDisplayMode(displayMode);
 	mediaWidget->setShortSkipDuration(shortSkipBox->value());
 	mediaWidget->setLongSkipDuration(longSkipBox->value());
 
@@ -191,7 +285,8 @@ MainWindow::MainWindow()
 	KAction *action = KStandardAction::open(this, SLOT(open()), collection);
 	menu->addAction(collection->addAction("file_open", action));
 
-	action = new KAction(KIcon("uri-mms"), i18n("Open URL"), collection);
+	action = new KAction(KIcon("text-html"),
+		i18nc("@action:inmenu", "Open URL..."), collection);
 	action->setShortcut(Qt::CTRL | Qt::Key_U);
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(openUrl()));
 	menu->addAction(collection->addAction("file_open_url", action));
@@ -214,6 +309,11 @@ MainWindow::MainWindow()
 	connect(action, SIGNAL(triggered(bool)), this, SLOT(openDvd()));
 	menu->addAction(collection->addAction("file_play_dvd", action));
 
+	action = new KAction(KIcon("media-optical"), i18nc("@action:inmenu", "Play DVD Folder"),
+		collection);
+	connect(action, SIGNAL(triggered()), this, SLOT(playDvdFolder()));
+	menu->addAction(collection->addAction("file_play_dvd_folder", action));
+
 	menu->addSeparator();
 
 	action = KStandardAction::quit(this, SLOT(close()), collection);
@@ -236,8 +336,7 @@ MainWindow::MainWindow()
 	action = KStandardAction::keyBindings(this, SLOT(configureKeys()), collection);
 	menu->addAction(collection->addAction("settings_keys", action));
 
-	action = new KAction(KIcon("configure"), i18nc("dialog", "Configure Kaffeine"), this);
-	connect(action, SIGNAL(triggered(bool)), this, SLOT(configureKaffeine()));
+	action = KStandardAction::preferences(this, SLOT(configureKaffeine()), collection);
 	menu->addAction(collection->addAction("settings_kaffeine", action));
 
 	menuBar->addSeparator();
@@ -275,7 +374,7 @@ MainWindow::MainWindow()
 	// main area
 
 	QWidget *widget = new QWidget(this);
-	stackedLayout = new QStackedLayout(widget);
+	stackedLayout = new StackedLayout(widget);
 	setCentralWidget(widget);
 
 	mediaWidget = new MediaWidget(playerMenu, controlBar, collection, widget);
@@ -325,8 +424,11 @@ MainWindow::MainWindow()
 	// workaround setAutoSaveSettings() which doesn't accept "IconOnly" as initial state
 	controlBar->setToolButtonStyle(Qt::ToolButtonIconOnly);
 
-	KSystemTrayIcon *trayIcon = new KSystemTrayIcon(KIcon("kaffeine"), this);
-	trayIcon->show();
+	KStatusNotifierItem *trayIcon = new KStatusNotifierItem(this);
+	trayIcon->setIconByName("kaffeine");
+	trayIcon->setStatus(KStatusNotifierItem::Active);
+	trayIcon->setToolTipIconByName("kaffeine");
+	trayIcon->setToolTipTitle(i18n("Kaffeine"));
 
 	// initialize random number generator
 	qsrand(QTime().msecsTo(QTime::currentTime()));
@@ -346,12 +448,55 @@ MainWindow::MainWindow()
 	QDBusConnection::sessionBus().registerService("org.mpris.kaffeine");
 
 	show();
+
+	// set display mode
+	switch (ConfigurationDialog::getStartupDisplayMode()) {
+	case ConfigurationDialog::StartupNormalMode:
+		// nothing to do
+		break;
+	case ConfigurationDialog::StartupMinimalMode:
+		mediaWidget->setDisplayMode(MediaWidget::MinimalMode);
+		break;
+	case ConfigurationDialog::StartupFullScreenMode:
+		mediaWidget->setDisplayMode(MediaWidget::FullScreenMode);
+		break;
+	case ConfigurationDialog::StartupRememberLastSetting: {
+		int value = KGlobal::config()->group("MainWindow").readEntry("DisplayMode", 0);
+
+		switch (value) {
+		case 0:
+			// nothing to do
+			break;
+		case 1:
+			mediaWidget->setDisplayMode(MediaWidget::MinimalMode);
+			break;
+		case 2:
+			mediaWidget->setDisplayMode(MediaWidget::FullScreenMode);
+			break;
+		}
+
+		break;
+	    }
+	}
 }
 
 MainWindow::~MainWindow()
 {
 	actionOpenRecent->saveEntries(KGlobal::config()->group("Recent Files"));
-	KIO::del(temporaryUrls);
+
+	if (!temporaryUrls.isEmpty()) {
+		KIO::del(temporaryUrls);
+	}
+
+	int value = 0;
+
+	switch (mediaWidget->getDisplayMode()) {
+	case MediaWidget::NormalMode: value = 0; break;
+	case MediaWidget::MinimalMode: value = 1; break;
+	case MediaWidget::FullScreenMode: value = 2; break;
+	}
+
+	KGlobal::config()->group("MainWindow").writeEntry("DisplayMode", value);
 }
 
 KCmdLineOptions MainWindow::cmdLineOptions()
@@ -379,24 +524,33 @@ void MainWindow::parseArgs()
 	}
 
 	if (args->isSet("audiocd")) {
-		// FIXME device is ignored
-		openAudioCd();
+		if (args->count() > 0) {
+			openAudioCd(args->arg(0));
+		} else {
+			openAudioCd();
+		}
 
 		args->clear();
 		return;
 	}
 
 	if (args->isSet("videocd")) {
-		// FIXME device is ignored
-		openVideoCd();
+		if (args->count() > 0) {
+			openVideoCd(args->arg(0));
+		} else {
+			openVideoCd();
+		}
 
 		args->clear();
 		return;
 	}
 
 	if (args->isSet("dvd")) {
-		// FIXME device is ignored
-		openDvd();
+		if (args->count() > 0) {
+			openDvd(args->arg(0));
+		} else {
+			openDvd();
+		}
 
 		args->clear();
 		return;
@@ -510,7 +664,7 @@ void MainWindow::open()
 
 void MainWindow::openUrl()
 {
-	openUrl(KInputDialog::getText(i18n("Open URL"), i18n("Enter a URL:")));
+	openUrl(KInputDialog::getText(i18nc("@title:window", "Open URL"), i18n("Enter a URL:")));
 }
 
 void MainWindow::openUrl(const KUrl &url)
@@ -530,22 +684,31 @@ void MainWindow::openUrl(const KUrl &url)
 	playlistTab->appendToVisiblePlaylist(QList<KUrl>() << copy, true);
 }
 
-void MainWindow::openAudioCd()
-{
-	activateTab(PlayerTabId); // FIXME
-	mediaWidget->playAudioCd();
-}
-
-void MainWindow::openVideoCd()
+void MainWindow::openAudioCd(const QString &device)
 {
 	activateTab(PlayerTabId);
-	mediaWidget->playVideoCd();
+	mediaWidget->playAudioCd(device);
 }
 
-void MainWindow::openDvd()
+void MainWindow::openVideoCd(const QString &device)
 {
 	activateTab(PlayerTabId);
-	mediaWidget->playDvd();
+	mediaWidget->playVideoCd(device);
+}
+
+void MainWindow::openDvd(const QString &device)
+{
+	activateTab(PlayerTabId);
+	mediaWidget->playDvd(device);
+}
+
+void MainWindow::playDvdFolder()
+{
+	QString folder = KFileDialog::getExistingDirectory(KUrl(), this);
+
+	if (!folder.isEmpty()) {
+		openDvd(folder);
+	}
 }
 
 void MainWindow::playDvb()
@@ -556,7 +719,7 @@ void MainWindow::playDvb()
 
 void MainWindow::resizeToVideo(int factor)
 {
-	if (!isFullScreen()) {
+	if (!isFullScreen() && !mediaWidget->sizeHint().isEmpty()) {
 		if (isMaximized()) {
 			setWindowState(windowState() & ~Qt::WindowMaximized);
 		}
