@@ -24,7 +24,6 @@
 #include <QBoxLayout>
 #include <QContextMenuEvent>
 #include <QDBusInterface>
-#include <QDBusPendingReply>
 #include <QFile>
 #include <QLabel>
 #include <QPushButton>
@@ -44,8 +43,8 @@
 #include <KToolBar>
 #include <errno.h>
 #include <fcntl.h>
-#include <sys/stat.h>
-#include <sys/types.h>
+#include <sys/stat.h> // bsd compatibility
+#include <sys/types.h> // bsd compatibility
 #include <unistd.h>
 #include <X11/extensions/scrnsaver.h>
 #include "backend-xine/xinemediawidget.h"
@@ -79,7 +78,7 @@ DvbFeed::DvbFeed(QObject *parent) : QObject(parent), timeShiftPrepared(false),
 		readFd = -1;
 	}
 
-	notifier = new QSocketNotifier(writeFd, QSocketNotifier::Write);
+	notifier = new QSocketNotifier(writeFd, QSocketNotifier::Write, this);
 	notifier->setEnabled(false);
 	connect(notifier, SIGNAL(activated(int)), this, SLOT(readyWrite()));
 }
@@ -154,7 +153,7 @@ bool DvbFeed::flush()
 JumpToPositionDialog::JumpToPositionDialog(MediaWidget *mediaWidget_) : KDialog(mediaWidget_),
 	mediaWidget(mediaWidget_)
 {
-	setCaption(i18n("Jump to Position"));
+	setCaption(i18nc("@title:window", "Jump to Position"));
 
 	QWidget *widget = new QWidget(this);
 	QBoxLayout *layout = new QVBoxLayout(widget);
@@ -191,8 +190,7 @@ void SeekSlider::mousePressEvent(QMouseEvent *event)
 }
 
 MediaWidget::MediaWidget(KMenu *menu_, KToolBar *toolBar, KActionCollection *collection,
-	QWidget *parent) : QWidget(parent), menu(menu_), dvbFeed(NULL), screenSaverDBusCall(NULL),
-	screenSaverSuspended(false)
+	QWidget *parent) : QWidget(parent), menu(menu_), dvbFeed(NULL), screenSaverSuspended(false)
 {
 	QBoxLayout *layout = new QVBoxLayout(this);
 	layout->setMargin(0);
@@ -403,7 +401,8 @@ MediaWidget::MediaWidget(KMenu *menu_, KToolBar *toolBar, KActionCollection *col
 	action->setDefaultWidget(volumeSlider);
 	toolBar->addAction(collection->addAction("controls_volume_slider", action));
 
-	jumpToPositionAction = new KAction(KIcon("go-jump"), i18n("Jump to Position"), this);
+	jumpToPositionAction = new KAction(KIcon("go-jump"),
+		i18nc("@action:inmenu", "Jump to Position..."), this);
 	jumpToPositionAction->setShortcut(Qt::CTRL + Qt::Key_J);
 	connect(jumpToPositionAction, SIGNAL(triggered()), this, SLOT(jumpToPosition()));
 	menu->addAction(collection->addAction("controls_jump_to_position", jumpToPositionAction));
@@ -520,13 +519,13 @@ QString MediaWidget::extensionFilter()
 		// generated from kaffeine.desktop's mime types
 		"*.669 *.aac *.ac3 *.aif *.aifc *.aiff *.anim1 *.anim2 *.anim3 *.anim4 *.anim5 "
 		"*.anim6 *.anim7 *.anim8 *.anim9 *.animj *.asf *.asx *.au *.avi *.bdm *.bdmv "
-		"*.clpi *.cpi *.divx *.dv *.flac *.flc *.fli *.flv *.it *.m15 *.m2t *.m2ts *.m3u "
-		"*.m3u8 *.m4a *.m4b *.m4v *.med *.mka *.mkv *.mng *.mod *.moov *.mov *.mp+ *.mp2 "
-		"*.mp3 *.mp4 *.mpc *.mpe *.mpeg *.mpg *.mpga *.mpl *.mpls *.mpp *.mtm *.mts *.nsv "
-		"*.oga *.ogg *.ogm *.ogv *.ogx *.pls *.qt *.qtl *.qtvr *.ra *.ram *.rax *.rm "
-		"*.rmj *.rmm *.rms *.rmvb *.rmx *.rv *.rvx *.s3m *.shn *.snd *.spx *.stm *.ts "
-		"*.tta *.ult *.uni *.vlc *.vob *.voc *.wav *.wax *.wma *.wmv *.wmx *.wv *.wvp "
-		"*.wvx *.xm *.xspf "
+		"*.clpi *.cpi *.divx *.dv *.f4a *.f4b *.f4v *.flac *.flc *.fli *.flv *.it *.m15 "
+		"*.m2t *.m2ts *.m3u *.m3u8 *.m4a *.m4b *.m4v *.med *.mka *.mkv *.mng *.mod *.moov "
+		"*.mov *.mp+ *.mp2 *.mp3 *.mp4 *.mpc *.mpe *.mpeg *.mpg *.mpga *.mpl *.mpls *.mpp "
+		"*.mtm *.mts *.nsv *.oga *.ogg *.ogm *.ogv *.ogx *.pls *.qt *.qtl *.qtvr *.ra "
+		"*.ram *.rax *.rm *.rmj *.rmm *.rms *.rmvb *.rmx *.rv *.rvx *.s3m *.shn *.snd "
+		"*.spx *.stm *.ts *.tta *.ult *.uni *.vlc *.vob *.voc *.wav *.wax *.wma *.wmv "
+		"*.wmx *.wv *.wvp *.wvx *.xm *.xspf "
 		// manual entries
 		"*.kaffeine *.iso|") + i18nc("file filter", "Supported Media Files") +
 		"\n*|" + i18nc("file filter", "All Files");
@@ -560,7 +559,8 @@ void MediaWidget::setDisplayMode(DisplayMode displayMode_)
 			break;
 		case FullScreenMode:
 			fullScreenAction->setIcon(KIcon("view-restore"));
-			fullScreenAction->setText(i18nc("'Playback' menu", "Exit Full Screen Mode"));
+			fullScreenAction->setText(i18nc("'Playback' menu",
+				"Exit Full Screen Mode"));
 			break;
 		case MinimalMode:
 			minimalModeAction->setIcon(KIcon("view-fullscreen"));
@@ -588,17 +588,20 @@ void MediaWidget::play(const KUrl &url, const KUrl &subtitleUrl)
 	backend->playUrl(url, subtitleUrl);
 }
 
-void MediaWidget::playAudioCd()
+void MediaWidget::playAudioCd(const QString &device)
 {
-	QList<Solid::Device> devices =
-		Solid::Device::listFromQuery("OpticalDisc.availableContent & 'Audio'");
-	QString deviceName;
+	QString deviceName = device;
 
-	if (!devices.isEmpty()) {
-		Solid::Block *block = devices.first().as<Solid::Block>();
+	if (deviceName.isEmpty()) {
+		QList<Solid::Device> devices =
+			Solid::Device::listFromQuery("OpticalDisc.availableContent & 'Audio'");
 
-		if (block != NULL) {
-			deviceName = block->device();
+		if (!devices.isEmpty()) {
+			Solid::Block *block = devices.first().as<Solid::Block>();
+
+			if (block != NULL) {
+				deviceName = block->device();
+			}
 		}
 	}
 
@@ -611,17 +614,20 @@ void MediaWidget::playAudioCd()
 	backend->playAudioCd(deviceName);
 }
 
-void MediaWidget::playVideoCd()
+void MediaWidget::playVideoCd(const QString &device)
 {
-	QList<Solid::Device> devices = Solid::Device::listFromQuery(
-		"OpticalDisc.availableContent & 'VideoCd|SuperVideoCd'");
-	QString deviceName;
+	QString deviceName = device;
 
-	if (!devices.isEmpty()) {
-		Solid::Block *block = devices.first().as<Solid::Block>();
+	if (deviceName.isEmpty()) {
+		QList<Solid::Device> devices = Solid::Device::listFromQuery(
+			"OpticalDisc.availableContent & 'VideoCd|SuperVideoCd'");
 
-		if (block != NULL) {
-			deviceName = block->device();
+		if (!devices.isEmpty()) {
+			Solid::Block *block = devices.first().as<Solid::Block>();
+
+			if (block != NULL) {
+				deviceName = block->device();
+			}
 		}
 	}
 
@@ -634,17 +640,20 @@ void MediaWidget::playVideoCd()
 	backend->playVideoCd(deviceName);
 }
 
-void MediaWidget::playDvd()
+void MediaWidget::playDvd(const QString &device)
 {
-	QList<Solid::Device> devices =
-		Solid::Device::listFromQuery("OpticalDisc.availableContent & 'VideoDvd'");
-	QString deviceName;
+	QString deviceName = device;
 
-	if (!devices.isEmpty()) {
-		Solid::Block *block = devices.first().as<Solid::Block>();
+	if (deviceName.isEmpty()) {
+		QList<Solid::Device> devices =
+			Solid::Device::listFromQuery("OpticalDisc.availableContent & 'VideoDvd'");
 
-		if (block != NULL) {
-			deviceName = block->device();
+		if (!devices.isEmpty()) {
+			Solid::Block *block = devices.first().as<Solid::Block>();
+
+			if (block != NULL) {
+				deviceName = block->device();
+			}
 		}
 	}
 
@@ -694,10 +703,8 @@ OsdWidget *MediaWidget::getOsdWidget()
 
 void MediaWidget::playDvb(const QString &channelName)
 {
-	if (dvbFeed != NULL) {
-		delete dvbFeed;
-		dvbFeed = NULL;
-	}
+	delete dvbFeed;
+	dvbFeed = NULL;
 
 	seekableChanged(false);
 	totalTimeChanged(0);
@@ -1170,26 +1177,19 @@ void MediaWidget::checkScreenSaver()
 {
 	bool suspendScreenSaver = (backend->isPlaying() && !isPaused() && isVisible());
 
-	if (suspendScreenSaver && (screenSaverDBusCall == NULL)) {
-		screenSaverDBusCall = new QDBusPendingCall(
-			QDBusInterface("org.freedesktop.ScreenSaver", "/ScreenSaver",
-			"org.freedesktop.ScreenSaver").asyncCall("Inhibit", "Kaffeine", ""));
-	} else if (!suspendScreenSaver && (screenSaverDBusCall != NULL)) {
-		QDBusPendingReply<unsigned int> reply(*screenSaverDBusCall);
+	if (suspendScreenSaver) {
+		// KDE - Inhibit doesn't inhibit "lock screen after inactivity"
+		QDBusInterface("org.freedesktop.ScreenSaver", "/ScreenSaver",
+			"org.freedesktop.ScreenSaver").call(QDBus::NoBlock,
+			"SimulateUserActivity");
 
-		if (reply.isValid()) {
-			QDBusInterface("org.freedesktop.ScreenSaver", "/ScreenSaver",
-				"org.freedesktop.ScreenSaver").call(
-				QDBus::NoBlock, "UnInhibit", reply.value());
-		}
-
-		if (reply.isFinished()) {
-			delete screenSaverDBusCall;
-			screenSaverDBusCall = NULL;
-		}
+		// GNOME - Inhibit doesn't inhibit power management functions
+		QDBusInterface("org.gnome.ScreenSaver", "/", "org.gnome.ScreenSaver").
+			call(QDBus::NoBlock, "SimulateUserActivity");
 	}
 
 	if (screenSaverSuspended != suspendScreenSaver) {
+		// X11 - needed if none of the above applications is running
 		screenSaverSuspended = suspendScreenSaver;
 		XScreenSaverSuspend(QX11Info::display(), suspendScreenSaver);
 	}
