@@ -26,7 +26,6 @@
 #include <KUrl>
 
 class QActionGroup;
-class QDBusPendingCall;
 class QPushButton;
 class QSlider;
 class KAction;
@@ -77,9 +76,9 @@ public:
 	 */
 
 	void play(const KUrl &url, const KUrl &subtitleUrl = KUrl());
-	void playAudioCd();
-	void playVideoCd();
-	void playDvd();
+	void playAudioCd(const QString &device);
+	void playVideoCd(const QString &device);
+	void playDvd(const QString &device);
 	void updateExternalSubtitles(const QList<KUrl> &subtitles, int currentSubtitle);
 
 	OsdWidget *getOsdWidget();
@@ -241,7 +240,6 @@ private:
 	KAction *jumpToPositionAction;
 	QPushButton *timeButton;
 	bool showElapsedTime;
-	QDBusPendingCall *screenSaverDBusCall;
 	bool screenSaverSuspended;
 };
 
