@@ -20,13 +20,12 @@
 
 #include "sqlhelper.h"
 
-#include <QCoreApplication>
 #include <QSqlError>
 #include <KDebug>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KStandardDirs>
-#include "sqltablemodel.h"
+#include "sqlinterface.h"
 
 SqlHelper::SqlHelper()
 {
@@ -68,7 +67,6 @@ bool SqlHelper::createInstance()
 		return false;
 	}
 
-	qAddPostRoutine(deleteInstance);
 	return true;
 }
 
@@ -108,7 +106,7 @@ void SqlHelper::exec(QSqlQuery &query)
 	}
 }
 
-void SqlHelper::requestSubmission(SqlTableModelInterface *object)
+void SqlHelper::requestSubmission(SqlInterface *object)
 {
 	if (!timer.isActive()) {
 		timer.start();
@@ -122,7 +120,7 @@ void SqlHelper::collectSubmissions()
 	exec("BEGIN");
 
 	for (int i = 0; i < objects.size(); ++i) {
-		objects.at(i)->submit();
+		objects.at(i)->sqlSubmit();
 	}
 
 	exec("COMMIT");
@@ -130,10 +128,4 @@ void SqlHelper::collectSubmissions()
 	objects.clear();
 }
 
-void SqlHelper::deleteInstance()
-{
-	delete instance;
-	instance = NULL;
-}
-
 SqlHelper *SqlHelper::instance = NULL;
