@@ -3,7 +3,7 @@ set -eu
 
 mkdir -p kaffeine_build
 cd kaffeine_build
-cmake ../kaffeine -DCMAKE_INSTALL_PREFIX=/usr -DBUILD_TOOLS=1
+cmake ../kaffeine -DBUILD_TOOLS=1
 make updatedvbsi
 cd ../kaffeine/tools
 ../../kaffeine_build/tools/updatedvbsi
